/*     */ package com.google.code.yanf4j.core.impl;
/*     */ 
/*     */ import com.google.code.yanf4j.buffer.IoBuffer;
/*     */ import com.google.code.yanf4j.core.CodecFactory;
/*     */ import com.google.code.yanf4j.core.CodecFactory.Decoder;
/*     */ import com.google.code.yanf4j.core.CodecFactory.Encoder;
/*     */ import com.google.code.yanf4j.core.Dispatcher;
/*     */ import com.google.code.yanf4j.core.Handler;
/*     */ import com.google.code.yanf4j.core.Session;
/*     */ import com.google.code.yanf4j.core.SessionConfig;
/*     */ import com.google.code.yanf4j.core.WriteMessage;
/*     */ import com.google.code.yanf4j.statistics.Statistics;
/*     */ import java.io.IOException;
/*     */ import java.nio.ByteOrder;
/*     */ import java.nio.channels.FileChannel;
/*     */ import java.util.Queue;
/*     */ import java.util.concurrent.ConcurrentHashMap;
/*     */ import java.util.concurrent.ExecutionException;
/*     */ import java.util.concurrent.Future;
/*     */ import java.util.concurrent.TimeUnit;
/*     */ import java.util.concurrent.TimeoutException;
/*     */ import java.util.concurrent.atomic.AtomicLong;
/*     */ import java.util.concurrent.atomic.AtomicReference;
/*     */ import java.util.concurrent.locks.ReentrantLock;
/*     */ import org.slf4j.Logger;
/*     */ import org.slf4j.LoggerFactory;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class AbstractSession
/*     */   implements Session
/*     */ {
/*     */   protected IoBuffer readBuffer;
/*  59 */   protected static final Logger log = LoggerFactory.getLogger(AbstractSession.class);
/*     */   
/*     */ 
/*  62 */   protected final ConcurrentHashMap<String, Object> attributes = new ConcurrentHashMap();
/*     */   
/*     */   protected Queue<WriteMessage> writeQueue;
/*     */   protected volatile long sessionIdleTimeout;
/*     */   protected volatile long sessionTimeout;
/*     */   protected CodecFactory.Encoder encoder;
/*     */   
/*     */   public long getSessionIdleTimeout()
/*     */   {
/*  71 */     return this.sessionIdleTimeout;
/*     */   }
/*     */   
/*     */   public void setSessionIdleTimeout(long sessionIdleTimeout) {
/*  75 */     this.sessionIdleTimeout = sessionIdleTimeout;
/*     */   }
/*     */   
/*     */   public long getSessionTimeout() {
/*  79 */     return this.sessionTimeout;
/*     */   }
/*     */   
/*     */   public void setSessionTimeout(long sessionTimeout) {
/*  83 */     this.sessionTimeout = sessionTimeout;
/*     */   }
/*     */   
/*     */   public Queue<WriteMessage> getWriteQueue() {
/*  87 */     return this.writeQueue;
/*     */   }
/*     */   
/*     */   public Statistics getStatistics() {
/*  91 */     return this.statistics;
/*     */   }
/*     */   
/*     */   public Handler getHandler() {
/*  95 */     return this.handler;
/*     */   }
/*     */   
/*     */   public Dispatcher getDispatchMessageDispatcher() {
/*  99 */     return this.dispatchMessageDispatcher;
/*     */   }
/*     */   
/*     */   public ReentrantLock getWriteLock() {
/* 103 */     return this.writeLock;
/*     */   }
/*     */   
/*     */ 
/*     */   protected CodecFactory.Decoder decoder;
/*     */   
/*     */   protected volatile boolean closed;
/*     */   
/*     */   protected Statistics statistics;
/*     */   
/*     */   protected Handler handler;
/*     */   
/*     */   protected boolean loopback;
/*     */   
/* 117 */   public AtomicLong lastOperationTimeStamp = new AtomicLong(0L);
/*     */   
/* 119 */   protected AtomicLong scheduleWritenBytes = new AtomicLong(0L);
/*     */   
/*     */   protected final Dispatcher dispatchMessageDispatcher;
/* 122 */   protected volatile boolean useBlockingWrite = false;
/* 123 */   protected volatile boolean useBlockingRead = true;
/* 124 */   protected volatile boolean handleReadWriteConcurrently = true;
/*     */   
/*     */   public abstract void decode();
/*     */   
/*     */   public void updateTimeStamp() {
/* 129 */     this.lastOperationTimeStamp.set(System.currentTimeMillis());
/*     */   }
/*     */   
/*     */   public long getLastOperationTimeStamp() {
/* 133 */     return this.lastOperationTimeStamp.get();
/*     */   }
/*     */   
/*     */   public final boolean isHandleReadWriteConcurrently() {
/* 137 */     return this.handleReadWriteConcurrently;
/*     */   }
/*     */   
/*     */   public final void setHandleReadWriteConcurrently(boolean handleReadWriteConcurrently)
/*     */   {
/* 142 */     this.handleReadWriteConcurrently = handleReadWriteConcurrently;
/*     */   }
/*     */   
/*     */   public long getScheduleWritenBytes() {
/* 146 */     return this.scheduleWritenBytes.get();
/*     */   }
/*     */   
/*     */   public CodecFactory.Encoder getEncoder() {
/* 150 */     return this.encoder;
/*     */   }
/*     */   
/*     */   public void setEncoder(CodecFactory.Encoder encoder) {
/* 154 */     this.encoder = encoder;
/*     */   }
/*     */   
/*     */   public CodecFactory.Decoder getDecoder() {
/* 158 */     return this.decoder;
/*     */   }
/*     */   
/*     */   public IoBuffer getReadBuffer() {
/* 162 */     return this.readBuffer;
/*     */   }
/*     */   
/*     */   public void setReadBuffer(IoBuffer readBuffer) {
/* 166 */     this.readBuffer = readBuffer;
/*     */   }
/*     */   
/*     */   public void setDecoder(CodecFactory.Decoder decoder) {
/* 170 */     this.decoder = decoder;
/*     */   }
/*     */   
/*     */   public final ByteOrder getReadBufferByteOrder() {
/* 174 */     if (this.readBuffer == null) {
/* 175 */       throw new IllegalStateException();
/*     */     }
/* 177 */     return this.readBuffer.order();
/*     */   }
/*     */   
/*     */   public final void setReadBufferByteOrder(ByteOrder readBufferByteOrder) {
/* 181 */     if (this.readBuffer == null) {
/* 182 */       throw new NullPointerException("Null ReadBuffer");
/*     */     }
/* 184 */     this.readBuffer.order(readBufferByteOrder);
/*     */   }
/*     */   
/*     */   protected synchronized void onIdle()
/*     */   {
/*     */     try
/*     */     {
/* 191 */       if (isIdle()) {
/* 192 */         updateTimeStamp();
/* 193 */         this.handler.onSessionIdle(this);
/*     */       }
/*     */     } catch (Throwable e) {
/* 196 */       onException(e);
/*     */     }
/*     */   }
/*     */   
/*     */   protected void onConnected() {
/*     */     try {
/* 202 */       this.handler.onSessionConnected(this);
/*     */     } catch (Throwable e) {
/* 204 */       onException(e);
/*     */     }
/*     */   }
/*     */   
/*     */   public void onExpired() {
/*     */     try {
/* 210 */       if (isExpired()) {
/* 211 */         this.handler.onSessionExpired(this);
/*     */       }
/*     */     } catch (Throwable e) {
/* 214 */       onException(e);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected abstract WriteMessage wrapMessage(Object paramObject, Future<Boolean> paramFuture);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected WriteMessage preprocessWriteMessage(WriteMessage writeMessage)
/*     */   {
/* 228 */     return writeMessage;
/*     */   }
/*     */   
/*     */   protected void dispatchReceivedMessage(final Object message) {
/* 232 */     if (this.dispatchMessageDispatcher == null) {
/* 233 */       long start = -1L;
/* 234 */       if ((this.statistics != null) && (this.statistics.isStatistics())) {
/* 235 */         start = System.currentTimeMillis();
/*     */       }
/* 237 */       onMessage(message, this);
/* 238 */       if (start != -1L) {
/* 239 */         this.statistics.statisticsProcess(System.currentTimeMillis() - start);
/*     */       }
/*     */     }
/*     */     else
/*     */     {
/* 244 */       this.dispatchMessageDispatcher.dispatch(new Runnable() {
/*     */         public void run() {
/* 246 */           long start = -1L;
/* 247 */           if ((AbstractSession.this.statistics != null) && (AbstractSession.this.statistics.isStatistics())) {
/* 248 */             start = System.currentTimeMillis();
/*     */           }
/* 250 */           AbstractSession.this.onMessage(message, AbstractSession.this);
/* 251 */           if (start != -1L) {
/* 252 */             AbstractSession.this.statistics.statisticsProcess(System.currentTimeMillis() - start);
/*     */           }
/*     */         }
/*     */       });
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   private void onMessage(Object message, Session session)
/*     */   {
/*     */     try
/*     */     {
/* 264 */       this.handler.onMessageReceived(session, message);
/*     */     } catch (Throwable e) {
/* 266 */       onException(e);
/*     */     }
/*     */   }
/*     */   
/*     */   public final boolean isClosed() {
/* 271 */     return this.closed;
/*     */   }
/*     */   
/*     */   public final void setClosed(boolean closed) {
/* 275 */     this.closed = closed;
/*     */   }
/*     */   
/*     */   public void close() {
/* 279 */     synchronized (this) {
/* 280 */       if (isClosed()) {
/* 281 */         return;
/*     */       }
/* 283 */       setClosed(true);
/*     */     }
/*     */     try {
/* 286 */       closeChannel();
/* 287 */       clearAttributes();
/* 288 */       log.debug("session closed");
/*     */     } catch (IOException e) {
/* 290 */       onException(e);
/* 291 */       log.error("Close session error", e);
/*     */     } finally {
/* 293 */       onClosed();
/*     */     }
/*     */   }
/*     */   
/*     */   protected abstract void closeChannel() throws IOException;
/*     */   
/*     */   public void onException(Throwable e) {
/* 300 */     this.handler.onExceptionCaught(this, e);
/*     */   }
/*     */   
/*     */   protected void onClosed() {
/*     */     try {
/* 305 */       this.handler.onSessionClosed(this);
/*     */     } catch (Throwable e) {
/* 307 */       onException(e);
/*     */     }
/*     */   }
/*     */   
/*     */   public void setAttribute(String key, Object value) {
/* 312 */     this.attributes.put(key, value);
/*     */   }
/*     */   
/*     */   public Object setAttributeIfAbsent(String key, Object value) {
/* 316 */     return this.attributes.putIfAbsent(key, value);
/*     */   }
/*     */   
/*     */   public void removeAttribute(String key) {
/* 320 */     this.attributes.remove(key);
/*     */   }
/*     */   
/*     */   public Object getAttribute(String key) {
/* 324 */     return this.attributes.get(key);
/*     */   }
/*     */   
/*     */   public void clearAttributes() {
/* 328 */     this.attributes.clear();
/*     */   }
/*     */   
/*     */   public synchronized void start() {
/* 332 */     log.debug("session started");
/* 333 */     onStarted();
/* 334 */     start0();
/*     */   }
/*     */   
/*     */   protected abstract void start0();
/*     */   
/*     */   protected void onStarted() {
/*     */     try {
/* 341 */       this.handler.onSessionStarted(this);
/*     */     } catch (Throwable e) {
/* 343 */       onException(e);
/*     */     }
/*     */   }
/*     */   
/* 347 */   protected ReentrantLock writeLock = new ReentrantLock();
/*     */   
/* 349 */   protected AtomicReference<WriteMessage> currentMessage = new AtomicReference();
/*     */   
/*     */   static final class FailFuture implements Future<Boolean>
/*     */   {
/*     */     public boolean cancel(boolean mayInterruptIfRunning) {
/* 354 */       return Boolean.FALSE.booleanValue();
/*     */     }
/*     */     
/*     */     public Boolean get() throws InterruptedException, ExecutionException {
/* 358 */       return Boolean.FALSE;
/*     */     }
/*     */     
/*     */     public Boolean get(long timeout, TimeUnit unit)
/*     */       throws InterruptedException, ExecutionException, TimeoutException
/*     */     {
/* 364 */       return Boolean.FALSE;
/*     */     }
/*     */     
/*     */     public boolean isCancelled() {
/* 368 */       return false;
/*     */     }
/*     */     
/*     */     public boolean isDone() {
/* 372 */       return true;
/*     */     }
/*     */   }
/*     */   
/*     */   public void write(Object packet)
/*     */   {
/* 378 */     if (this.closed) {
/* 379 */       return;
/*     */     }
/* 381 */     WriteMessage message = wrapMessage(packet, null);
/* 382 */     this.scheduleWritenBytes.addAndGet(message.getWriteBuffer().remaining());
/* 383 */     writeFromUserCode(message);
/*     */   }
/*     */   
/*     */   public abstract void writeFromUserCode(WriteMessage paramWriteMessage);
/*     */   
/*     */   public final boolean isLoopbackConnection()
/*     */   {
/* 390 */     return this.loopback;
/*     */   }
/*     */   
/*     */   public boolean isUseBlockingWrite() {
/* 394 */     return this.useBlockingWrite;
/*     */   }
/*     */   
/*     */   public void setUseBlockingWrite(boolean useBlockingWrite) {
/* 398 */     this.useBlockingWrite = useBlockingWrite;
/*     */   }
/*     */   
/*     */   public boolean isUseBlockingRead() {
/* 402 */     return this.useBlockingRead;
/*     */   }
/*     */   
/*     */   public void setUseBlockingRead(boolean useBlockingRead) {
/* 406 */     this.useBlockingRead = useBlockingRead;
/*     */   }
/*     */   
/*     */   public void clearWriteQueue() {
/* 410 */     this.writeQueue.clear();
/*     */   }
/*     */   
/*     */   public boolean isExpired() {
/* 414 */     return false;
/*     */   }
/*     */   
/*     */   public boolean isIdle() {
/* 418 */     long lastOpTimestamp = getLastOperationTimeStamp();
/* 419 */     return (lastOpTimestamp > 0L) && (System.currentTimeMillis() - lastOpTimestamp > this.sessionIdleTimeout);
/*     */   }
/*     */   
/*     */ 
/*     */   public AbstractSession(SessionConfig sessionConfig)
/*     */   {
/* 425 */     this.lastOperationTimeStamp.set(System.currentTimeMillis());
/* 426 */     this.statistics = sessionConfig.statistics;
/* 427 */     this.handler = sessionConfig.handler;
/* 428 */     this.writeQueue = sessionConfig.queue;
/* 429 */     this.encoder = sessionConfig.codecFactory.getEncoder();
/* 430 */     this.decoder = sessionConfig.codecFactory.getDecoder();
/* 431 */     this.dispatchMessageDispatcher = sessionConfig.dispatchMessageDispatcher;
/* 432 */     this.handleReadWriteConcurrently = sessionConfig.handleReadWriteConcurrently;
/* 433 */     this.sessionTimeout = sessionConfig.sessionTimeout;
/* 434 */     this.sessionIdleTimeout = sessionConfig.sessionIdelTimeout;
/*     */   }
/*     */   
/*     */   public long transferTo(long position, long count, FileChannel target) throws IOException
/*     */   {
/* 439 */     throw new UnsupportedOperationException();
/*     */   }
/*     */   
/*     */   public long transferFrom(long position, long count, FileChannel source) throws IOException
/*     */   {
/* 444 */     throw new UnsupportedOperationException();
/*     */   }
/*     */   
/*     */   protected void onCreated() {
/*     */     try {
/* 449 */       this.handler.onSessionCreated(this);
/*     */     } catch (Throwable e) {
/* 451 */       onException(e);
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\google\code\yanf4j\core\impl\AbstractSession.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */