/*     */ package com.google.code.yanf4j.nio;
/*     */ 
/*     */ import com.google.code.yanf4j.config.Configuration;
/*     */ import com.google.code.yanf4j.core.CodecFactory;
/*     */ import com.google.code.yanf4j.core.EventType;
/*     */ import com.google.code.yanf4j.core.Handler;
/*     */ import com.google.code.yanf4j.core.Session;
/*     */ import com.google.code.yanf4j.core.SocketOption;
/*     */ import com.google.code.yanf4j.core.impl.StandardSocketOption;
/*     */ import com.google.code.yanf4j.nio.impl.SelectorManager;
/*     */ import com.google.code.yanf4j.nio.impl.SocketChannelController;
/*     */ import java.io.IOException;
/*     */ import java.net.InetSocketAddress;
/*     */ import java.net.ServerSocket;
/*     */ import java.net.Socket;
/*     */ import java.net.SocketException;
/*     */ import java.nio.channels.SelectionKey;
/*     */ import java.nio.channels.Selector;
/*     */ import java.nio.channels.ServerSocketChannel;
/*     */ import java.nio.channels.SocketChannel;
/*     */ import java.util.Map;
/*     */ import org.slf4j.Logger;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class TCPController
/*     */   extends SocketChannelController
/*     */ {
/*     */   private ServerSocketChannel serverSocketChannel;
/*  44 */   private int backlog = 500;
/*     */   private int connectionTime;
/*     */   
/*  47 */   public int getBacklog() { return this.backlog; }
/*     */   
/*     */   public void setBacklog(int backlog)
/*     */   {
/*  51 */     if (isStarted()) {
/*  52 */       throw new IllegalStateException();
/*     */     }
/*  54 */     if (backlog < 0) {
/*  55 */       throw new IllegalArgumentException("backlog<0");
/*     */     }
/*  57 */     this.backlog = backlog;
/*     */   }
/*     */   
/*     */ 
/*     */   public TCPController() {}
/*     */   
/*     */   public TCPController(Configuration configuration)
/*     */   {
/*  65 */     super(configuration, null, null);
/*     */   }
/*     */   
/*     */   public TCPController(Configuration configuration, CodecFactory codecFactory)
/*     */   {
/*  70 */     super(configuration, null, codecFactory);
/*     */   }
/*     */   
/*     */   public TCPController(Configuration configuration, Handler handler, CodecFactory codecFactory)
/*     */   {
/*  75 */     super(configuration, handler, codecFactory);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public void setPerformancePreferences(int connectionTime, int latency, int bandwidth)
/*     */   {
/*  82 */     this.connectionTime = connectionTime;
/*  83 */     this.latency = latency;
/*  84 */     this.bandwidth = bandwidth;
/*     */   }
/*     */   
/*     */   protected void doStart() throws IOException
/*     */   {
/*  89 */     this.serverSocketChannel = ServerSocketChannel.open();
/*  90 */     this.serverSocketChannel.socket().setSoTimeout(this.soTimeout);
/*  91 */     if ((this.connectionTime != 0) || (this.latency != 0) || (this.bandwidth != 0))
/*     */     {
/*  93 */       this.serverSocketChannel.socket().setPerformancePreferences(this.connectionTime, this.latency, this.bandwidth);
/*     */     }
/*     */     
/*  96 */     this.serverSocketChannel.configureBlocking(false);
/*     */     
/*  98 */     if (this.socketOptions.get(StandardSocketOption.SO_REUSEADDR) != null) {
/*  99 */       this.serverSocketChannel.socket().setReuseAddress(((Boolean)StandardSocketOption.SO_REUSEADDR.type().cast(this.socketOptions.get(StandardSocketOption.SO_REUSEADDR))).booleanValue());
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 104 */     if (this.socketOptions.get(StandardSocketOption.SO_RCVBUF) != null) {
/* 105 */       this.serverSocketChannel.socket().setReceiveBufferSize(((Integer)StandardSocketOption.SO_RCVBUF.type().cast(this.socketOptions.get(StandardSocketOption.SO_RCVBUF))).intValue());
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 111 */     if (this.localSocketAddress != null) {
/* 112 */       this.serverSocketChannel.socket().bind(this.localSocketAddress, this.backlog);
/*     */     }
/*     */     else {
/* 115 */       this.serverSocketChannel.socket().bind(new InetSocketAddress("localhost", 0), this.backlog);
/*     */     }
/*     */     
/* 118 */     setLocalSocketAddress((InetSocketAddress)this.serverSocketChannel.socket().getLocalSocketAddress());
/*     */     
/* 120 */     this.selectorManager.registerChannel(this.serverSocketChannel, 16, null);
/*     */   }
/*     */   
/*     */ 
/*     */   public void onAccept(SelectionKey selectionKey)
/*     */     throws IOException
/*     */   {
/* 127 */     if (!this.serverSocketChannel.isOpen()) {
/* 128 */       selectionKey.cancel();
/* 129 */       return;
/*     */     }
/* 131 */     SocketChannel sc = null;
/*     */     try {
/* 133 */       sc = this.serverSocketChannel.accept();
/* 134 */       if (sc != null) {
/* 135 */         configureSocketChannel(sc);
/* 136 */         Session session = buildSession(sc);
/*     */         
/* 138 */         this.selectorManager.registerSession(session, EventType.ENABLE_READ);
/*     */         
/* 140 */         session.start();
/* 141 */         super.onAccept(selectionKey);
/*     */       } else {
/* 143 */         log.debug("Accept fail");
/*     */       }
/*     */     } catch (IOException e) {
/* 146 */       closeAcceptChannel(selectionKey, sc);
/* 147 */       log.error("Accept connection error", e);
/* 148 */       notifyException(e);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   private int latency;
/*     */   
/*     */   private int bandwidth;
/*     */   
/*     */   private void closeAcceptChannel(SelectionKey sk, SocketChannel sc)
/*     */     throws IOException, SocketException
/*     */   {
/* 161 */     if (sk != null) {
/* 162 */       sk.cancel();
/*     */     }
/* 164 */     if (sc != null) {
/* 165 */       sc.socket().setSoLinger(true, 0);
/* 166 */       sc.socket().shutdownOutput();
/* 167 */       sc.close();
/*     */     }
/*     */   }
/*     */   
/*     */   public void closeChannel(Selector selector) throws IOException {
/* 172 */     if (this.serverSocketChannel != null) {
/* 173 */       this.serverSocketChannel.close();
/*     */     }
/*     */   }
/*     */   
/*     */   public void unbind() throws IOException {
/* 178 */     stop();
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\google\code\yanf4j\nio\TCPController.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */