/*     */ package com.google.code.yanf4j.util;
/*     */ 
/*     */ import java.util.AbstractQueue;
/*     */ import java.util.Collection;
/*     */ import java.util.Iterator;
/*     */ import java.util.NoSuchElementException;
/*     */ import java.util.concurrent.BlockingQueue;
/*     */ import java.util.concurrent.TimeUnit;
/*     */ import java.util.concurrent.atomic.AtomicReference;
/*     */ import java.util.concurrent.atomic.AtomicReferenceFieldUpdater;
/*     */ import java.util.concurrent.locks.LockSupport;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class LinkedTransferQueue<E>
/*     */   extends AbstractQueue<E>
/*     */   implements BlockingQueue<E>
/*     */ {
/*     */   private static final int NOWAIT = 0;
/*     */   private static final int TIMEOUT = 1;
/*     */   private static final int WAIT = 2;
/* 110 */   private static final int NCPUS = Runtime.getRuntime().availableProcessors();
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 119 */   private static final int maxTimedSpins = NCPUS < 2 ? 0 : 32;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 126 */   private static final int maxUntimedSpins = maxTimedSpins * 16;
/*     */   
/*     */   private static final long spinForTimeoutThreshold = 1000L;
/*     */   
/*     */   private final PaddedAtomicReference<QNode> head;
/*     */   
/*     */   private final PaddedAtomicReference<QNode> tail;
/*     */   
/*     */   private final PaddedAtomicReference<QNode> cleanMe;
/*     */   
/*     */ 
/*     */   private static final class QNode
/*     */     extends AtomicReference<Object>
/*     */   {
/*     */     private static final long serialVersionUID = 5925596372370723938L;
/*     */     volatile transient QNode next;
/*     */     volatile transient Thread waiter;
/*     */     final boolean isData;
/*     */     private static final AtomicReferenceFieldUpdater<QNode, QNode> nextUpdater;
/*     */     
/*     */     QNode(Object item, boolean isData)
/*     */     {
/* 148 */       super();
/* 149 */       this.isData = isData;
/*     */     }
/*     */     
/*     */     static
/*     */     {
/* 154 */       AtomicReferenceFieldUpdater<QNode, QNode> tmp = null;
/*     */       try {
/* 156 */         tmp = AtomicReferenceFieldUpdater.newUpdater(QNode.class, QNode.class, "next");
/*     */         
/*     */ 
/*     */ 
/* 160 */         QNode testNode = new QNode(null, false);
/* 161 */         tmp.set(testNode, testNode);
/* 162 */         if (testNode.next != testNode)
/*     */         {
/* 164 */           throw new Exception();
/*     */         }
/*     */       }
/*     */       catch (Throwable t) {
/* 168 */         tmp = null;
/*     */       }
/* 170 */       nextUpdater = tmp;
/*     */     }
/*     */     
/*     */     boolean casNext(QNode cmp, QNode val) {
/* 174 */       if (nextUpdater != null) {
/* 175 */         return nextUpdater.compareAndSet(this, cmp, val);
/*     */       }
/* 177 */       return alternativeCasNext(cmp, val);
/*     */     }
/*     */     
/*     */     private synchronized boolean alternativeCasNext(QNode cmp, QNode val)
/*     */     {
/* 182 */       if (this.next == cmp) {
/* 183 */         this.next = val;
/* 184 */         return true;
/*     */       }
/* 186 */       return false;
/*     */     }
/*     */   }
/*     */   
/*     */   private static final class PaddedAtomicReference<T> extends AtomicReference<T>
/*     */   {
/*     */     private static final long serialVersionUID = 4684288940772921317L;
/*     */     Object p0;
/*     */     Object p1;
/*     */     Object p2;
/*     */     Object p3;
/*     */     Object p4;
/*     */     Object p5;
/*     */     Object p6;
/*     */     
/*     */     PaddedAtomicReference(T r) {
/* 202 */       super();
/*     */     }
/*     */     
/*     */ 
/*     */     Object p7;
/*     */     
/*     */     Object p8;
/*     */     
/*     */     Object p9;
/*     */     
/*     */     Object pa;
/*     */     
/*     */     Object pb;
/*     */     
/*     */     Object pc;
/*     */     Object pd;
/*     */     Object pe;
/*     */   }
/*     */   
/*     */   private boolean advanceHead(QNode h, QNode nh)
/*     */   {
/* 223 */     if ((h == this.head.get()) && (this.head.compareAndSet(h, nh))) {
/* 224 */       h.next = h;
/* 225 */       return true;
/*     */     }
/* 227 */     return false;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private Object xfer(Object e, int mode, long nanos)
/*     */   {
/* 240 */     boolean isData = e != null;
/* 241 */     QNode s = null;
/* 242 */     PaddedAtomicReference<QNode> head = this.head;
/* 243 */     PaddedAtomicReference<QNode> tail = this.tail;
/*     */     for (;;)
/*     */     {
/* 246 */       QNode t = (QNode)tail.get();
/* 247 */       QNode h = (QNode)head.get();
/*     */       
/* 249 */       if ((t != null) && ((t == h) || (t.isData == isData))) {
/* 250 */         if (s == null) {
/* 251 */           s = new QNode(e, isData);
/*     */         }
/* 253 */         QNode last = t.next;
/* 254 */         if (last != null) {
/* 255 */           if (t == tail.get()) {
/* 256 */             tail.compareAndSet(t, last);
/*     */           }
/*     */         }
/* 259 */         else if (t.casNext(null, s)) {
/* 260 */           tail.compareAndSet(t, s);
/* 261 */           return awaitFulfill(t, s, e, mode, nanos);
/*     */         }
/*     */         
/*     */       }
/* 265 */       else if (h != null) {
/* 266 */         QNode first = h.next;
/* 267 */         if ((t == tail.get()) && (first != null) && (advanceHead(h, first)))
/*     */         {
/* 269 */           Object x = first.get();
/* 270 */           if ((x != first) && (first.compareAndSet(x, e))) {
/* 271 */             LockSupport.unpark(first.waiter);
/* 272 */             return isData ? e : x;
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private Object fulfill(Object e)
/*     */   {
/* 285 */     boolean isData = e != null;
/* 286 */     PaddedAtomicReference<QNode> head = this.head;
/* 287 */     PaddedAtomicReference<QNode> tail = this.tail;
/*     */     for (;;)
/*     */     {
/* 290 */       QNode t = (QNode)tail.get();
/* 291 */       QNode h = (QNode)head.get();
/*     */       
/* 293 */       if ((t != null) && ((t == h) || (t.isData == isData))) {
/* 294 */         QNode last = t.next;
/* 295 */         if (t == tail.get()) {
/* 296 */           if (last != null) {
/* 297 */             tail.compareAndSet(t, last);
/*     */           } else {
/* 299 */             return null;
/*     */           }
/*     */         }
/*     */       }
/* 303 */       else if (h != null) {
/* 304 */         QNode first = h.next;
/* 305 */         if ((t == tail.get()) && (first != null) && (advanceHead(h, first)))
/*     */         {
/*     */ 
/* 308 */           Object x = first.get();
/* 309 */           if ((x != first) && (first.compareAndSet(x, e))) {
/* 310 */             LockSupport.unpark(first.waiter);
/* 311 */             return isData ? e : x;
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private Object awaitFulfill(QNode pred, QNode s, Object e, int mode, long nanos)
/*     */   {
/* 331 */     if (mode == 0) {
/* 332 */       return null;
/*     */     }
/*     */     
/* 335 */     long lastTime = mode == 1 ? System.nanoTime() : 0L;
/* 336 */     Thread w = Thread.currentThread();
/* 337 */     int spins = -1;
/*     */     for (;;) {
/* 339 */       if (w.isInterrupted()) {
/* 340 */         s.compareAndSet(e, s);
/*     */       }
/* 342 */       Object x = s.get();
/* 343 */       if (x != e) {
/* 344 */         advanceHead(pred, s);
/* 345 */         if (x == s) {
/* 346 */           clean(pred, s);
/* 347 */           return null;
/*     */         }
/* 349 */         if (x != null) {
/* 350 */           s.set(s);
/* 351 */           return x;
/*     */         }
/* 353 */         return e;
/*     */       }
/*     */       
/* 356 */       if (mode == 1) {
/* 357 */         long now = System.nanoTime();
/* 358 */         nanos -= now - lastTime;
/* 359 */         lastTime = now;
/* 360 */         if (nanos <= 0L) {
/* 361 */           s.compareAndSet(e, s);
/* 362 */           continue;
/*     */         }
/*     */       }
/* 365 */       if (spins < 0) {
/* 366 */         QNode h = (QNode)this.head.get();
/* 367 */         spins = (h != null) && (h.next == s) ? maxUntimedSpins : mode == 1 ? maxTimedSpins : 0;
/*     */       }
/*     */       
/*     */ 
/* 371 */       if (spins > 0) {
/* 372 */         spins--;
/* 373 */       } else if (s.waiter == null) {
/* 374 */         s.waiter = w;
/* 375 */       } else if (mode != 1)
/*     */       {
/* 377 */         LockSupport.park();
/* 378 */         s.waiter = null;
/* 379 */         spins = -1;
/*     */       }
/* 381 */       else if (nanos > 1000L)
/*     */       {
/* 383 */         LockSupport.parkNanos(nanos);
/* 384 */         s.waiter = null;
/* 385 */         spins = -1;
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   private QNode getValidatedTail()
/*     */   {
/*     */     for (;;)
/*     */     {
/* 395 */       QNode h = (QNode)this.head.get();
/* 396 */       QNode first = h.next;
/* 397 */       if ((first != null) && (first.next == first)) {
/* 398 */         advanceHead(h, first);
/*     */       }
/*     */       else {
/* 401 */         QNode t = (QNode)this.tail.get();
/* 402 */         QNode last = t.next;
/* 403 */         if (t == this.tail.get()) {
/* 404 */           if (last != null) {
/* 405 */             this.tail.compareAndSet(t, last);
/*     */           } else {
/* 407 */             return t;
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   void clean(QNode pred, QNode s)
/*     */   {
/* 419 */     Thread w = s.waiter;
/* 420 */     if (w != null) {
/* 421 */       s.waiter = null;
/* 422 */       if (w != Thread.currentThread()) {
/* 423 */         LockSupport.unpark(w);
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 434 */     while (pred.next == s) {
/* 435 */       QNode oldpred = reclean();
/* 436 */       QNode t = getValidatedTail();
/* 437 */       if (s != t) {
/* 438 */         QNode sn = s.next;
/* 439 */         if ((sn == s) || (pred.casNext(s, sn))) {
/*     */           break;
/*     */         }
/*     */       } else {
/* 443 */         if ((oldpred == pred) || ((oldpred == null) && (this.cleanMe.compareAndSet(null, pred)))) {
/*     */           break;
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private QNode reclean()
/*     */   {
/*     */     QNode pred;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 468 */     while ((pred = (QNode)this.cleanMe.get()) != null) {
/* 469 */       QNode t = getValidatedTail();
/* 470 */       QNode s = pred.next;
/* 471 */       if (s == t) break;
/*     */       QNode sn;
/* 473 */       if ((s == null) || (s == pred) || (s.get() != s) || ((sn = s.next) == s) || (pred.casNext(s, sn)))
/*     */       {
/* 475 */         this.cleanMe.compareAndSet(pred, null);
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 481 */     return pred;
/*     */   }
/*     */   
/*     */   E cast(Object e)
/*     */   {
/* 486 */     return (E)e;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public LinkedTransferQueue()
/*     */   {
/* 493 */     QNode dummy = new QNode(null, false);
/* 494 */     this.head = new PaddedAtomicReference(dummy);
/* 495 */     this.tail = new PaddedAtomicReference(dummy);
/* 496 */     this.cleanMe = new PaddedAtomicReference(null);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public LinkedTransferQueue(Collection<? extends E> c)
/*     */   {
/* 508 */     this();
/* 509 */     addAll(c);
/*     */   }
/*     */   
/*     */   public void put(E e) throws InterruptedException {
/* 513 */     if (e == null) {
/* 514 */       throw new NullPointerException();
/*     */     }
/* 516 */     if (Thread.interrupted()) {
/* 517 */       throw new InterruptedException();
/*     */     }
/* 519 */     xfer(e, 0, 0L);
/*     */   }
/*     */   
/*     */   public boolean offer(E e, long timeout, TimeUnit unit) throws InterruptedException
/*     */   {
/* 524 */     if (e == null) {
/* 525 */       throw new NullPointerException();
/*     */     }
/* 527 */     if (Thread.interrupted()) {
/* 528 */       throw new InterruptedException();
/*     */     }
/* 530 */     xfer(e, 0, 0L);
/* 531 */     return true;
/*     */   }
/*     */   
/*     */   public boolean offer(E e) {
/* 535 */     if (e == null) {
/* 536 */       throw new NullPointerException();
/*     */     }
/* 538 */     xfer(e, 0, 0L);
/* 539 */     return true;
/*     */   }
/*     */   
/*     */   public void transfer(E e) throws InterruptedException {
/* 543 */     if (e == null) {
/* 544 */       throw new NullPointerException();
/*     */     }
/* 546 */     if (xfer(e, 2, 0L) == null) {
/* 547 */       Thread.interrupted();
/* 548 */       throw new InterruptedException();
/*     */     }
/*     */   }
/*     */   
/*     */   public boolean tryTransfer(E e, long timeout, TimeUnit unit) throws InterruptedException
/*     */   {
/* 554 */     if (e == null) {
/* 555 */       throw new NullPointerException();
/*     */     }
/* 557 */     if (xfer(e, 1, unit.toNanos(timeout)) != null) {
/* 558 */       return true;
/*     */     }
/* 560 */     if (!Thread.interrupted()) {
/* 561 */       return false;
/*     */     }
/* 563 */     throw new InterruptedException();
/*     */   }
/*     */   
/*     */   public boolean tryTransfer(E e) {
/* 567 */     if (e == null) {
/* 568 */       throw new NullPointerException();
/*     */     }
/* 570 */     return fulfill(e) != null;
/*     */   }
/*     */   
/*     */   public E take() throws InterruptedException {
/* 574 */     Object e = xfer(null, 2, 0L);
/* 575 */     if (e != null) {
/* 576 */       return (E)cast(e);
/*     */     }
/* 578 */     Thread.interrupted();
/* 579 */     throw new InterruptedException();
/*     */   }
/*     */   
/*     */   public E poll(long timeout, TimeUnit unit) throws InterruptedException {
/* 583 */     Object e = xfer(null, 1, unit.toNanos(timeout));
/* 584 */     if ((e != null) || (!Thread.interrupted())) {
/* 585 */       return (E)cast(e);
/*     */     }
/* 587 */     throw new InterruptedException();
/*     */   }
/*     */   
/*     */   public E poll() {
/* 591 */     return (E)cast(fulfill(null));
/*     */   }
/*     */   
/*     */   public int drainTo(Collection<? super E> c) {
/* 595 */     if (c == null) {
/* 596 */       throw new NullPointerException();
/*     */     }
/* 598 */     if (c == this) {
/* 599 */       throw new IllegalArgumentException();
/*     */     }
/* 601 */     int n = 0;
/*     */     E e;
/* 603 */     while ((e = poll()) != null) {
/* 604 */       c.add(e);
/* 605 */       n++;
/*     */     }
/* 607 */     return n;
/*     */   }
/*     */   
/*     */   public int drainTo(Collection<? super E> c, int maxElements) {
/* 611 */     if (c == null) {
/* 612 */       throw new NullPointerException();
/*     */     }
/* 614 */     if (c == this) {
/* 615 */       throw new IllegalArgumentException();
/*     */     }
/* 617 */     int n = 0;
/*     */     E e;
/* 619 */     while ((n < maxElements) && ((e = poll()) != null)) {
/* 620 */       c.add(e);
/* 621 */       n++;
/*     */     }
/* 623 */     return n;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   QNode traversalHead()
/*     */   {
/*     */     for (;;)
/*     */     {
/* 633 */       QNode t = (QNode)this.tail.get();
/* 634 */       QNode h = (QNode)this.head.get();
/* 635 */       if ((h != null) && (t != null)) {
/* 636 */         QNode last = t.next;
/* 637 */         QNode first = h.next;
/* 638 */         if (t == this.tail.get()) {
/* 639 */           if (last != null) {
/* 640 */             this.tail.compareAndSet(t, last);
/* 641 */           } else if (first != null) {
/* 642 */             Object x = first.get();
/* 643 */             if (x == first) {
/* 644 */               advanceHead(h, first);
/*     */             } else {
/* 646 */               return h;
/*     */             }
/*     */           } else {
/* 649 */             return h;
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   public Iterator<E> iterator()
/*     */   {
/* 658 */     return new Itr();
/*     */   }
/*     */   
/*     */ 
/*     */   class Itr
/*     */     implements Iterator<E>
/*     */   {
/*     */     LinkedTransferQueue.QNode nextNode;
/*     */     
/*     */     LinkedTransferQueue.QNode currentNode;
/*     */     
/*     */     LinkedTransferQueue.QNode prevNode;
/*     */     
/*     */     E nextItem;
/*     */     
/*     */     Itr()
/*     */     {
/* 675 */       this.nextNode = LinkedTransferQueue.this.traversalHead();
/* 676 */       advance();
/*     */     }
/*     */     
/*     */     E advance() {
/* 680 */       this.prevNode = this.currentNode;
/* 681 */       this.currentNode = this.nextNode;
/* 682 */       E x = this.nextItem;
/*     */       
/* 684 */       LinkedTransferQueue.QNode p = this.nextNode.next;
/*     */       for (;;) {
/* 686 */         if ((p == null) || (!p.isData)) {
/* 687 */           this.nextNode = null;
/* 688 */           this.nextItem = null;
/* 689 */           return x;
/*     */         }
/* 691 */         Object item = p.get();
/* 692 */         if ((item != p) && (item != null)) {
/* 693 */           this.nextNode = p;
/* 694 */           this.nextItem = LinkedTransferQueue.this.cast(item);
/* 695 */           return x;
/*     */         }
/* 697 */         this.prevNode = p;
/* 698 */         p = p.next;
/*     */       }
/*     */     }
/*     */     
/*     */     public boolean hasNext() {
/* 703 */       return this.nextNode != null;
/*     */     }
/*     */     
/*     */     public E next() {
/* 707 */       if (this.nextNode == null) {
/* 708 */         throw new NoSuchElementException();
/*     */       }
/* 710 */       return (E)advance();
/*     */     }
/*     */     
/*     */     public void remove() {
/* 714 */       LinkedTransferQueue.QNode p = this.currentNode;
/* 715 */       LinkedTransferQueue.QNode prev = this.prevNode;
/* 716 */       if ((prev == null) || (p == null)) {
/* 717 */         throw new IllegalStateException();
/*     */       }
/* 719 */       Object x = p.get();
/* 720 */       if ((x != null) && (x != p) && (p.compareAndSet(x, p))) {
/* 721 */         LinkedTransferQueue.this.clean(prev, p);
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   public E peek() {
/*     */     for (;;) {
/* 728 */       QNode h = traversalHead();
/* 729 */       QNode p = h.next;
/* 730 */       if (p == null) {
/* 731 */         return null;
/*     */       }
/* 733 */       Object x = p.get();
/* 734 */       if (p != x) {
/* 735 */         if (!p.isData) {
/* 736 */           return null;
/*     */         }
/* 738 */         if (x != null) {
/* 739 */           return (E)cast(x);
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   public boolean isEmpty()
/*     */   {
/*     */     for (;;) {
/* 748 */       QNode h = traversalHead();
/* 749 */       QNode p = h.next;
/* 750 */       if (p == null) {
/* 751 */         return true;
/*     */       }
/* 753 */       Object x = p.get();
/* 754 */       if (p != x) {
/* 755 */         if (!p.isData) {
/* 756 */           return true;
/*     */         }
/* 758 */         if (x != null) {
/* 759 */           return false;
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   public boolean hasWaitingConsumer() {
/*     */     for (;;) {
/* 767 */       QNode h = traversalHead();
/* 768 */       QNode p = h.next;
/* 769 */       if (p == null) {
/* 770 */         return false;
/*     */       }
/* 772 */       Object x = p.get();
/* 773 */       if (p != x) {
/* 774 */         return !p.isData;
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int size()
/*     */   {
/* 793 */     int count = 0;
/* 794 */     QNode h = traversalHead();
/* 795 */     for (QNode p = h.next; (p != null) && (p.isData); p = p.next) {
/* 796 */       Object x = p.get();
/* 797 */       if ((x != null) && (x != p)) {
/* 798 */         count++; if (count == Integer.MAX_VALUE) {
/*     */           break;
/*     */         }
/*     */       }
/*     */     }
/* 803 */     return count;
/*     */   }
/*     */   
/*     */   public int getWaitingConsumerCount() {
/* 807 */     int count = 0;
/* 808 */     QNode h = traversalHead();
/* 809 */     for (QNode p = h.next; (p != null) && (!p.isData); p = p.next) {
/* 810 */       if (p.get() == null) {
/* 811 */         count++; if (count == Integer.MAX_VALUE) {
/*     */           break;
/*     */         }
/*     */       }
/*     */     }
/* 816 */     return count;
/*     */   }
/*     */   
/*     */   public int remainingCapacity() {
/* 820 */     return Integer.MAX_VALUE;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\google\code\yanf4j\util\LinkedTransferQueue.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */