/*     */ package com.netapp.asup.metamodel;
/*     */ 
/*     */ import com.netapp.asup.common.AsupLogger;
/*     */ import com.netapp.asup.config.LinkedProperties;
/*     */ import com.netapp.asup.exception.ARMAConfigurationException;
/*     */ import com.netapp.asup.exception.ARMAMetaModelException;
/*     */ import java.io.IOException;
/*     */ import java.io.InputStream;
/*     */ import java.util.Collection;
/*     */ import java.util.HashMap;
/*     */ import java.util.LinkedHashMap;
/*     */ import java.util.Map;
/*     */ import java.util.Set;
/*     */ import java.util.regex.Matcher;
/*     */ import java.util.regex.Pattern;
/*     */ import prefuse.data.Graph;
/*     */ 
/*     */ 
/*     */ public class MetaModelFactory
/*     */ {
/*  21 */   private static final AsupLogger LOGGER = AsupLogger.getLogger(MetaModelFactory.class);
/*     */   
/*     */ 
/*     */   private final Connector connector;
/*     */   
/*  26 */   private final Map<String, MetaModel> metaModelMap = new HashMap();
/*     */   
/*     */ 
/*     */   public static enum AsupType
/*     */   {
/*  31 */     NON_DFM_ASUP_TYPE("NON_DFM", "version-detail.properties"), 
/*  32 */     DFM_ASUP_TYPE("DFM", "dfm-version-details.properties");
/*     */     
/*     */     public String getValue() {
/*  35 */       return this.value;
/*     */     }
/*     */     
/*     */     public String getFileName() {
/*  39 */       return this.fileName;
/*     */     }
/*     */     
/*     */     private final String value;
/*     */     private final String fileName;
/*     */     private AsupType(String value, String fileName)
/*     */     {
/*  46 */       this.value = value;
/*  47 */       this.fileName = fileName;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*  53 */   private final Map<String, Map<Pattern, String>> asupTypeToVersionMap = new HashMap();
/*     */   
/*     */   public MetaModelFactory(Connector connector)
/*     */     throws ARMAConfigurationException
/*     */   {
/*  58 */     this.connector = connector;
/*  59 */     LinkedProperties prop = null;
/*  60 */     String versionPropFileName = null;
/*     */     try {
/*  62 */       for (AsupType asupType : AsupType.values()) {
/*  63 */         prop = new LinkedProperties();
/*  64 */         versionPropFileName = asupType.getFileName();
/*  65 */         prop.load(getClass().getClassLoader().getResourceAsStream(versionPropFileName));
/*     */         
/*  67 */         savePatterns(asupType.getValue(), prop);
/*     */       }
/*     */     }
/*     */     catch (IOException e) {
/*  71 */       throw new ARMAConfigurationException(LOGGER, "Filure to load the properties file :" + versionPropFileName, e);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   private void savePatterns(String asupType, LinkedProperties prop)
/*     */   {
/*  79 */     for (String key : prop.getOrderedKeySet()) {
/*  80 */       Pattern sysVersionPattern = Pattern.compile(key);
/*  81 */       Map<Pattern, String> versionMap = (Map)this.asupTypeToVersionMap.get(asupType);
/*     */       
/*  83 */       if (versionMap == null) {
/*  84 */         versionMap = new LinkedHashMap();
/*     */       }
/*  86 */       versionMap.put(sysVersionPattern, prop.getProperty(key));
/*  87 */       this.asupTypeToVersionMap.put(asupType, versionMap);
/*     */     }
/*     */   }
/*     */   
/*     */   public InputStream getMetaModelStream(String metamodelVersion) throws ARMAMetaModelException
/*     */   {
/*  93 */     return this.connector.getMetaModelStream(metamodelVersion);
/*     */   }
/*     */   
/*     */   public MetaModel getMetaModel(String sysVersion, String asupType)
/*     */     throws ARMAMetaModelException
/*     */   {
/*  99 */     LOGGER.debug(sysVersion, "GraphML requested for sysversion : " + sysVersion + " and asupType : " + asupType);
/*     */     
/*     */ 
/* 102 */     Map<Pattern, String> versionMap = (Map)this.asupTypeToVersionMap.get(AsupType.NON_DFM_ASUP_TYPE.getValue());
/*     */     
/* 104 */     if (AsupType.DFM_ASUP_TYPE.getValue().equals(asupType)) {
/* 105 */       versionMap = (Map)this.asupTypeToVersionMap.get(asupType);
/*     */     }
/*     */     
/* 108 */     Set<Pattern> patterns = versionMap.keySet();
/* 109 */     Matcher matcher = null;
/* 110 */     String metaModelVersion = null;
/* 111 */     if (null != sysVersion) {
/* 112 */       for (Pattern pattern : patterns) {
/* 113 */         matcher = pattern.matcher(sysVersion);
/* 114 */         if (matcher.find()) {
/* 115 */           metaModelVersion = (String)versionMap.get(pattern);
/* 116 */           break;
/*     */         }
/*     */       }
/*     */     }
/*     */     else
/*     */     {
/* 122 */       String[] allPatterns = (String[])versionMap.values().toArray(new String[versionMap.size()]);
/* 123 */       metaModelVersion = allPatterns[(versionMap.size() - 1)];
/* 124 */       LOGGER.warn(sysVersion, "Sysversion is null considering the default metamodel,sysversion: " + sysVersion + ",Metamodel version:" + metaModelVersion);
/*     */     }
/*     */     
/*     */ 
/* 128 */     LOGGER.debug(sysVersion, "GraphML for the sysversion : " + sysVersion + ",Metamodel version:" + metaModelVersion);
/*     */     
/*     */ 
/* 131 */     if (metaModelVersion != null) {
/* 132 */       String trimmedVersion = metaModelVersion.trim();
/* 133 */       MetaModel model = (MetaModel)this.metaModelMap.get(trimmedVersion);
/* 134 */       if (model != null) {
/* 135 */         return model;
/*     */       }
/* 137 */       Graph graph = this.connector.getMetaModel(trimmedVersion);
/* 138 */       if (graph != null) {
/* 139 */         model = new MetaModel(graph, trimmedVersion);
/* 140 */         this.metaModelMap.put(trimmedVersion, model);
/* 141 */         return model;
/*     */       }
/* 143 */       throw new ARMAMetaModelException(LOGGER, sysVersion, "Unable to retreive metamodel for sys version");
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 152 */     throw new ARMAMetaModelException(LOGGER, sysVersion, "Unable to find meta model mapping for sys version");
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asup\metamodel\MetaModelFactory.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */