/*     */ package org.apache.avro.file;
/*     */ 
/*     */ import java.io.BufferedOutputStream;
/*     */ import java.io.ByteArrayOutputStream;
/*     */ import java.io.Closeable;
/*     */ import java.io.File;
/*     */ import java.io.FileOutputStream;
/*     */ import java.io.FilterOutputStream;
/*     */ import java.io.Flushable;
/*     */ import java.io.IOException;
/*     */ import java.io.OutputStream;
/*     */ import java.io.UnsupportedEncodingException;
/*     */ import java.nio.ByteBuffer;
/*     */ import java.security.MessageDigest;
/*     */ import java.security.NoSuchAlgorithmException;
/*     */ import java.util.HashMap;
/*     */ import java.util.Map;
/*     */ import java.util.Map.Entry;
/*     */ import java.util.UUID;
/*     */ import org.apache.avro.AvroRuntimeException;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.generic.GenericDatumReader;
/*     */ import org.apache.avro.io.BinaryEncoder;
/*     */ import org.apache.avro.io.DatumWriter;
/*     */ import org.apache.avro.io.EncoderFactory;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class DataFileWriter<D>
/*     */   implements Closeable, Flushable
/*     */ {
/*     */   private Schema schema;
/*     */   private DatumWriter<D> dout;
/*     */   private DataFileWriter<D>.BufferedFileOutputStream out;
/*     */   private BinaryEncoder vout;
/*  60 */   private final Map<String, byte[]> meta = new HashMap();
/*     */   
/*     */   private long blockCount;
/*     */   
/*     */   private NonCopyingByteArrayOutputStream buffer;
/*     */   
/*     */   private BinaryEncoder bufOut;
/*     */   private byte[] sync;
/*  68 */   private int syncInterval = 16000;
/*     */   
/*     */   private boolean isOpen;
/*     */   private Codec codec;
/*     */   
/*     */   public DataFileWriter(DatumWriter<D> dout)
/*     */   {
/*  75 */     this.dout = dout;
/*     */   }
/*     */   
/*     */   private void assertOpen() {
/*  79 */     if (!this.isOpen) throw new AvroRuntimeException("not open");
/*     */   }
/*     */   
/*  82 */   private void assertNotOpen() { if (this.isOpen) { throw new AvroRuntimeException("already open");
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public DataFileWriter<D> setCodec(CodecFactory c)
/*     */   {
/*  90 */     assertNotOpen();
/*  91 */     this.codec = c.createInstance();
/*  92 */     setMetaInternal("avro.codec", this.codec.getName());
/*  93 */     return this;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public DataFileWriter<D> setSyncInterval(int syncInterval)
/*     */   {
/* 109 */     if ((syncInterval < 32) || (syncInterval > 1073741824)) {
/* 110 */       throw new IllegalArgumentException("Invalid syncInterval value: " + syncInterval);
/*     */     }
/* 112 */     this.syncInterval = syncInterval;
/* 113 */     return this;
/*     */   }
/*     */   
/*     */   public DataFileWriter<D> create(Schema schema, File file) throws IOException
/*     */   {
/* 118 */     return create(schema, new FileOutputStream(file));
/*     */   }
/*     */   
/*     */   public DataFileWriter<D> create(Schema schema, OutputStream outs)
/*     */     throws IOException
/*     */   {
/* 124 */     assertNotOpen();
/*     */     
/* 126 */     this.schema = schema;
/* 127 */     setMetaInternal("avro.schema", schema.toString());
/* 128 */     this.sync = generateSync();
/*     */     
/* 130 */     init(outs);
/*     */     
/* 132 */     this.vout.writeFixed(DataFileConstants.MAGIC);
/*     */     
/* 134 */     this.vout.writeMapStart();
/* 135 */     this.vout.setItemCount(this.meta.size());
/* 136 */     for (Map.Entry<String, byte[]> entry : this.meta.entrySet()) {
/* 137 */       this.vout.startItem();
/* 138 */       this.vout.writeString((String)entry.getKey());
/* 139 */       this.vout.writeBytes((byte[])entry.getValue());
/*     */     }
/* 141 */     this.vout.writeMapEnd();
/* 142 */     this.vout.writeFixed(this.sync);
/* 143 */     this.vout.flush();
/* 144 */     return this;
/*     */   }
/*     */   
/*     */   public DataFileWriter<D> appendTo(File file) throws IOException
/*     */   {
/* 149 */     return appendTo(new SeekableFileInput(file), new FileOutputStream(file, true));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public DataFileWriter<D> appendTo(SeekableInput in, OutputStream out)
/*     */     throws IOException
/*     */   {
/* 159 */     assertNotOpen();
/* 160 */     DataFileReader<D> reader = new DataFileReader(in, new GenericDatumReader());
/*     */     
/* 162 */     this.schema = reader.getSchema();
/* 163 */     this.sync = reader.getHeader().sync;
/* 164 */     this.meta.putAll(reader.getHeader().meta);
/* 165 */     byte[] codecBytes = (byte[])this.meta.get("avro.codec");
/* 166 */     if (codecBytes != null) {
/* 167 */       String strCodec = new String(codecBytes, "UTF-8");
/* 168 */       this.codec = CodecFactory.fromString(strCodec).createInstance();
/*     */     } else {
/* 170 */       this.codec = CodecFactory.nullCodec().createInstance();
/*     */     }
/* 172 */     reader.close();
/*     */     
/* 174 */     init(out);
/*     */     
/* 176 */     return this;
/*     */   }
/*     */   
/*     */   private void init(OutputStream outs) throws IOException {
/* 180 */     this.out = new BufferedFileOutputStream(outs);
/* 181 */     EncoderFactory efactory = new EncoderFactory();
/* 182 */     this.vout = efactory.binaryEncoder(this.out, null);
/* 183 */     this.dout.setSchema(this.schema);
/* 184 */     this.buffer = new NonCopyingByteArrayOutputStream(Math.min((int)(this.syncInterval * 1.25D), 1073741822));
/*     */     
/* 186 */     this.bufOut = efactory.binaryEncoder(this.buffer, null);
/* 187 */     if (this.codec == null) {
/* 188 */       this.codec = CodecFactory.nullCodec().createInstance();
/*     */     }
/* 190 */     this.isOpen = true;
/*     */   }
/*     */   
/*     */   private static byte[] generateSync() {
/*     */     try {
/* 195 */       MessageDigest digester = MessageDigest.getInstance("MD5");
/* 196 */       long time = System.currentTimeMillis();
/* 197 */       digester.update((UUID.randomUUID() + "@" + time).getBytes());
/* 198 */       return digester.digest();
/*     */     } catch (NoSuchAlgorithmException e) {
/* 200 */       throw new RuntimeException(e);
/*     */     }
/*     */   }
/*     */   
/*     */   private DataFileWriter<D> setMetaInternal(String key, byte[] value) {
/* 205 */     assertNotOpen();
/* 206 */     this.meta.put(key, value);
/* 207 */     return this;
/*     */   }
/*     */   
/*     */   private DataFileWriter<D> setMetaInternal(String key, String value) {
/*     */     try {
/* 212 */       return setMetaInternal(key, value.getBytes("UTF-8"));
/*     */     } catch (UnsupportedEncodingException e) {
/* 214 */       throw new RuntimeException(e);
/*     */     }
/*     */   }
/*     */   
/*     */   public DataFileWriter<D> setMeta(String key, byte[] value)
/*     */   {
/* 220 */     if (isReservedMeta(key)) {
/* 221 */       throw new AvroRuntimeException("Cannot set reserved meta key: " + key);
/*     */     }
/* 223 */     return setMetaInternal(key, value);
/*     */   }
/*     */   
/*     */   public static boolean isReservedMeta(String key) {
/* 227 */     return key.startsWith("avro.");
/*     */   }
/*     */   
/*     */   public DataFileWriter<D> setMeta(String key, String value)
/*     */   {
/*     */     try {
/* 233 */       return setMeta(key, value.getBytes("UTF-8"));
/*     */     } catch (UnsupportedEncodingException e) {
/* 235 */       throw new RuntimeException(e);
/*     */     }
/*     */   }
/*     */   
/*     */   public DataFileWriter<D> setMeta(String key, long value) {
/* 240 */     return setMeta(key, Long.toString(value));
/*     */   }
/*     */   
/*     */   public static class AppendWriteException extends RuntimeException
/*     */   {
/*     */     public AppendWriteException(Exception e)
/*     */     {
/* 247 */       super();
/*     */     }
/*     */   }
/*     */   
/*     */   public void append(D datum)
/*     */     throws IOException
/*     */   {
/* 254 */     assertOpen();
/* 255 */     int usedBuffer = bufferInUse();
/*     */     try {
/* 257 */       this.dout.write(datum, this.bufOut);
/*     */     } catch (IOException e) {
/* 259 */       resetBufferTo(usedBuffer);
/* 260 */       throw new AppendWriteException(e);
/*     */     } catch (RuntimeException re) {
/* 262 */       resetBufferTo(usedBuffer);
/* 263 */       throw new AppendWriteException(re);
/*     */     }
/* 265 */     this.blockCount += 1L;
/* 266 */     writeIfBlockFull();
/*     */   }
/*     */   
/*     */ 
/*     */   private void resetBufferTo(int size)
/*     */     throws IOException
/*     */   {
/* 273 */     this.bufOut.flush();
/* 274 */     byte[] data = this.buffer.toByteArray();
/* 275 */     this.buffer.reset();
/* 276 */     this.buffer.write(data, 0, size);
/*     */   }
/*     */   
/*     */ 
/*     */   public void appendEncoded(ByteBuffer datum)
/*     */     throws IOException
/*     */   {
/* 283 */     assertOpen();
/* 284 */     this.bufOut.writeFixed(datum);
/* 285 */     this.blockCount += 1L;
/* 286 */     writeIfBlockFull();
/*     */   }
/*     */   
/*     */   private int bufferInUse() {
/* 290 */     return this.buffer.size() + this.bufOut.bytesBuffered();
/*     */   }
/*     */   
/*     */   private void writeIfBlockFull() throws IOException {
/* 294 */     if (bufferInUse() >= this.syncInterval) {
/* 295 */       writeBlock();
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void appendAllFrom(DataFileStream<D> otherFile, boolean recompress)
/*     */     throws IOException
/*     */   {
/* 317 */     assertOpen();
/*     */     
/* 319 */     Schema otherSchema = otherFile.getSchema();
/* 320 */     if (!this.schema.equals(otherSchema)) {
/* 321 */       throw new IOException("Schema from file " + otherFile + " does not match");
/*     */     }
/*     */     
/* 324 */     writeBlock();
/* 325 */     Codec otherCodec = otherFile.resolveCodec();
/* 326 */     DataFileStream.DataBlock nextBlockRaw = null;
/* 327 */     if ((this.codec.equals(otherCodec)) && (!recompress)) {}
/*     */     
/* 329 */     while (otherFile.hasNextBlock()) {
/* 330 */       nextBlockRaw = otherFile.nextRawBlock(nextBlockRaw);
/* 331 */       nextBlockRaw.writeBlockTo(this.vout, this.sync); continue;
/*     */       
/*     */ 
/* 334 */       while (otherFile.hasNextBlock()) {
/* 335 */         nextBlockRaw = otherFile.nextRawBlock(nextBlockRaw);
/* 336 */         nextBlockRaw.decompressUsing(otherCodec);
/* 337 */         nextBlockRaw.compressUsing(this.codec);
/* 338 */         nextBlockRaw.writeBlockTo(this.vout, this.sync);
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   private void writeBlock() throws IOException {
/* 344 */     if (this.blockCount > 0L) {
/* 345 */       this.bufOut.flush();
/* 346 */       ByteBuffer uncompressed = this.buffer.getByteArrayAsByteBuffer();
/* 347 */       DataFileStream.DataBlock block = new DataFileStream.DataBlock(uncompressed, this.blockCount);
/* 348 */       block.compressUsing(this.codec);
/* 349 */       block.writeBlockTo(this.vout, this.sync);
/* 350 */       this.buffer.reset();
/* 351 */       this.blockCount = 0L;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   public long sync()
/*     */     throws IOException
/*     */   {
/* 359 */     assertOpen();
/* 360 */     writeBlock();
/* 361 */     return this.out.tell();
/*     */   }
/*     */   
/*     */   public void flush()
/*     */     throws IOException
/*     */   {
/* 367 */     sync();
/* 368 */     this.vout.flush();
/*     */   }
/*     */   
/*     */   public void close()
/*     */     throws IOException
/*     */   {
/* 374 */     flush();
/* 375 */     this.out.close();
/* 376 */     this.isOpen = false;
/*     */   }
/*     */   
/*     */   private class BufferedFileOutputStream extends BufferedOutputStream {
/*     */     private long position;
/*     */     
/*     */     private class PositionFilter extends FilterOutputStream {
/* 383 */       public PositionFilter(OutputStream out) throws IOException { super(); }
/*     */       
/*     */       public void write(byte[] b, int off, int len) throws IOException {
/* 386 */         this.out.write(b, off, len);
/* 387 */         DataFileWriter.BufferedFileOutputStream.access$014(DataFileWriter.BufferedFileOutputStream.this, len);
/*     */       }
/*     */     }
/*     */     
/*     */     public BufferedFileOutputStream(OutputStream out) throws IOException {
/* 392 */       super();
/* 393 */       this.out = new PositionFilter(out);
/*     */     }
/*     */     
/* 396 */     public long tell() { return this.position + this.count; }
/*     */   }
/*     */   
/*     */   private static class NonCopyingByteArrayOutputStream extends ByteArrayOutputStream {
/*     */     NonCopyingByteArrayOutputStream(int initialSize) {
/* 401 */       super();
/*     */     }
/*     */     
/* 404 */     ByteBuffer getByteArrayAsByteBuffer() { return ByteBuffer.wrap(this.buf, 0, this.count); }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\file\DataFileWriter.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */