/*     */ package org.codehaus.jackson.map.deser;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import org.codehaus.jackson.JsonParser;
/*     */ import org.codehaus.jackson.JsonProcessingException;
/*     */ import org.codehaus.jackson.JsonToken;
/*     */ import org.codehaus.jackson.map.DeserializationConfig.Feature;
/*     */ import org.codehaus.jackson.map.DeserializationContext;
/*     */ import org.codehaus.jackson.map.JsonDeserializer;
/*     */ import org.codehaus.jackson.map.TypeDeserializer;
/*     */ import org.codehaus.jackson.map.annotate.JacksonStdImpl;
/*     */ import org.codehaus.jackson.map.util.ArrayBuilders;
/*     */ import org.codehaus.jackson.map.util.ArrayBuilders.BooleanBuilder;
/*     */ import org.codehaus.jackson.map.util.ArrayBuilders.FloatBuilder;
/*     */ import org.codehaus.jackson.map.util.ArrayBuilders.LongBuilder;
/*     */ import org.codehaus.jackson.map.util.ArrayBuilders.ShortBuilder;
/*     */ import org.codehaus.jackson.map.util.ObjectBuffer;
/*     */ 
/*     */ public class ArrayDeserializers
/*     */ {
/*     */   java.util.HashMap<org.codehaus.jackson.type.JavaType, JsonDeserializer<Object>> _allDeserializers;
/*  22 */   static final ArrayDeserializers instance = new ArrayDeserializers();
/*     */   
/*     */   private ArrayDeserializers()
/*     */   {
/*  26 */     this._allDeserializers = new java.util.HashMap();
/*     */     
/*  28 */     add(Boolean.TYPE, new BooleanDeser());
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  34 */     add(Byte.TYPE, new ByteDeser());
/*  35 */     add(Short.TYPE, new ShortDeser());
/*  36 */     add(Integer.TYPE, new IntDeser());
/*  37 */     add(Long.TYPE, new LongDeser());
/*     */     
/*  39 */     add(Float.TYPE, new FloatDeser());
/*  40 */     add(Double.TYPE, new DoubleDeser());
/*     */     
/*  42 */     add(String.class, new StringDeser());
/*     */     
/*     */ 
/*     */ 
/*  46 */     add(Character.TYPE, new CharDeser());
/*     */   }
/*     */   
/*     */   public static java.util.HashMap<org.codehaus.jackson.type.JavaType, JsonDeserializer<Object>> getAll()
/*     */   {
/*  51 */     return instance._allDeserializers;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private void add(Class<?> cls, JsonDeserializer<?> deser)
/*     */   {
/*  61 */     this._allDeserializers.put(org.codehaus.jackson.map.type.TypeFactory.defaultInstance().constructType(cls), deser);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Object deserializeWithType(JsonParser jp, DeserializationContext ctxt, TypeDeserializer typeDeserializer)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/*  72 */     return typeDeserializer.deserializeTypedFromArray(jp, ctxt);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   static abstract class ArrayDeser<T>
/*     */     extends StdDeserializer<T>
/*     */   {
/*     */     protected ArrayDeser(Class<T> cls)
/*     */     {
/*  85 */       super();
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */     public Object deserializeWithType(JsonParser jp, DeserializationContext ctxt, TypeDeserializer typeDeserializer)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/*  93 */       return typeDeserializer.deserializeTypedFromArray(jp, ctxt);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @JacksonStdImpl
/*     */   static final class StringDeser
/*     */     extends ArrayDeserializers.ArrayDeser<String[]>
/*     */   {
/*     */     public StringDeser()
/*     */     {
/* 107 */       super();
/*     */     }
/*     */     
/*     */ 
/*     */     public String[] deserialize(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 114 */       if (!jp.isExpectedStartArrayToken()) {
/* 115 */         return handleNonArray(jp, ctxt);
/*     */       }
/* 117 */       ObjectBuffer buffer = ctxt.leaseObjectBuffer();
/* 118 */       Object[] chunk = buffer.resetAndStart();
/* 119 */       int ix = 0;
/*     */       
/*     */       JsonToken t;
/* 122 */       while ((t = jp.nextToken()) != JsonToken.END_ARRAY)
/*     */       {
/* 124 */         String value = t == JsonToken.VALUE_NULL ? null : jp.getText();
/* 125 */         if (ix >= chunk.length) {
/* 126 */           chunk = buffer.appendCompletedChunk(chunk);
/* 127 */           ix = 0;
/*     */         }
/* 129 */         chunk[(ix++)] = value;
/*     */       }
/* 131 */       String[] result = (String[])buffer.completeAndClearBuffer(chunk, ix, String.class);
/* 132 */       ctxt.returnObjectBuffer(buffer);
/* 133 */       return result;
/*     */     }
/*     */     
/*     */ 
/*     */     private final String[] handleNonArray(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 140 */       if (!ctxt.isEnabled(DeserializationConfig.Feature.ACCEPT_SINGLE_VALUE_AS_ARRAY)) {
/* 141 */         throw ctxt.mappingException(this._valueClass);
/*     */       }
/* 143 */       return new String[] { jp.getCurrentToken() == JsonToken.VALUE_NULL ? null : jp.getText() };
/*     */     }
/*     */   }
/*     */   
/*     */   @JacksonStdImpl
/*     */   static final class CharDeser extends ArrayDeserializers.ArrayDeser<char[]>
/*     */   {
/*     */     public CharDeser() {
/* 151 */       super();
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public char[] deserialize(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 161 */       JsonToken t = jp.getCurrentToken();
/* 162 */       if (t == JsonToken.VALUE_STRING)
/*     */       {
/* 164 */         char[] buffer = jp.getTextCharacters();
/* 165 */         int offset = jp.getTextOffset();
/* 166 */         int len = jp.getTextLength();
/*     */         
/* 168 */         char[] result = new char[len];
/* 169 */         System.arraycopy(buffer, offset, result, 0, len);
/* 170 */         return result;
/*     */       }
/* 172 */       if (jp.isExpectedStartArrayToken())
/*     */       {
/* 174 */         StringBuilder sb = new StringBuilder(64);
/* 175 */         while ((t = jp.nextToken()) != JsonToken.END_ARRAY) {
/* 176 */           if (t != JsonToken.VALUE_STRING) {
/* 177 */             throw ctxt.mappingException(Character.TYPE);
/*     */           }
/* 179 */           String str = jp.getText();
/* 180 */           if (str.length() != 1) {
/* 181 */             throw org.codehaus.jackson.map.JsonMappingException.from(jp, "Can not convert a JSON String of length " + str.length() + " into a char element of char array");
/*     */           }
/* 183 */           sb.append(str.charAt(0));
/*     */         }
/* 185 */         return sb.toString().toCharArray();
/*     */       }
/*     */       
/* 188 */       if (t == JsonToken.VALUE_EMBEDDED_OBJECT) {
/* 189 */         Object ob = jp.getEmbeddedObject();
/* 190 */         if (ob == null) return null;
/* 191 */         if ((ob instanceof char[])) {
/* 192 */           return (char[])ob;
/*     */         }
/* 194 */         if ((ob instanceof String)) {
/* 195 */           return ((String)ob).toCharArray();
/*     */         }
/*     */         
/* 198 */         if ((ob instanceof byte[])) {
/* 199 */           return org.codehaus.jackson.Base64Variants.getDefaultVariant().encode((byte[])ob, false).toCharArray();
/*     */         }
/*     */       }
/*     */       
/* 203 */       throw ctxt.mappingException(this._valueClass);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @JacksonStdImpl
/*     */   static final class BooleanDeser
/*     */     extends ArrayDeserializers.ArrayDeser<boolean[]>
/*     */   {
/*     */     public BooleanDeser()
/*     */     {
/* 217 */       super();
/*     */     }
/*     */     
/*     */     public boolean[] deserialize(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 223 */       if (!jp.isExpectedStartArrayToken()) {
/* 224 */         return handleNonArray(jp, ctxt);
/*     */       }
/* 226 */       ArrayBuilders.BooleanBuilder builder = ctxt.getArrayBuilders().getBooleanBuilder();
/* 227 */       boolean[] chunk = (boolean[])builder.resetAndStart();
/* 228 */       int ix = 0;
/*     */       
/* 230 */       while (jp.nextToken() != JsonToken.END_ARRAY)
/*     */       {
/* 232 */         boolean value = _parseBooleanPrimitive(jp, ctxt);
/* 233 */         if (ix >= chunk.length) {
/* 234 */           chunk = (boolean[])builder.appendCompletedChunk(chunk, ix);
/* 235 */           ix = 0;
/*     */         }
/* 237 */         chunk[(ix++)] = value;
/*     */       }
/* 239 */       return (boolean[])builder.completeAndClearBuffer(chunk, ix);
/*     */     }
/*     */     
/*     */     private final boolean[] handleNonArray(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 245 */       if (!ctxt.isEnabled(DeserializationConfig.Feature.ACCEPT_SINGLE_VALUE_AS_ARRAY)) {
/* 246 */         throw ctxt.mappingException(this._valueClass);
/*     */       }
/* 248 */       return new boolean[] { _parseBooleanPrimitive(jp, ctxt) };
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   @JacksonStdImpl
/*     */   static final class ByteDeser
/*     */     extends ArrayDeserializers.ArrayDeser<byte[]>
/*     */   {
/*     */     public ByteDeser()
/*     */     {
/* 260 */       super();
/*     */     }
/*     */     
/*     */     public byte[] deserialize(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 266 */       JsonToken t = jp.getCurrentToken();
/*     */       
/*     */ 
/* 269 */       if (t == JsonToken.VALUE_STRING) {
/* 270 */         return jp.getBinaryValue(ctxt.getBase64Variant());
/*     */       }
/*     */       
/* 273 */       if (t == JsonToken.VALUE_EMBEDDED_OBJECT) {
/* 274 */         Object ob = jp.getEmbeddedObject();
/* 275 */         if (ob == null) return null;
/* 276 */         if ((ob instanceof byte[])) {
/* 277 */           return (byte[])ob;
/*     */         }
/*     */       }
/* 280 */       if (!jp.isExpectedStartArrayToken()) {
/* 281 */         return handleNonArray(jp, ctxt);
/*     */       }
/* 283 */       org.codehaus.jackson.map.util.ArrayBuilders.ByteBuilder builder = ctxt.getArrayBuilders().getByteBuilder();
/* 284 */       byte[] chunk = (byte[])builder.resetAndStart();
/* 285 */       int ix = 0;
/*     */       
/* 287 */       while ((t = jp.nextToken()) != JsonToken.END_ARRAY) {
/*     */         byte value;
/*     */         byte value;
/* 290 */         if ((t == JsonToken.VALUE_NUMBER_INT) || (t == JsonToken.VALUE_NUMBER_FLOAT))
/*     */         {
/* 292 */           value = jp.getByteValue();
/*     */         }
/*     */         else {
/* 295 */           if (t != JsonToken.VALUE_NULL) {
/* 296 */             throw ctxt.mappingException(this._valueClass.getComponentType());
/*     */           }
/* 298 */           value = 0;
/*     */         }
/* 300 */         if (ix >= chunk.length) {
/* 301 */           chunk = (byte[])builder.appendCompletedChunk(chunk, ix);
/* 302 */           ix = 0;
/*     */         }
/* 304 */         chunk[(ix++)] = value;
/*     */       }
/* 306 */       return (byte[])builder.completeAndClearBuffer(chunk, ix);
/*     */     }
/*     */     
/*     */     private final byte[] handleNonArray(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 312 */       if (!ctxt.isEnabled(DeserializationConfig.Feature.ACCEPT_SINGLE_VALUE_AS_ARRAY)) {
/* 313 */         throw ctxt.mappingException(this._valueClass);
/*     */       }
/*     */       
/* 316 */       JsonToken t = jp.getCurrentToken();
/* 317 */       byte value; byte value; if ((t == JsonToken.VALUE_NUMBER_INT) || (t == JsonToken.VALUE_NUMBER_FLOAT))
/*     */       {
/* 319 */         value = jp.getByteValue();
/*     */       }
/*     */       else {
/* 322 */         if (t != JsonToken.VALUE_NULL) {
/* 323 */           throw ctxt.mappingException(this._valueClass.getComponentType());
/*     */         }
/* 325 */         value = 0;
/*     */       }
/* 327 */       return new byte[] { value };
/*     */     }
/*     */   }
/*     */   
/*     */   @JacksonStdImpl
/*     */   static final class ShortDeser extends ArrayDeserializers.ArrayDeser<short[]>
/*     */   {
/*     */     public ShortDeser() {
/* 335 */       super();
/*     */     }
/*     */     
/*     */     public short[] deserialize(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 341 */       if (!jp.isExpectedStartArrayToken()) {
/* 342 */         return handleNonArray(jp, ctxt);
/*     */       }
/* 344 */       ArrayBuilders.ShortBuilder builder = ctxt.getArrayBuilders().getShortBuilder();
/* 345 */       short[] chunk = (short[])builder.resetAndStart();
/* 346 */       int ix = 0;
/*     */       
/* 348 */       while (jp.nextToken() != JsonToken.END_ARRAY) {
/* 349 */         short value = _parseShortPrimitive(jp, ctxt);
/* 350 */         if (ix >= chunk.length) {
/* 351 */           chunk = (short[])builder.appendCompletedChunk(chunk, ix);
/* 352 */           ix = 0;
/*     */         }
/* 354 */         chunk[(ix++)] = value;
/*     */       }
/* 356 */       return (short[])builder.completeAndClearBuffer(chunk, ix);
/*     */     }
/*     */     
/*     */     private final short[] handleNonArray(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 362 */       if (!ctxt.isEnabled(DeserializationConfig.Feature.ACCEPT_SINGLE_VALUE_AS_ARRAY)) {
/* 363 */         throw ctxt.mappingException(this._valueClass);
/*     */       }
/* 365 */       return new short[] { _parseShortPrimitive(jp, ctxt) };
/*     */     }
/*     */   }
/*     */   
/*     */   @JacksonStdImpl
/*     */   static final class IntDeser extends ArrayDeserializers.ArrayDeser<int[]>
/*     */   {
/*     */     public IntDeser() {
/* 373 */       super();
/*     */     }
/*     */     
/*     */     public int[] deserialize(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 379 */       if (!jp.isExpectedStartArrayToken()) {
/* 380 */         return handleNonArray(jp, ctxt);
/*     */       }
/* 382 */       org.codehaus.jackson.map.util.ArrayBuilders.IntBuilder builder = ctxt.getArrayBuilders().getIntBuilder();
/* 383 */       int[] chunk = (int[])builder.resetAndStart();
/* 384 */       int ix = 0;
/*     */       
/* 386 */       while (jp.nextToken() != JsonToken.END_ARRAY)
/*     */       {
/* 388 */         int value = _parseIntPrimitive(jp, ctxt);
/* 389 */         if (ix >= chunk.length) {
/* 390 */           chunk = (int[])builder.appendCompletedChunk(chunk, ix);
/* 391 */           ix = 0;
/*     */         }
/* 393 */         chunk[(ix++)] = value;
/*     */       }
/* 395 */       return (int[])builder.completeAndClearBuffer(chunk, ix);
/*     */     }
/*     */     
/*     */     private final int[] handleNonArray(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 401 */       if (!ctxt.isEnabled(DeserializationConfig.Feature.ACCEPT_SINGLE_VALUE_AS_ARRAY)) {
/* 402 */         throw ctxt.mappingException(this._valueClass);
/*     */       }
/* 404 */       return new int[] { _parseIntPrimitive(jp, ctxt) };
/*     */     }
/*     */   }
/*     */   
/*     */   @JacksonStdImpl
/*     */   static final class LongDeser extends ArrayDeserializers.ArrayDeser<long[]>
/*     */   {
/*     */     public LongDeser() {
/* 412 */       super();
/*     */     }
/*     */     
/*     */     public long[] deserialize(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 418 */       if (!jp.isExpectedStartArrayToken()) {
/* 419 */         return handleNonArray(jp, ctxt);
/*     */       }
/* 421 */       ArrayBuilders.LongBuilder builder = ctxt.getArrayBuilders().getLongBuilder();
/* 422 */       long[] chunk = (long[])builder.resetAndStart();
/* 423 */       int ix = 0;
/*     */       
/* 425 */       while (jp.nextToken() != JsonToken.END_ARRAY) {
/* 426 */         long value = _parseLongPrimitive(jp, ctxt);
/* 427 */         if (ix >= chunk.length) {
/* 428 */           chunk = (long[])builder.appendCompletedChunk(chunk, ix);
/* 429 */           ix = 0;
/*     */         }
/* 431 */         chunk[(ix++)] = value;
/*     */       }
/* 433 */       return (long[])builder.completeAndClearBuffer(chunk, ix);
/*     */     }
/*     */     
/*     */     private final long[] handleNonArray(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 439 */       if (!ctxt.isEnabled(DeserializationConfig.Feature.ACCEPT_SINGLE_VALUE_AS_ARRAY)) {
/* 440 */         throw ctxt.mappingException(this._valueClass);
/*     */       }
/* 442 */       return new long[] { _parseLongPrimitive(jp, ctxt) };
/*     */     }
/*     */   }
/*     */   
/*     */   @JacksonStdImpl
/*     */   static final class FloatDeser extends ArrayDeserializers.ArrayDeser<float[]>
/*     */   {
/*     */     public FloatDeser() {
/* 450 */       super();
/*     */     }
/*     */     
/*     */     public float[] deserialize(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 456 */       if (!jp.isExpectedStartArrayToken()) {
/* 457 */         return handleNonArray(jp, ctxt);
/*     */       }
/* 459 */       ArrayBuilders.FloatBuilder builder = ctxt.getArrayBuilders().getFloatBuilder();
/* 460 */       float[] chunk = (float[])builder.resetAndStart();
/* 461 */       int ix = 0;
/*     */       
/* 463 */       while (jp.nextToken() != JsonToken.END_ARRAY)
/*     */       {
/* 465 */         float value = _parseFloatPrimitive(jp, ctxt);
/* 466 */         if (ix >= chunk.length) {
/* 467 */           chunk = (float[])builder.appendCompletedChunk(chunk, ix);
/* 468 */           ix = 0;
/*     */         }
/* 470 */         chunk[(ix++)] = value;
/*     */       }
/* 472 */       return (float[])builder.completeAndClearBuffer(chunk, ix);
/*     */     }
/*     */     
/*     */     private final float[] handleNonArray(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 478 */       if (!ctxt.isEnabled(DeserializationConfig.Feature.ACCEPT_SINGLE_VALUE_AS_ARRAY)) {
/* 479 */         throw ctxt.mappingException(this._valueClass);
/*     */       }
/* 481 */       return new float[] { _parseFloatPrimitive(jp, ctxt) };
/*     */     }
/*     */   }
/*     */   
/*     */   @JacksonStdImpl
/*     */   static final class DoubleDeser extends ArrayDeserializers.ArrayDeser<double[]>
/*     */   {
/*     */     public DoubleDeser() {
/* 489 */       super();
/*     */     }
/*     */     
/*     */     public double[] deserialize(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 495 */       if (!jp.isExpectedStartArrayToken()) {
/* 496 */         return handleNonArray(jp, ctxt);
/*     */       }
/* 498 */       org.codehaus.jackson.map.util.ArrayBuilders.DoubleBuilder builder = ctxt.getArrayBuilders().getDoubleBuilder();
/* 499 */       double[] chunk = (double[])builder.resetAndStart();
/* 500 */       int ix = 0;
/*     */       
/* 502 */       while (jp.nextToken() != JsonToken.END_ARRAY) {
/* 503 */         double value = _parseDoublePrimitive(jp, ctxt);
/* 504 */         if (ix >= chunk.length) {
/* 505 */           chunk = (double[])builder.appendCompletedChunk(chunk, ix);
/* 506 */           ix = 0;
/*     */         }
/* 508 */         chunk[(ix++)] = value;
/*     */       }
/* 510 */       return (double[])builder.completeAndClearBuffer(chunk, ix);
/*     */     }
/*     */     
/*     */     private final double[] handleNonArray(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 516 */       if (!ctxt.isEnabled(DeserializationConfig.Feature.ACCEPT_SINGLE_VALUE_AS_ARRAY)) {
/* 517 */         throw ctxt.mappingException(this._valueClass);
/*     */       }
/* 519 */       return new double[] { _parseDoublePrimitive(jp, ctxt) };
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\deser\ArrayDeserializers.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */