/*     */ package org.codehaus.jackson.map.ext;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.util.Arrays;
/*     */ import java.util.Collection;
/*     */ import org.codehaus.jackson.JsonParser;
/*     */ import org.codehaus.jackson.JsonProcessingException;
/*     */ import org.codehaus.jackson.JsonToken;
/*     */ import org.codehaus.jackson.map.DeserializationContext;
/*     */ import org.codehaus.jackson.map.deser.StdDeserializer;
/*     */ import org.codehaus.jackson.map.deser.StdScalarDeserializer;
/*     */ import org.codehaus.jackson.map.util.Provider;
/*     */ import org.joda.time.DateMidnight;
/*     */ import org.joda.time.DateTime;
/*     */ import org.joda.time.DateTimeZone;
/*     */ import org.joda.time.LocalDate;
/*     */ import org.joda.time.LocalDateTime;
/*     */ import org.joda.time.ReadableDateTime;
/*     */ import org.joda.time.ReadableInstant;
/*     */ import org.joda.time.format.DateTimeFormatter;
/*     */ import org.joda.time.format.ISODateTimeFormat;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class JodaDeserializers
/*     */   implements Provider<StdDeserializer<?>>
/*     */ {
/*     */   public Collection<StdDeserializer<?>> provide()
/*     */   {
/*  32 */     return Arrays.asList(new StdDeserializer[] { new DateTimeDeserializer(DateTime.class), new DateTimeDeserializer(ReadableDateTime.class), new DateTimeDeserializer(ReadableInstant.class), new LocalDateDeserializer(), new LocalDateTimeDeserializer(), new DateMidnightDeserializer() });
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   static abstract class JodaDeserializer<T>
/*     */     extends StdScalarDeserializer<T>
/*     */   {
/*  50 */     static final DateTimeFormatter _localDateTimeFormat = ;
/*     */     
/*  52 */     protected JodaDeserializer(Class<T> cls) { super(); }
/*     */     
/*     */     protected DateTime parseLocal(JsonParser jp)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/*  57 */       String str = jp.getText().trim();
/*  58 */       if (str.length() == 0) {
/*  59 */         return null;
/*     */       }
/*  61 */       return _localDateTimeFormat.parseDateTime(str);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static class DateTimeDeserializer<T extends ReadableInstant>
/*     */     extends JodaDeserializers.JodaDeserializer<T>
/*     */   {
/*     */     public DateTimeDeserializer(Class<T> cls)
/*     */     {
/*  82 */       super();
/*     */     }
/*     */     
/*     */ 
/*     */     public T deserialize(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/*  89 */       JsonToken t = jp.getCurrentToken();
/*  90 */       if (t == JsonToken.VALUE_NUMBER_INT) {
/*  91 */         return new DateTime(jp.getLongValue(), DateTimeZone.UTC);
/*     */       }
/*  93 */       if (t == JsonToken.VALUE_STRING) {
/*  94 */         String str = jp.getText().trim();
/*  95 */         if (str.length() == 0) {
/*  96 */           return null;
/*     */         }
/*  98 */         return new DateTime(str, DateTimeZone.UTC);
/*     */       }
/* 100 */       throw ctxt.mappingException(getValueClass());
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   public static class LocalDateDeserializer
/*     */     extends JodaDeserializers.JodaDeserializer<LocalDate>
/*     */   {
/*     */     public LocalDateDeserializer()
/*     */     {
/* 110 */       super();
/*     */     }
/*     */     
/*     */ 
/*     */     public LocalDate deserialize(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 117 */       if (jp.isExpectedStartArrayToken()) {
/* 118 */         jp.nextToken();
/* 119 */         int year = jp.getIntValue();
/* 120 */         jp.nextToken();
/* 121 */         int month = jp.getIntValue();
/* 122 */         jp.nextToken();
/* 123 */         int day = jp.getIntValue();
/* 124 */         if (jp.nextToken() != JsonToken.END_ARRAY) {
/* 125 */           throw ctxt.wrongTokenException(jp, JsonToken.END_ARRAY, "after LocalDate ints");
/*     */         }
/* 127 */         return new LocalDate(year, month, day);
/*     */       }
/* 129 */       switch (JodaDeserializers.1.$SwitchMap$org$codehaus$jackson$JsonToken[jp.getCurrentToken().ordinal()]) {
/*     */       case 1: 
/* 131 */         return new LocalDate(jp.getLongValue());
/*     */       case 2: 
/* 133 */         DateTime local = parseLocal(jp);
/* 134 */         if (local == null) {
/* 135 */           return null;
/*     */         }
/* 137 */         return local.toLocalDate();
/*     */       }
/* 139 */       throw ctxt.wrongTokenException(jp, JsonToken.START_ARRAY, "expected JSON Array, String or Number");
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   public static class LocalDateTimeDeserializer
/*     */     extends JodaDeserializers.JodaDeserializer<LocalDateTime>
/*     */   {
/*     */     public LocalDateTimeDeserializer()
/*     */     {
/* 149 */       super();
/*     */     }
/*     */     
/*     */ 
/*     */     public LocalDateTime deserialize(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 156 */       if (jp.isExpectedStartArrayToken()) {
/* 157 */         jp.nextToken();
/* 158 */         int year = jp.getIntValue();
/* 159 */         jp.nextToken();
/* 160 */         int month = jp.getIntValue();
/* 161 */         jp.nextToken();
/* 162 */         int day = jp.getIntValue();
/* 163 */         jp.nextToken();
/* 164 */         int hour = jp.getIntValue();
/* 165 */         jp.nextToken();
/* 166 */         int minute = jp.getIntValue();
/* 167 */         jp.nextToken();
/* 168 */         int second = jp.getIntValue();
/*     */         
/* 170 */         int millisecond = 0;
/* 171 */         if (jp.nextToken() != JsonToken.END_ARRAY) {
/* 172 */           millisecond = jp.getIntValue();
/* 173 */           jp.nextToken();
/*     */         }
/* 175 */         if (jp.getCurrentToken() != JsonToken.END_ARRAY) {
/* 176 */           throw ctxt.wrongTokenException(jp, JsonToken.END_ARRAY, "after LocalDateTime ints");
/*     */         }
/* 178 */         return new LocalDateTime(year, month, day, hour, minute, second, millisecond);
/*     */       }
/*     */       
/* 181 */       switch (JodaDeserializers.1.$SwitchMap$org$codehaus$jackson$JsonToken[jp.getCurrentToken().ordinal()]) {
/*     */       case 1: 
/* 183 */         return new LocalDateTime(jp.getLongValue());
/*     */       case 2: 
/* 185 */         DateTime local = parseLocal(jp);
/* 186 */         if (local == null) {
/* 187 */           return null;
/*     */         }
/* 189 */         return local.toLocalDateTime();
/*     */       }
/* 191 */       throw ctxt.wrongTokenException(jp, JsonToken.START_ARRAY, "expected JSON Array or Number");
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   public static class DateMidnightDeserializer
/*     */     extends JodaDeserializers.JodaDeserializer<DateMidnight>
/*     */   {
/*     */     public DateMidnightDeserializer()
/*     */     {
/* 201 */       super();
/*     */     }
/*     */     
/*     */ 
/*     */     public DateMidnight deserialize(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 208 */       if (jp.isExpectedStartArrayToken()) {
/* 209 */         jp.nextToken();
/* 210 */         int year = jp.getIntValue();
/* 211 */         jp.nextToken();
/* 212 */         int month = jp.getIntValue();
/* 213 */         jp.nextToken();
/* 214 */         int day = jp.getIntValue();
/* 215 */         if (jp.nextToken() != JsonToken.END_ARRAY) {
/* 216 */           throw ctxt.wrongTokenException(jp, JsonToken.END_ARRAY, "after DateMidnight ints");
/*     */         }
/* 218 */         return new DateMidnight(year, month, day);
/*     */       }
/* 220 */       switch (JodaDeserializers.1.$SwitchMap$org$codehaus$jackson$JsonToken[jp.getCurrentToken().ordinal()]) {
/*     */       case 1: 
/* 222 */         return new DateMidnight(jp.getLongValue());
/*     */       case 2: 
/* 224 */         DateTime local = parseLocal(jp);
/* 225 */         if (local == null) {
/* 226 */           return null;
/*     */         }
/* 228 */         return local.toDateMidnight();
/*     */       }
/* 230 */       throw ctxt.wrongTokenException(jp, JsonToken.START_ARRAY, "expected JSON Array, Number or String");
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\ext\JodaDeserializers.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */