/*     */ package org.codehaus.jackson.map.introspect;
/*     */ 
/*     */ import java.lang.annotation.Annotation;
/*     */ import java.lang.reflect.Constructor;
/*     */ import java.lang.reflect.Field;
/*     */ import java.lang.reflect.Method;
/*     */ import java.util.Collections;
/*     */ import java.util.Iterator;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import org.codehaus.jackson.map.AnnotationIntrospector;
/*     */ import org.codehaus.jackson.map.ClassIntrospector.MixInResolver;
/*     */ 
/*     */ public final class AnnotatedClass extends Annotated
/*     */ {
/*  16 */   private static final AnnotationMap[] NO_ANNOTATION_MAPS = new AnnotationMap[0];
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final Class<?> _class;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final java.util.Collection<Class<?>> _superTypes;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final AnnotationIntrospector _annotationIntrospector;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final ClassIntrospector.MixInResolver _mixInResolver;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final Class<?> _primaryMixIn;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected AnnotationMap _classAnnotations;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected AnnotatedConstructor _defaultConstructor;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected List<AnnotatedConstructor> _constructors;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected List<AnnotatedMethod> _creatorMethods;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected AnnotatedMethodMap _memberMethods;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected List<AnnotatedField> _fields;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected List<AnnotatedMethod> _ignoredMethods;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected List<AnnotatedField> _ignoredFields;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private AnnotatedClass(Class<?> cls, List<Class<?>> superTypes, AnnotationIntrospector aintr, ClassIntrospector.MixInResolver mir)
/*     */   {
/* 123 */     this._class = cls;
/* 124 */     this._superTypes = superTypes;
/* 125 */     this._annotationIntrospector = aintr;
/* 126 */     this._mixInResolver = mir;
/* 127 */     this._primaryMixIn = (this._mixInResolver == null ? null : this._mixInResolver.findMixInClassFor(this._class));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static AnnotatedClass construct(Class<?> cls, AnnotationIntrospector aintr, ClassIntrospector.MixInResolver mir)
/*     */   {
/* 139 */     List<Class<?>> st = org.codehaus.jackson.map.util.ClassUtil.findSuperTypes(cls, null);
/* 140 */     AnnotatedClass ac = new AnnotatedClass(cls, st, aintr, mir);
/* 141 */     ac.resolveClassAnnotations();
/* 142 */     return ac;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static AnnotatedClass constructWithoutSuperTypes(Class<?> cls, AnnotationIntrospector aintr, ClassIntrospector.MixInResolver mir)
/*     */   {
/* 153 */     List<Class<?>> empty = Collections.emptyList();
/* 154 */     AnnotatedClass ac = new AnnotatedClass(cls, empty, aintr, mir);
/* 155 */     ac.resolveClassAnnotations();
/* 156 */     return ac;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Class<?> getAnnotated()
/*     */   {
/* 166 */     return this._class;
/*     */   }
/*     */   
/* 169 */   public int getModifiers() { return this._class.getModifiers(); }
/*     */   
/*     */   public String getName() {
/* 172 */     return this._class.getName();
/*     */   }
/*     */   
/*     */   public <A extends Annotation> A getAnnotation(Class<A> acls)
/*     */   {
/* 177 */     if (this._classAnnotations == null) {
/* 178 */       return null;
/*     */     }
/* 180 */     return this._classAnnotations.get(acls);
/*     */   }
/*     */   
/*     */   public java.lang.reflect.Type getGenericType()
/*     */   {
/* 185 */     return this._class;
/*     */   }
/*     */   
/*     */   public Class<?> getRawType()
/*     */   {
/* 190 */     return this._class;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 202 */   public org.codehaus.jackson.map.util.Annotations getAnnotations() { return this._classAnnotations; }
/*     */   
/* 204 */   public boolean hasAnnotations() { return this._classAnnotations.size() > 0; }
/*     */   
/* 206 */   public AnnotatedConstructor getDefaultConstructor() { return this._defaultConstructor; }
/*     */   
/*     */   public List<AnnotatedConstructor> getConstructors()
/*     */   {
/* 210 */     if (this._constructors == null) {
/* 211 */       return Collections.emptyList();
/*     */     }
/* 213 */     return this._constructors;
/*     */   }
/*     */   
/*     */   public List<AnnotatedMethod> getStaticMethods()
/*     */   {
/* 218 */     if (this._creatorMethods == null) {
/* 219 */       return Collections.emptyList();
/*     */     }
/* 221 */     return this._creatorMethods;
/*     */   }
/*     */   
/*     */   public Iterable<AnnotatedMethod> memberMethods()
/*     */   {
/* 226 */     return this._memberMethods;
/*     */   }
/*     */   
/*     */   public Iterable<AnnotatedMethod> ignoredMemberMethods()
/*     */   {
/* 231 */     if (this._ignoredMethods == null) {
/* 232 */       List<AnnotatedMethod> l = Collections.emptyList();
/* 233 */       return l;
/*     */     }
/* 235 */     return this._ignoredMethods;
/*     */   }
/*     */   
/*     */   public int getMemberMethodCount()
/*     */   {
/* 240 */     return this._memberMethods.size();
/*     */   }
/*     */   
/*     */   public AnnotatedMethod findMethod(String name, Class<?>[] paramTypes)
/*     */   {
/* 245 */     return this._memberMethods.find(name, paramTypes);
/*     */   }
/*     */   
/*     */   public int getFieldCount() {
/* 249 */     return this._fields == null ? 0 : this._fields.size();
/*     */   }
/*     */   
/*     */   public Iterable<AnnotatedField> fields()
/*     */   {
/* 254 */     if (this._fields == null) {
/* 255 */       List<AnnotatedField> l = Collections.emptyList();
/* 256 */       return l;
/*     */     }
/* 258 */     return this._fields;
/*     */   }
/*     */   
/*     */   public Iterable<AnnotatedField> ignoredFields()
/*     */   {
/* 263 */     if (this._ignoredFields == null) {
/* 264 */       List<AnnotatedField> l = Collections.emptyList();
/* 265 */       return l;
/*     */     }
/* 267 */     return this._ignoredFields;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void resolveClassAnnotations()
/*     */   {
/* 288 */     this._classAnnotations = new AnnotationMap();
/*     */     
/* 290 */     if (this._annotationIntrospector == null) {
/* 291 */       return;
/*     */     }
/*     */     
/*     */ 
/* 295 */     if (this._primaryMixIn != null) {
/* 296 */       _addClassMixIns(this._classAnnotations, this._class, this._primaryMixIn);
/*     */     }
/*     */     
/* 299 */     for (Annotation a : this._class.getDeclaredAnnotations()) {
/* 300 */       if (this._annotationIntrospector.isHandled(a)) {
/* 301 */         this._classAnnotations.addIfNotPresent(a);
/*     */       }
/*     */     }
/*     */     
/*     */ 
/* 306 */     for (Class<?> cls : this._superTypes)
/*     */     {
/* 308 */       _addClassMixIns(this._classAnnotations, cls);
/* 309 */       for (Annotation a : cls.getDeclaredAnnotations()) {
/* 310 */         if (this._annotationIntrospector.isHandled(a)) {
/* 311 */           this._classAnnotations.addIfNotPresent(a);
/*     */         }
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 324 */     _addClassMixIns(this._classAnnotations, Object.class);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void _addClassMixIns(AnnotationMap annotations, Class<?> toMask)
/*     */   {
/* 333 */     if (this._mixInResolver != null) {
/* 334 */       _addClassMixIns(annotations, toMask, this._mixInResolver.findMixInClassFor(toMask));
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   protected void _addClassMixIns(AnnotationMap annotations, Class<?> toMask, Class<?> mixin)
/*     */   {
/* 341 */     if (mixin == null) {
/* 342 */       return;
/*     */     }
/*     */     
/* 345 */     for (Annotation a : mixin.getDeclaredAnnotations()) {
/* 346 */       if (this._annotationIntrospector.isHandled(a)) {
/* 347 */         annotations.addIfNotPresent(a);
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 357 */     for (Class<?> parent : org.codehaus.jackson.map.util.ClassUtil.findSuperTypes(mixin, toMask)) {
/* 358 */       for (Annotation a : parent.getDeclaredAnnotations()) {
/* 359 */         if (this._annotationIntrospector.isHandled(a)) {
/* 360 */           annotations.addIfNotPresent(a);
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void resolveCreators(boolean includeAll)
/*     */   {
/* 385 */     this._constructors = null;
/* 386 */     for (Constructor<?> ctor : this._class.getDeclaredConstructors()) {
/* 387 */       switch (ctor.getParameterTypes().length) {
/*     */       case 0: 
/* 389 */         this._defaultConstructor = _constructConstructor(ctor, true);
/* 390 */         break;
/*     */       default: 
/* 392 */         if (includeAll) {
/* 393 */           if (this._constructors == null) {
/* 394 */             this._constructors = new java.util.ArrayList();
/*     */           }
/* 396 */           this._constructors.add(_constructConstructor(ctor, false));
/*     */         }
/*     */         break;
/*     */       }
/*     */     }
/* 401 */     if ((this._primaryMixIn != null) && (
/* 402 */       (this._defaultConstructor != null) || (this._constructors != null))) {
/* 403 */       _addConstructorMixIns(this._primaryMixIn);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 412 */     if (this._annotationIntrospector != null) {
/* 413 */       if ((this._defaultConstructor != null) && 
/* 414 */         (this._annotationIntrospector.isIgnorableConstructor(this._defaultConstructor))) {
/* 415 */         this._defaultConstructor = null;
/*     */       }
/*     */       
/* 418 */       if (this._constructors != null)
/*     */       {
/* 420 */         int i = this._constructors.size(); for (;;) { i--; if (i < 0) break;
/* 421 */           if (this._annotationIntrospector.isIgnorableConstructor((AnnotatedConstructor)this._constructors.get(i))) {
/* 422 */             this._constructors.remove(i);
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/*     */     
/* 428 */     this._creatorMethods = null;
/*     */     
/* 430 */     if (includeAll)
/*     */     {
/*     */ 
/*     */ 
/* 434 */       for (Method m : this._class.getDeclaredMethods())
/* 435 */         if (java.lang.reflect.Modifier.isStatic(m.getModifiers()))
/*     */         {
/*     */ 
/* 438 */           int argCount = m.getParameterTypes().length;
/*     */           
/* 440 */           if (argCount >= 1)
/*     */           {
/*     */ 
/* 443 */             if (this._creatorMethods == null) {
/* 444 */               this._creatorMethods = new java.util.ArrayList();
/*     */             }
/* 446 */             this._creatorMethods.add(_constructCreatorMethod(m));
/*     */           }
/*     */         }
/* 449 */       if ((this._primaryMixIn != null) && (this._creatorMethods != null)) {
/* 450 */         _addFactoryMixIns(this._primaryMixIn);
/*     */       }
/*     */       
/* 453 */       if ((this._annotationIntrospector != null) && 
/* 454 */         (this._creatorMethods != null))
/*     */       {
/* 456 */         int i = this._creatorMethods.size(); for (;;) { i--; if (i < 0) break;
/* 457 */           if (this._annotationIntrospector.isIgnorableMethod((AnnotatedMethod)this._creatorMethods.get(i))) {
/* 458 */             this._creatorMethods.remove(i);
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   protected void _addConstructorMixIns(Class<?> mixin)
/*     */   {
/* 468 */     MemberKey[] ctorKeys = null;
/* 469 */     int ctorCount = this._constructors == null ? 0 : this._constructors.size();
/* 470 */     for (Constructor<?> ctor : mixin.getDeclaredConstructors()) {
/* 471 */       switch (ctor.getParameterTypes().length) {
/*     */       case 0: 
/* 473 */         if (this._defaultConstructor != null) {
/* 474 */           _addMixOvers(ctor, this._defaultConstructor, false);
/*     */         }
/*     */         break;
/*     */       default: 
/* 478 */         if (ctorKeys == null) {
/* 479 */           ctorKeys = new MemberKey[ctorCount];
/* 480 */           for (int i = 0; i < ctorCount; i++) {
/* 481 */             ctorKeys[i] = new MemberKey(((AnnotatedConstructor)this._constructors.get(i)).getAnnotated());
/*     */           }
/*     */         }
/* 484 */         MemberKey key = new MemberKey(ctor);
/*     */         
/* 486 */         for (int i = 0; i < ctorCount; i++) {
/* 487 */           if (key.equals(ctorKeys[i]))
/*     */           {
/*     */ 
/* 490 */             _addMixOvers(ctor, (AnnotatedConstructor)this._constructors.get(i), true);
/* 491 */             break;
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   protected void _addFactoryMixIns(Class<?> mixin) {
/* 499 */     MemberKey[] methodKeys = null;
/* 500 */     int methodCount = this._creatorMethods.size();
/*     */     
/* 502 */     for (Method m : mixin.getDeclaredMethods()) {
/* 503 */       if (java.lang.reflect.Modifier.isStatic(m.getModifiers()))
/*     */       {
/*     */ 
/* 506 */         if (m.getParameterTypes().length != 0)
/*     */         {
/*     */ 
/* 509 */           if (methodKeys == null) {
/* 510 */             methodKeys = new MemberKey[methodCount];
/* 511 */             for (int i = 0; i < methodCount; i++) {
/* 512 */               methodKeys[i] = new MemberKey(((AnnotatedMethod)this._creatorMethods.get(i)).getAnnotated());
/*     */             }
/*     */           }
/* 515 */           MemberKey key = new MemberKey(m);
/* 516 */           for (int i = 0; i < methodCount; i++) {
/* 517 */             if (key.equals(methodKeys[i]))
/*     */             {
/*     */ 
/* 520 */               _addMixOvers(m, (AnnotatedMethod)this._creatorMethods.get(i), true);
/* 521 */               break;
/*     */             }
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void resolveMemberMethods(MethodFilter methodFilter, boolean collectIgnored)
/*     */   {
/* 540 */     this._memberMethods = new AnnotatedMethodMap();
/* 541 */     AnnotatedMethodMap mixins = new AnnotatedMethodMap();
/*     */     
/* 543 */     _addMemberMethods(this._class, methodFilter, this._memberMethods, this._primaryMixIn, mixins);
/*     */     
/*     */ 
/* 546 */     for (Class<?> cls : this._superTypes) {
/* 547 */       Class<?> mixin = this._mixInResolver == null ? null : this._mixInResolver.findMixInClassFor(cls);
/* 548 */       _addMemberMethods(cls, methodFilter, this._memberMethods, mixin, mixins);
/*     */     }
/*     */     
/* 551 */     if (this._mixInResolver != null) {
/* 552 */       Class<?> mixin = this._mixInResolver.findMixInClassFor(Object.class);
/* 553 */       if (mixin != null) {
/* 554 */         _addMethodMixIns(methodFilter, this._memberMethods, mixin, mixins);
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 564 */     if (this._annotationIntrospector != null) {
/* 565 */       if (!mixins.isEmpty()) {
/* 566 */         Iterator<AnnotatedMethod> it = mixins.iterator();
/* 567 */         while (it.hasNext()) {
/* 568 */           AnnotatedMethod mixIn = (AnnotatedMethod)it.next();
/*     */           try {
/* 570 */             Method m = Object.class.getDeclaredMethod(mixIn.getName(), mixIn.getParameterClasses());
/* 571 */             if (m != null) {
/* 572 */               AnnotatedMethod am = _constructMethod(m);
/* 573 */               _addMixOvers(mixIn.getAnnotated(), am, false);
/* 574 */               this._memberMethods.add(am);
/*     */             }
/*     */           }
/*     */           catch (Exception e) {}
/*     */         }
/*     */       }
/*     */       
/*     */ 
/*     */ 
/*     */ 
/* 584 */       Iterator<AnnotatedMethod> it = this._memberMethods.iterator();
/* 585 */       while (it.hasNext()) {
/* 586 */         AnnotatedMethod am = (AnnotatedMethod)it.next();
/* 587 */         if (this._annotationIntrospector.isIgnorableMethod(am)) {
/* 588 */           it.remove();
/* 589 */           if (collectIgnored) {
/* 590 */             this._ignoredMethods = org.codehaus.jackson.map.util.ArrayBuilders.addToList(this._ignoredMethods, am);
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void _addMemberMethods(Class<?> cls, MethodFilter methodFilter, AnnotatedMethodMap methods, Class<?> mixInCls, AnnotatedMethodMap mixIns)
/*     */   {
/* 602 */     if (mixInCls != null) {
/* 603 */       _addMethodMixIns(methodFilter, methods, mixInCls, mixIns);
/*     */     }
/*     */     
/* 606 */     if (cls == null) {
/* 607 */       return;
/*     */     }
/*     */     
/* 610 */     for (Method m : cls.getDeclaredMethods()) {
/* 611 */       if (_isIncludableMethod(m, methodFilter))
/*     */       {
/*     */ 
/* 614 */         AnnotatedMethod old = methods.find(m);
/* 615 */         if (old == null) {
/* 616 */           AnnotatedMethod newM = _constructMethod(m);
/* 617 */           methods.add(newM);
/*     */           
/* 619 */           old = mixIns.remove(m);
/* 620 */           if (old != null) {
/* 621 */             _addMixOvers(old.getAnnotated(), newM, false);
/*     */           }
/*     */           
/*     */         }
/*     */         else
/*     */         {
/* 627 */           _addMixUnders(m, old);
/*     */           
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 636 */           if ((old.getDeclaringClass().isInterface()) && (!m.getDeclaringClass().isInterface())) {
/* 637 */             methods.add(old.withMethod(m));
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   protected void _addMethodMixIns(MethodFilter methodFilter, AnnotatedMethodMap methods, Class<?> mixInCls, AnnotatedMethodMap mixIns)
/*     */   {
/* 646 */     for (Method m : mixInCls.getDeclaredMethods()) {
/* 647 */       if (_isIncludableMethod(m, methodFilter))
/*     */       {
/*     */ 
/* 650 */         AnnotatedMethod am = methods.find(m);
/*     */         
/*     */ 
/*     */ 
/*     */ 
/* 655 */         if (am != null) {
/* 656 */           _addMixUnders(m, am);
/*     */ 
/*     */         }
/*     */         else
/*     */         {
/*     */ 
/* 662 */           mixIns.add(_constructMethod(m));
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void resolveFields(boolean collectIgnored)
/*     */   {
/* 682 */     java.util.LinkedHashMap<String, AnnotatedField> foundFields = new java.util.LinkedHashMap();
/* 683 */     _addFields(foundFields, this._class);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 688 */     if (this._annotationIntrospector != null) {
/* 689 */       Iterator<java.util.Map.Entry<String, AnnotatedField>> it = foundFields.entrySet().iterator();
/* 690 */       while (it.hasNext()) {
/* 691 */         AnnotatedField f = (AnnotatedField)((java.util.Map.Entry)it.next()).getValue();
/* 692 */         if (this._annotationIntrospector.isIgnorableField(f)) {
/* 693 */           it.remove();
/* 694 */           if (collectIgnored) {
/* 695 */             this._ignoredFields = org.codehaus.jackson.map.util.ArrayBuilders.addToList(this._ignoredFields, f);
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/* 700 */     if (foundFields.isEmpty()) {
/* 701 */       this._fields = Collections.emptyList();
/*     */     } else {
/* 703 */       this._fields = new java.util.ArrayList(foundFields.size());
/* 704 */       this._fields.addAll(foundFields.values());
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void _addFields(Map<String, AnnotatedField> fields, Class<?> c)
/*     */   {
/* 715 */     Class<?> parent = c.getSuperclass();
/* 716 */     if (parent != null)
/*     */     {
/*     */ 
/*     */ 
/*     */ 
/* 721 */       _addFields(fields, parent);
/* 722 */       for (Field f : c.getDeclaredFields())
/*     */       {
/* 724 */         if (_isIncludableField(f))
/*     */         {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 732 */           fields.put(f.getName(), _constructField(f));
/*     */         }
/*     */       }
/* 735 */       if (this._mixInResolver != null) {
/* 736 */         Class<?> mixin = this._mixInResolver.findMixInClassFor(c);
/* 737 */         if (mixin != null) {
/* 738 */           _addFieldMixIns(mixin, fields);
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void _addFieldMixIns(Class<?> mixin, Map<String, AnnotatedField> fields)
/*     */   {
/* 751 */     for (Field mixinField : mixin.getDeclaredFields())
/*     */     {
/*     */ 
/*     */ 
/* 755 */       if (_isIncludableField(mixinField))
/*     */       {
/*     */ 
/* 758 */         String name = mixinField.getName();
/*     */         
/* 760 */         AnnotatedField maskedField = (AnnotatedField)fields.get(name);
/* 761 */         if (maskedField != null) {
/* 762 */           for (Annotation a : mixinField.getDeclaredAnnotations()) {
/* 763 */             if (this._annotationIntrospector.isHandled(a)) {
/* 764 */               maskedField.addOrOverride(a);
/*     */             }
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected AnnotatedMethod _constructMethod(Method m)
/*     */   {
/* 783 */     if (this._annotationIntrospector == null) {
/* 784 */       return new AnnotatedMethod(m, _emptyAnnotationMap(), null);
/*     */     }
/* 786 */     return new AnnotatedMethod(m, _collectRelevantAnnotations(m.getDeclaredAnnotations()), null);
/*     */   }
/*     */   
/*     */   protected AnnotatedConstructor _constructConstructor(Constructor<?> ctor, boolean defaultCtor)
/*     */   {
/* 791 */     if (this._annotationIntrospector == null) {
/* 792 */       return new AnnotatedConstructor(ctor, _emptyAnnotationMap(), _emptyAnnotationMaps(ctor.getParameterTypes().length));
/*     */     }
/* 794 */     if (defaultCtor) {
/* 795 */       return new AnnotatedConstructor(ctor, _collectRelevantAnnotations(ctor.getDeclaredAnnotations()), null);
/*     */     }
/* 797 */     Annotation[][] paramAnns = ctor.getParameterAnnotations();
/* 798 */     int paramCount = ctor.getParameterTypes().length;
/*     */     
/*     */ 
/*     */     AnnotationMap[] resolvedAnnotations;
/*     */     
/*     */ 
/* 804 */     if (paramCount != paramAnns.length)
/*     */     {
/*     */       AnnotationMap[] resolvedAnnotations;
/* 807 */       if ((ctor.getDeclaringClass().isMemberClass()) && (paramCount == paramAnns.length + 1))
/*     */       {
/* 809 */         Annotation[][] old = paramAnns;
/* 810 */         paramAnns = new Annotation[old.length + 1][];
/* 811 */         System.arraycopy(old, 0, paramAnns, 1, old.length);
/* 812 */         resolvedAnnotations = _collectRelevantAnnotations(paramAnns);
/*     */       } else {
/* 814 */         throw new IllegalStateException("Internal error: constructor for " + ctor.getDeclaringClass().getName() + " has mismatch: " + paramCount + " parameters; " + paramAnns.length + " sets of annotations");
/*     */       }
/*     */     }
/*     */     else {
/* 818 */       resolvedAnnotations = _collectRelevantAnnotations(paramAnns);
/*     */     }
/* 820 */     return new AnnotatedConstructor(ctor, _collectRelevantAnnotations(ctor.getDeclaredAnnotations()), resolvedAnnotations);
/*     */   }
/*     */   
/*     */ 
/*     */   protected AnnotatedMethod _constructCreatorMethod(Method m)
/*     */   {
/* 826 */     if (this._annotationIntrospector == null) {
/* 827 */       return new AnnotatedMethod(m, _emptyAnnotationMap(), _emptyAnnotationMaps(m.getParameterTypes().length));
/*     */     }
/* 829 */     return new AnnotatedMethod(m, _collectRelevantAnnotations(m.getDeclaredAnnotations()), _collectRelevantAnnotations(m.getParameterAnnotations()));
/*     */   }
/*     */   
/*     */ 
/*     */   protected AnnotatedField _constructField(Field f)
/*     */   {
/* 835 */     if (this._annotationIntrospector == null) {
/* 836 */       return new AnnotatedField(f, _emptyAnnotationMap());
/*     */     }
/* 838 */     return new AnnotatedField(f, _collectRelevantAnnotations(f.getDeclaredAnnotations()));
/*     */   }
/*     */   
/*     */   protected AnnotationMap[] _collectRelevantAnnotations(Annotation[][] anns)
/*     */   {
/* 843 */     int len = anns.length;
/* 844 */     AnnotationMap[] result = new AnnotationMap[len];
/* 845 */     for (int i = 0; i < len; i++) {
/* 846 */       result[i] = _collectRelevantAnnotations(anns[i]);
/*     */     }
/* 848 */     return result;
/*     */   }
/*     */   
/*     */   protected AnnotationMap _collectRelevantAnnotations(Annotation[] anns)
/*     */   {
/* 853 */     AnnotationMap annMap = new AnnotationMap();
/* 854 */     if (anns != null) {
/* 855 */       for (Annotation a : anns) {
/* 856 */         if (this._annotationIntrospector.isHandled(a)) {
/* 857 */           annMap.add(a);
/*     */         }
/*     */       }
/*     */     }
/* 861 */     return annMap;
/*     */   }
/*     */   
/*     */   private AnnotationMap _emptyAnnotationMap() {
/* 865 */     return new AnnotationMap();
/*     */   }
/*     */   
/*     */   private AnnotationMap[] _emptyAnnotationMaps(int count) {
/* 869 */     if (count == 0) {
/* 870 */       return NO_ANNOTATION_MAPS;
/*     */     }
/* 872 */     AnnotationMap[] maps = new AnnotationMap[count];
/* 873 */     for (int i = 0; i < count; i++) {
/* 874 */       maps[i] = _emptyAnnotationMap();
/*     */     }
/* 876 */     return maps;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected boolean _isIncludableMethod(Method m, MethodFilter filter)
/*     */   {
/* 887 */     if ((filter != null) && (!filter.includeMethod(m))) {
/* 888 */       return false;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 894 */     if ((m.isSynthetic()) || (m.isBridge())) {
/* 895 */       return false;
/*     */     }
/* 897 */     return true;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   private boolean _isIncludableField(Field f)
/*     */   {
/* 905 */     if (f.isSynthetic()) {
/* 906 */       return false;
/*     */     }
/*     */     
/* 909 */     int mods = f.getModifiers();
/* 910 */     if ((java.lang.reflect.Modifier.isStatic(mods)) || (java.lang.reflect.Modifier.isTransient(mods))) {
/* 911 */       return false;
/*     */     }
/* 913 */     return true;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void _addMixOvers(Constructor<?> mixin, AnnotatedConstructor target, boolean addParamAnnotations)
/*     */   {
/* 929 */     for (Annotation a : mixin.getDeclaredAnnotations()) {
/* 930 */       if (this._annotationIntrospector.isHandled(a)) {
/* 931 */         target.addOrOverride(a);
/*     */       }
/*     */     }
/* 934 */     if (addParamAnnotations) {
/* 935 */       Annotation[][] pa = mixin.getParameterAnnotations();
/* 936 */       int i = 0; for (int len = pa.length; i < len; i++) {
/* 937 */         for (Annotation a : pa[i]) {
/* 938 */           target.addOrOverrideParam(i, a);
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void _addMixOvers(Method mixin, AnnotatedMethod target, boolean addParamAnnotations)
/*     */   {
/* 951 */     for (Annotation a : mixin.getDeclaredAnnotations()) {
/* 952 */       if (this._annotationIntrospector.isHandled(a)) {
/* 953 */         target.addOrOverride(a);
/*     */       }
/*     */     }
/* 956 */     if (addParamAnnotations) {
/* 957 */       Annotation[][] pa = mixin.getParameterAnnotations();
/* 958 */       int i = 0; for (int len = pa.length; i < len; i++) {
/* 959 */         for (Annotation a : pa[i]) {
/* 960 */           target.addOrOverrideParam(i, a);
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void _addMixUnders(Method src, AnnotatedMethod target)
/*     */   {
/* 972 */     for (Annotation a : src.getDeclaredAnnotations()) {
/* 973 */       if (this._annotationIntrospector.isHandled(a)) {
/* 974 */         target.addIfNotPresent(a);
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public String toString()
/*     */   {
/* 988 */     return "[AnnotedClass " + this._class.getName() + "]";
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\introspect\AnnotatedClass.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */