/*     */ package org.codehaus.jackson.node;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.math.BigDecimal;
/*     */ import java.math.BigInteger;
/*     */ import org.codehaus.jackson.Base64Variant;
/*     */ import org.codehaus.jackson.JsonLocation;
/*     */ import org.codehaus.jackson.JsonNode;
/*     */ import org.codehaus.jackson.JsonParseException;
/*     */ import org.codehaus.jackson.JsonParser;
/*     */ import org.codehaus.jackson.JsonParser.NumberType;
/*     */ import org.codehaus.jackson.JsonStreamContext;
/*     */ import org.codehaus.jackson.JsonToken;
/*     */ import org.codehaus.jackson.ObjectCodec;
/*     */ import org.codehaus.jackson.impl.JsonParserMinimalBase;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class TreeTraversingParser
/*     */   extends JsonParserMinimalBase
/*     */ {
/*     */   protected ObjectCodec _objectCodec;
/*     */   protected NodeCursor _nodeCursor;
/*     */   protected JsonToken _nextToken;
/*     */   protected boolean _startContainer;
/*     */   protected boolean _closed;
/*     */   
/*     */   public TreeTraversingParser(JsonNode n)
/*     */   {
/*  66 */     this(n, null);
/*     */   }
/*     */   
/*     */   public TreeTraversingParser(JsonNode n, ObjectCodec codec) {
/*  70 */     super(0);
/*  71 */     this._objectCodec = codec;
/*  72 */     if (n.isArray()) {
/*  73 */       this._nextToken = JsonToken.START_ARRAY;
/*  74 */       this._nodeCursor = new NodeCursor.Array(n, null);
/*  75 */     } else if (n.isObject()) {
/*  76 */       this._nextToken = JsonToken.START_OBJECT;
/*  77 */       this._nodeCursor = new NodeCursor.Object(n, null);
/*     */     } else {
/*  79 */       this._nodeCursor = new NodeCursor.RootValue(n, null);
/*     */     }
/*     */   }
/*     */   
/*     */   public void setCodec(ObjectCodec c)
/*     */   {
/*  85 */     this._objectCodec = c;
/*     */   }
/*     */   
/*     */   public ObjectCodec getCodec()
/*     */   {
/*  90 */     return this._objectCodec;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void close()
/*     */     throws IOException
/*     */   {
/* 102 */     if (!this._closed) {
/* 103 */       this._closed = true;
/* 104 */       this._nodeCursor = null;
/* 105 */       this._currToken = null;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonToken nextToken()
/*     */     throws IOException, JsonParseException
/*     */   {
/* 118 */     if (this._nextToken != null) {
/* 119 */       this._currToken = this._nextToken;
/* 120 */       this._nextToken = null;
/* 121 */       return this._currToken;
/*     */     }
/*     */     
/* 124 */     if (this._startContainer) {
/* 125 */       this._startContainer = false;
/*     */       
/* 127 */       if (!this._nodeCursor.currentHasChildren()) {
/* 128 */         this._currToken = (this._currToken == JsonToken.START_OBJECT ? JsonToken.END_OBJECT : JsonToken.END_ARRAY);
/*     */         
/* 130 */         return this._currToken;
/*     */       }
/* 132 */       this._nodeCursor = this._nodeCursor.iterateChildren();
/* 133 */       this._currToken = this._nodeCursor.nextToken();
/* 134 */       if ((this._currToken == JsonToken.START_OBJECT) || (this._currToken == JsonToken.START_ARRAY)) {
/* 135 */         this._startContainer = true;
/*     */       }
/* 137 */       return this._currToken;
/*     */     }
/*     */     
/* 140 */     if (this._nodeCursor == null) {
/* 141 */       this._closed = true;
/* 142 */       return null;
/*     */     }
/*     */     
/* 145 */     this._currToken = this._nodeCursor.nextToken();
/* 146 */     if (this._currToken != null) {
/* 147 */       if ((this._currToken == JsonToken.START_OBJECT) || (this._currToken == JsonToken.START_ARRAY)) {
/* 148 */         this._startContainer = true;
/*     */       }
/* 150 */       return this._currToken;
/*     */     }
/*     */     
/* 153 */     this._currToken = this._nodeCursor.endToken();
/* 154 */     this._nodeCursor = this._nodeCursor.getParent();
/* 155 */     return this._currToken;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonParser skipChildren()
/*     */     throws IOException, JsonParseException
/*     */   {
/* 164 */     if (this._currToken == JsonToken.START_OBJECT) {
/* 165 */       this._startContainer = false;
/* 166 */       this._currToken = JsonToken.END_OBJECT;
/* 167 */     } else if (this._currToken == JsonToken.START_ARRAY) {
/* 168 */       this._startContainer = false;
/* 169 */       this._currToken = JsonToken.END_ARRAY;
/*     */     }
/* 171 */     return this;
/*     */   }
/*     */   
/*     */   public boolean isClosed()
/*     */   {
/* 176 */     return this._closed;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public String getCurrentName()
/*     */   {
/* 187 */     return this._nodeCursor == null ? null : this._nodeCursor.getCurrentName();
/*     */   }
/*     */   
/*     */   public JsonStreamContext getParsingContext()
/*     */   {
/* 192 */     return this._nodeCursor;
/*     */   }
/*     */   
/*     */   public JsonLocation getTokenLocation()
/*     */   {
/* 197 */     return JsonLocation.NA;
/*     */   }
/*     */   
/*     */   public JsonLocation getCurrentLocation()
/*     */   {
/* 202 */     return JsonLocation.NA;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public String getText()
/*     */   {
/* 214 */     if (this._closed) {
/* 215 */       return null;
/*     */     }
/*     */     
/* 218 */     switch (this._currToken) {
/*     */     case FIELD_NAME: 
/* 220 */       return this._nodeCursor.getCurrentName();
/*     */     case VALUE_STRING: 
/* 222 */       return currentNode().getTextValue();
/*     */     case VALUE_NUMBER_INT: 
/*     */     case VALUE_NUMBER_FLOAT: 
/* 225 */       return String.valueOf(currentNode().getNumberValue());
/*     */     case VALUE_EMBEDDED_OBJECT: 
/* 227 */       JsonNode n = currentNode();
/* 228 */       if ((n != null) && (n.isBinary()))
/*     */       {
/* 230 */         return n.getValueAsText();
/*     */       }
/*     */       break;
/*     */     }
/* 234 */     return this._currToken == null ? null : this._currToken.asString();
/*     */   }
/*     */   
/*     */   public char[] getTextCharacters() throws IOException, JsonParseException
/*     */   {
/* 239 */     return getText().toCharArray();
/*     */   }
/*     */   
/*     */   public int getTextLength() throws IOException, JsonParseException
/*     */   {
/* 244 */     return getText().length();
/*     */   }
/*     */   
/*     */   public int getTextOffset() throws IOException, JsonParseException
/*     */   {
/* 249 */     return 0;
/*     */   }
/*     */   
/*     */ 
/*     */   public boolean hasTextCharacters()
/*     */   {
/* 255 */     return false;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonParser.NumberType getNumberType()
/*     */     throws IOException, JsonParseException
/*     */   {
/* 268 */     JsonNode n = currentNumericNode();
/* 269 */     return n == null ? null : n.getNumberType();
/*     */   }
/*     */   
/*     */   public BigInteger getBigIntegerValue()
/*     */     throws IOException, JsonParseException
/*     */   {
/* 275 */     return currentNumericNode().getBigIntegerValue();
/*     */   }
/*     */   
/*     */   public BigDecimal getDecimalValue() throws IOException, JsonParseException
/*     */   {
/* 280 */     return currentNumericNode().getDecimalValue();
/*     */   }
/*     */   
/*     */   public double getDoubleValue() throws IOException, JsonParseException
/*     */   {
/* 285 */     return currentNumericNode().getDoubleValue();
/*     */   }
/*     */   
/*     */   public float getFloatValue() throws IOException, JsonParseException
/*     */   {
/* 290 */     return (float)currentNumericNode().getDoubleValue();
/*     */   }
/*     */   
/*     */   public long getLongValue() throws IOException, JsonParseException
/*     */   {
/* 295 */     return currentNumericNode().getLongValue();
/*     */   }
/*     */   
/*     */   public int getIntValue() throws IOException, JsonParseException
/*     */   {
/* 300 */     return currentNumericNode().getIntValue();
/*     */   }
/*     */   
/*     */   public Number getNumberValue() throws IOException, JsonParseException
/*     */   {
/* 305 */     return currentNumericNode().getNumberValue();
/*     */   }
/*     */   
/*     */ 
/*     */   public Object getEmbeddedObject()
/*     */   {
/* 311 */     if (!this._closed) {
/* 312 */       JsonNode n = currentNode();
/* 313 */       if (n != null) {
/* 314 */         if (n.isPojo()) {
/* 315 */           return ((POJONode)n).getPojo();
/*     */         }
/* 317 */         if (n.isBinary()) {
/* 318 */           return ((BinaryNode)n).getBinaryValue();
/*     */         }
/*     */       }
/*     */     }
/* 322 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public byte[] getBinaryValue(Base64Variant b64variant)
/*     */     throws IOException, JsonParseException
/*     */   {
/* 336 */     JsonNode n = currentNode();
/* 337 */     if (n != null) {
/* 338 */       byte[] data = n.getBinaryValue();
/*     */       
/* 340 */       if (data != null) {
/* 341 */         return data;
/*     */       }
/*     */       
/* 344 */       if (n.isPojo()) {
/* 345 */         Object ob = ((POJONode)n).getPojo();
/* 346 */         if ((ob instanceof byte[])) {
/* 347 */           return (byte[])ob;
/*     */         }
/*     */       }
/*     */     }
/*     */     
/* 352 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonNode currentNode()
/*     */   {
/* 362 */     if ((this._closed) || (this._nodeCursor == null)) {
/* 363 */       return null;
/*     */     }
/* 365 */     return this._nodeCursor.currentNode();
/*     */   }
/*     */   
/*     */   protected JsonNode currentNumericNode()
/*     */     throws JsonParseException
/*     */   {
/* 371 */     JsonNode n = currentNode();
/* 372 */     if ((n == null) || (!n.isNumber())) {
/* 373 */       JsonToken t = n == null ? null : n.asToken();
/* 374 */       throw _constructError("Current token (" + t + ") not numeric, can not use numeric value accessors");
/*     */     }
/* 376 */     return n;
/*     */   }
/*     */   
/*     */   protected void _handleEOF() throws JsonParseException
/*     */   {
/* 381 */     _throwInternal();
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\node\TreeTraversingParser.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */