/*     */ package org.codehaus.jackson.util;
/*     */ 
/*     */ import java.math.BigDecimal;
/*     */ import java.util.ArrayList;
/*     */ import org.codehaus.jackson.io.NumberInput;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public final class TextBuffer
/*     */ {
/*  28 */   static final char[] NO_CHARS = new char[0];
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   static final int MIN_SEGMENT_LEN = 1000;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   static final int MAX_SEGMENT_LEN = 262144;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private final BufferRecycler _allocator;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private char[] _inputBuffer;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private int _inputStart;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private int _inputLen;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private ArrayList<char[]> _segments;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  84 */   private boolean _hasSegments = false;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private int _segmentSize;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private char[] _currentSegment;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private int _currentSize;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private String _resultString;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private char[] _resultArray;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public TextBuffer(BufferRecycler allocator)
/*     */   {
/* 122 */     this._allocator = allocator;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void releaseBuffers()
/*     */   {
/* 136 */     if (this._allocator == null) {
/* 137 */       resetWithEmpty();
/*     */     }
/* 139 */     else if (this._currentSegment != null)
/*     */     {
/* 141 */       resetWithEmpty();
/*     */       
/* 143 */       char[] buf = this._currentSegment;
/* 144 */       this._currentSegment = null;
/* 145 */       this._allocator.releaseCharBuffer(BufferRecycler.CharBufferType.TEXT_BUFFER, buf);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void resetWithEmpty()
/*     */   {
/* 156 */     this._inputStart = -1;
/* 157 */     this._currentSize = 0;
/* 158 */     this._inputLen = 0;
/*     */     
/* 160 */     this._inputBuffer = null;
/* 161 */     this._resultString = null;
/* 162 */     this._resultArray = null;
/*     */     
/*     */ 
/* 165 */     if (this._hasSegments) {
/* 166 */       clearSegments();
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void resetWithShared(char[] buf, int start, int len)
/*     */   {
/* 179 */     this._resultString = null;
/* 180 */     this._resultArray = null;
/*     */     
/*     */ 
/* 183 */     this._inputBuffer = buf;
/* 184 */     this._inputStart = start;
/* 185 */     this._inputLen = len;
/*     */     
/*     */ 
/* 188 */     if (this._hasSegments) {
/* 189 */       clearSegments();
/*     */     }
/*     */   }
/*     */   
/*     */   public void resetWithCopy(char[] buf, int start, int len)
/*     */   {
/* 195 */     this._inputBuffer = null;
/* 196 */     this._inputStart = -1;
/* 197 */     this._inputLen = 0;
/*     */     
/* 199 */     this._resultString = null;
/* 200 */     this._resultArray = null;
/*     */     
/*     */ 
/* 203 */     if (this._hasSegments) {
/* 204 */       clearSegments();
/* 205 */     } else if (this._currentSegment == null) {
/* 206 */       this._currentSegment = findBuffer(len);
/*     */     }
/* 208 */     this._currentSize = (this._segmentSize = 0);
/* 209 */     append(buf, start, len);
/*     */   }
/*     */   
/*     */   public void resetWithString(String value)
/*     */   {
/* 214 */     this._inputBuffer = null;
/* 215 */     this._inputStart = -1;
/* 216 */     this._inputLen = 0;
/*     */     
/* 218 */     this._resultString = value;
/* 219 */     this._resultArray = null;
/*     */     
/* 221 */     if (this._hasSegments) {
/* 222 */       clearSegments();
/*     */     }
/* 224 */     this._currentSize = 0;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private final char[] findBuffer(int needed)
/*     */   {
/* 234 */     if (this._allocator != null) {
/* 235 */       return this._allocator.allocCharBuffer(BufferRecycler.CharBufferType.TEXT_BUFFER, needed);
/*     */     }
/* 237 */     return new char[Math.max(needed, 1000)];
/*     */   }
/*     */   
/*     */   private final void clearSegments()
/*     */   {
/* 242 */     this._hasSegments = false;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 250 */     this._segments.clear();
/* 251 */     this._currentSize = (this._segmentSize = 0);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int size()
/*     */   {
/* 264 */     if (this._inputStart >= 0) {
/* 265 */       return this._inputLen;
/*     */     }
/* 267 */     if (this._resultArray != null) {
/* 268 */       return this._resultArray.length;
/*     */     }
/* 270 */     if (this._resultString != null) {
/* 271 */       return this._resultString.length();
/*     */     }
/*     */     
/* 274 */     return this._segmentSize + this._currentSize;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int getTextOffset()
/*     */   {
/* 283 */     return this._inputStart >= 0 ? this._inputStart : 0;
/*     */   }
/*     */   
/*     */ 
/*     */   public char[] getTextBuffer()
/*     */   {
/* 289 */     if (this._inputStart >= 0) {
/* 290 */       return this._inputBuffer;
/*     */     }
/* 292 */     if (this._resultArray != null) {
/* 293 */       return this._resultArray;
/*     */     }
/* 295 */     if (this._resultString != null) {
/* 296 */       return this._resultArray = this._resultString.toCharArray();
/*     */     }
/*     */     
/* 299 */     if (!this._hasSegments) {
/* 300 */       return this._currentSegment;
/*     */     }
/*     */     
/* 303 */     return contentsAsArray();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public String contentsAsString()
/*     */   {
/* 314 */     if (this._resultString == null)
/*     */     {
/* 316 */       if (this._resultArray != null) {
/* 317 */         this._resultString = new String(this._resultArray);
/*     */ 
/*     */       }
/* 320 */       else if (this._inputStart >= 0) {
/* 321 */         if (this._inputLen < 1) {
/* 322 */           return this._resultString = "";
/*     */         }
/* 324 */         this._resultString = new String(this._inputBuffer, this._inputStart, this._inputLen);
/*     */       }
/*     */       else {
/* 327 */         int segLen = this._segmentSize;
/* 328 */         int currLen = this._currentSize;
/*     */         
/* 330 */         if (segLen == 0) {
/* 331 */           this._resultString = (currLen == 0 ? "" : new String(this._currentSegment, 0, currLen));
/*     */         } else {
/* 333 */           StringBuilder sb = new StringBuilder(segLen + currLen);
/*     */           
/* 335 */           if (this._segments != null) {
/* 336 */             int i = 0; for (int len = this._segments.size(); i < len; i++) {
/* 337 */               char[] curr = (char[])this._segments.get(i);
/* 338 */               sb.append(curr, 0, curr.length);
/*     */             }
/*     */           }
/*     */           
/* 342 */           sb.append(this._currentSegment, 0, this._currentSize);
/* 343 */           this._resultString = sb.toString();
/*     */         }
/*     */       }
/*     */     }
/*     */     
/* 348 */     return this._resultString;
/*     */   }
/*     */   
/*     */   public char[] contentsAsArray()
/*     */   {
/* 353 */     char[] result = this._resultArray;
/* 354 */     if (result == null) {
/* 355 */       this._resultArray = (result = buildResultArray());
/*     */     }
/* 357 */     return result;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public BigDecimal contentsAsDecimal()
/*     */     throws NumberFormatException
/*     */   {
/* 368 */     if (this._resultArray != null) {
/* 369 */       return new BigDecimal(this._resultArray);
/*     */     }
/*     */     
/* 372 */     if (this._inputStart >= 0) {
/* 373 */       return new BigDecimal(this._inputBuffer, this._inputStart, this._inputLen);
/*     */     }
/*     */     
/* 376 */     if (this._segmentSize == 0) {
/* 377 */       return new BigDecimal(this._currentSegment, 0, this._currentSize);
/*     */     }
/*     */     
/* 380 */     return new BigDecimal(contentsAsArray());
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public double contentsAsDouble()
/*     */     throws NumberFormatException
/*     */   {
/* 390 */     return NumberInput.parseDouble(contentsAsString());
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void ensureNotShared()
/*     */   {
/* 404 */     if (this._inputStart >= 0) {
/* 405 */       unshare(16);
/*     */     }
/*     */   }
/*     */   
/*     */   public void append(char c)
/*     */   {
/* 411 */     if (this._inputStart >= 0) {
/* 412 */       unshare(16);
/*     */     }
/* 414 */     this._resultString = null;
/* 415 */     this._resultArray = null;
/*     */     
/* 417 */     char[] curr = this._currentSegment;
/* 418 */     if (this._currentSize >= curr.length) {
/* 419 */       expand(1);
/* 420 */       curr = this._currentSegment;
/*     */     }
/* 422 */     curr[(this._currentSize++)] = c;
/*     */   }
/*     */   
/*     */ 
/*     */   public void append(char[] c, int start, int len)
/*     */   {
/* 428 */     if (this._inputStart >= 0) {
/* 429 */       unshare(len);
/*     */     }
/* 431 */     this._resultString = null;
/* 432 */     this._resultArray = null;
/*     */     
/*     */ 
/* 435 */     char[] curr = this._currentSegment;
/* 436 */     int max = curr.length - this._currentSize;
/*     */     
/* 438 */     if (max >= len) {
/* 439 */       System.arraycopy(c, start, curr, this._currentSize, len);
/* 440 */       this._currentSize += len;
/*     */     }
/*     */     else {
/* 443 */       if (max > 0) {
/* 444 */         System.arraycopy(c, start, curr, this._currentSize, max);
/* 445 */         start += max;
/* 446 */         len -= max;
/*     */       }
/*     */       
/*     */ 
/* 450 */       expand(len);
/* 451 */       System.arraycopy(c, start, this._currentSegment, 0, len);
/* 452 */       this._currentSize = len;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   public void append(String str, int offset, int len)
/*     */   {
/* 459 */     if (this._inputStart >= 0) {
/* 460 */       unshare(len);
/*     */     }
/* 462 */     this._resultString = null;
/* 463 */     this._resultArray = null;
/*     */     
/*     */ 
/* 466 */     char[] curr = this._currentSegment;
/* 467 */     int max = curr.length - this._currentSize;
/* 468 */     if (max >= len) {
/* 469 */       str.getChars(offset, offset + len, curr, this._currentSize);
/* 470 */       this._currentSize += len;
/*     */     }
/*     */     else {
/* 473 */       if (max > 0) {
/* 474 */         str.getChars(offset, offset + max, curr, this._currentSize);
/* 475 */         len -= max;
/* 476 */         offset += max;
/*     */       }
/*     */       
/*     */ 
/*     */ 
/* 481 */       expand(len);
/* 482 */       str.getChars(offset, offset + len, this._currentSegment, 0);
/* 483 */       this._currentSize = len;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public char[] getCurrentSegment()
/*     */   {
/* 499 */     if (this._inputStart >= 0) {
/* 500 */       unshare(1);
/*     */     } else {
/* 502 */       char[] curr = this._currentSegment;
/* 503 */       if (curr == null) {
/* 504 */         this._currentSegment = findBuffer(0);
/* 505 */       } else if (this._currentSize >= curr.length)
/*     */       {
/* 507 */         expand(1);
/*     */       }
/*     */     }
/* 510 */     return this._currentSegment;
/*     */   }
/*     */   
/*     */ 
/*     */   public final char[] emptyAndGetCurrentSegment()
/*     */   {
/* 516 */     this._inputStart = -1;
/* 517 */     this._currentSize = 0;
/* 518 */     this._inputLen = 0;
/*     */     
/* 520 */     this._inputBuffer = null;
/* 521 */     this._resultString = null;
/* 522 */     this._resultArray = null;
/*     */     
/*     */ 
/* 525 */     if (this._hasSegments) {
/* 526 */       clearSegments();
/*     */     }
/* 528 */     char[] curr = this._currentSegment;
/* 529 */     if (curr == null) {
/* 530 */       this._currentSegment = (curr = findBuffer(0));
/*     */     }
/* 532 */     return curr;
/*     */   }
/*     */   
/*     */   public int getCurrentSegmentSize() {
/* 536 */     return this._currentSize;
/*     */   }
/*     */   
/*     */   public void setCurrentLength(int len) {
/* 540 */     this._currentSize = len;
/*     */   }
/*     */   
/*     */   public char[] finishCurrentSegment()
/*     */   {
/* 545 */     if (this._segments == null) {
/* 546 */       this._segments = new ArrayList();
/*     */     }
/* 548 */     this._hasSegments = true;
/* 549 */     this._segments.add(this._currentSegment);
/* 550 */     int oldLen = this._currentSegment.length;
/* 551 */     this._segmentSize += oldLen;
/*     */     
/* 553 */     int newLen = Math.min(oldLen + (oldLen >> 1), 262144);
/* 554 */     char[] curr = _charArray(newLen);
/* 555 */     this._currentSize = 0;
/* 556 */     this._currentSegment = curr;
/* 557 */     return curr;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public char[] expandCurrentSegment()
/*     */   {
/* 567 */     char[] curr = this._currentSegment;
/*     */     
/* 569 */     int len = curr.length;
/*     */     
/* 571 */     int newLen = len == 262144 ? 262145 : Math.min(262144, len + (len >> 1));
/*     */     
/* 573 */     this._currentSegment = _charArray(newLen);
/* 574 */     System.arraycopy(curr, 0, this._currentSegment, 0, len);
/* 575 */     return this._currentSegment;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public String toString()
/*     */   {
/* 591 */     return contentsAsString();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private void unshare(int needExtra)
/*     */   {
/* 606 */     int sharedLen = this._inputLen;
/* 607 */     this._inputLen = 0;
/* 608 */     char[] inputBuf = this._inputBuffer;
/* 609 */     this._inputBuffer = null;
/* 610 */     int start = this._inputStart;
/* 611 */     this._inputStart = -1;
/*     */     
/*     */ 
/* 614 */     int needed = sharedLen + needExtra;
/* 615 */     if ((this._currentSegment == null) || (needed > this._currentSegment.length)) {
/* 616 */       this._currentSegment = findBuffer(needed);
/*     */     }
/* 618 */     if (sharedLen > 0) {
/* 619 */       System.arraycopy(inputBuf, start, this._currentSegment, 0, sharedLen);
/*     */     }
/* 621 */     this._segmentSize = 0;
/* 622 */     this._currentSize = sharedLen;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private void expand(int minNewSegmentSize)
/*     */   {
/* 632 */     if (this._segments == null) {
/* 633 */       this._segments = new ArrayList();
/*     */     }
/* 635 */     char[] curr = this._currentSegment;
/* 636 */     this._hasSegments = true;
/* 637 */     this._segments.add(curr);
/* 638 */     this._segmentSize += curr.length;
/* 639 */     int oldLen = curr.length;
/*     */     
/* 641 */     int sizeAddition = oldLen >> 1;
/* 642 */     if (sizeAddition < minNewSegmentSize) {
/* 643 */       sizeAddition = minNewSegmentSize;
/*     */     }
/* 645 */     curr = _charArray(Math.min(262144, oldLen + sizeAddition));
/* 646 */     this._currentSize = 0;
/* 647 */     this._currentSegment = curr;
/*     */   }
/*     */   
/*     */   private char[] buildResultArray()
/*     */   {
/* 652 */     if (this._resultString != null) {
/* 653 */       return this._resultString.toCharArray();
/*     */     }
/*     */     
/*     */     char[] result;
/*     */     
/* 658 */     if (this._inputStart >= 0) {
/* 659 */       if (this._inputLen < 1) {
/* 660 */         return NO_CHARS;
/*     */       }
/* 662 */       char[] result = _charArray(this._inputLen);
/* 663 */       System.arraycopy(this._inputBuffer, this._inputStart, result, 0, this._inputLen);
/*     */     }
/*     */     else {
/* 666 */       int size = size();
/* 667 */       if (size < 1) {
/* 668 */         return NO_CHARS;
/*     */       }
/* 670 */       int offset = 0;
/* 671 */       result = _charArray(size);
/* 672 */       if (this._segments != null) {
/* 673 */         int i = 0; for (int len = this._segments.size(); i < len; i++) {
/* 674 */           char[] curr = (char[])this._segments.get(i);
/* 675 */           int currLen = curr.length;
/* 676 */           System.arraycopy(curr, 0, result, offset, currLen);
/* 677 */           offset += currLen;
/*     */         }
/*     */       }
/* 680 */       System.arraycopy(this._currentSegment, 0, result, offset, this._currentSize);
/*     */     }
/* 682 */     return result;
/*     */   }
/*     */   
/*     */   private final char[] _charArray(int len) {
/* 686 */     return new char[len];
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\util\TextBuffer.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */