/*     */ package org.codehaus.janino;
/*     */ 
/*     */ import java.io.FileInputStream;
/*     */ import java.io.IOException;
/*     */ import java.io.InputStream;
/*     */ import java.io.PrintStream;
/*     */ import java.io.Reader;
/*     */ import java.lang.reflect.Method;
/*     */ import java.security.AccessController;
/*     */ import java.security.PrivilegedAction;
/*     */ import java.util.HashMap;
/*     */ import java.util.Map;
/*     */ import org.codehaus.commons.compiler.CompileException;
/*     */ import org.codehaus.commons.compiler.Cookable;
/*     */ import org.codehaus.commons.compiler.ISimpleCompiler;
/*     */ import org.codehaus.commons.compiler.Location;
/*     */ import org.codehaus.janino.util.ClassFile;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class SimpleCompiler
/*     */   extends Cookable
/*     */   implements ISimpleCompiler
/*     */ {
/*     */   private static final boolean DEBUG = false;
/*  47 */   private ClassLoader parentClassLoader = Thread.currentThread().getContextClassLoader();
/*  48 */   private Class[] optionalAuxiliaryClasses = null;
/*     */   
/*     */ 
/*  51 */   private AuxiliaryClassLoader classLoader = null;
/*  52 */   private IClassLoader iClassLoader = null;
/*     */   
/*  54 */   private ClassLoader result = null;
/*     */   
/*  56 */   protected boolean debugSource = Boolean.getBoolean("org.codehaus.janino.source_debugging.enable");
/*  57 */   protected boolean debugLines = this.debugSource;
/*  58 */   protected boolean debugVars = this.debugSource;
/*     */   
/*     */   public static void main(String[] args) throws Exception {
/*  61 */     if ((args.length >= 1) && (args[0].equals("-help"))) {
/*  62 */       System.out.println("Usage:");
/*  63 */       System.out.println("    org.codehaus.janino.SimpleCompiler <source-file> <class-name> { <argument> }");
/*  64 */       System.out.println("Reads a compilation unit from the given <source-file> and invokes method");
/*  65 */       System.out.println("\"public static void main(String[])\" of class <class-name>, passing the.");
/*  66 */       System.out.println("given <argument>s.");
/*  67 */       System.exit(1);
/*     */     }
/*     */     
/*  70 */     if (args.length < 2) {
/*  71 */       System.err.println("Source file and/or class name missing; try \"-help\".");
/*  72 */       System.exit(1);
/*     */     }
/*     */     
/*     */ 
/*  76 */     String sourceFileName = args[0];
/*     */     
/*     */ 
/*  79 */     String className = args[1];
/*     */     
/*     */ 
/*  82 */     String[] arguments = new String[args.length - 2];
/*  83 */     System.arraycopy(args, 2, arguments, 0, arguments.length);
/*     */     
/*     */ 
/*  86 */     ClassLoader cl = new SimpleCompiler(sourceFileName, new FileInputStream(sourceFileName)).getClassLoader();
/*     */     
/*     */ 
/*  89 */     Class c = cl.loadClass(className);
/*     */     
/*     */ 
/*  92 */     Method m = c.getMethod("main", new Class[] { new String[0].getClass() });
/*  93 */     m.invoke(null, new Object[] { arguments });
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public SimpleCompiler(String optionalFileName, Reader in)
/*     */     throws IOException, CompileException
/*     */   {
/* 108 */     cook(optionalFileName, in);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public SimpleCompiler(String optionalFileName, InputStream is)
/*     */     throws IOException, CompileException
/*     */   {
/* 123 */     cook(optionalFileName, is);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public SimpleCompiler(String fileName)
/*     */     throws IOException, CompileException
/*     */   {
/* 137 */     cookFile(fileName);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public SimpleCompiler(Scanner scanner, ClassLoader optionalParentClassLoader)
/*     */     throws IOException, CompileException
/*     */   {
/* 154 */     setParentClassLoader(optionalParentClassLoader);
/* 155 */     cook(scanner);
/*     */   }
/*     */   
/*     */   public SimpleCompiler() {}
/*     */   
/*     */   public void setParentClassLoader(ClassLoader optionalParentClassLoader) {
/* 161 */     setParentClassLoader(optionalParentClassLoader, null);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void setParentClassLoader(ClassLoader optionalParentClassLoader, Class[] auxiliaryClasses)
/*     */   {
/* 175 */     assertNotCooked();
/* 176 */     this.parentClassLoader = (optionalParentClassLoader != null ? optionalParentClassLoader : Thread.currentThread().getContextClassLoader());
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 181 */     this.optionalAuxiliaryClasses = auxiliaryClasses;
/*     */   }
/*     */   
/*     */   public void setDebuggingInformation(boolean debugSource, boolean debugLines, boolean debugVars) {
/* 185 */     this.debugSource = debugSource;
/* 186 */     this.debugLines = debugLines;
/* 187 */     this.debugVars = debugVars;
/*     */   }
/*     */   
/*     */ 
/*     */   public final void cook(String optionalFileName, Reader r)
/*     */     throws CompileException, IOException
/*     */   {
/* 194 */     cook(new Scanner(optionalFileName, r));
/*     */   }
/*     */   
/*     */   public void cook(Scanner scanner) throws CompileException, IOException {
/* 198 */     setUpClassLoaders();
/*     */     
/*     */ 
/* 201 */     Java.CompilationUnit compilationUnit = new Parser(scanner).parseCompilationUnit();
/*     */     
/*     */ 
/* 204 */     compileToClassLoader(compilationUnit);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public void cook(Java.CompilationUnit compilationUnit)
/*     */     throws CompileException
/*     */   {
/* 212 */     setUpClassLoaders();
/*     */     
/*     */ 
/* 215 */     compileToClassLoader(compilationUnit);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final void setUpClassLoaders()
/*     */   {
/* 224 */     assertNotCooked();
/*     */     
/*     */ 
/* 227 */     this.classLoader = ((AuxiliaryClassLoader)AccessController.doPrivileged(new PrivilegedAction() {
/*     */       public Object run() {
/* 229 */         return new SimpleCompiler.AuxiliaryClassLoader(SimpleCompiler.this.parentClassLoader, null);
/*     */       }
/*     */     }));
/* 232 */     if (this.optionalAuxiliaryClasses != null) {
/* 233 */       for (int i = 0; i < this.optionalAuxiliaryClasses.length; i++) {
/* 234 */         this.classLoader.addAuxiliaryClass(this.optionalAuxiliaryClasses[i]);
/*     */       }
/*     */     }
/*     */     
/* 238 */     this.iClassLoader = new ClassLoaderIClassLoader(this.classLoader);
/*     */   }
/*     */   
/*     */ 
/*     */   private static final class AuxiliaryClassLoader
/*     */     extends ClassLoader
/*     */   {
/* 245 */     AuxiliaryClassLoader(ClassLoader x0, SimpleCompiler.1 x1) { this(x0); }
/* 246 */     private final Map auxiliaryClasses = new HashMap();
/*     */     
/*     */     private AuxiliaryClassLoader(ClassLoader parent) {
/* 249 */       super();
/*     */     }
/*     */     
/*     */     protected Class loadClass(String name, boolean resolve) throws ClassNotFoundException {
/* 253 */       Class c = (Class)this.auxiliaryClasses.get(name);
/* 254 */       if (c != null) { return c;
/*     */       }
/* 256 */       return super.loadClass(name, resolve);
/*     */     }
/*     */     
/*     */     private void addAuxiliaryClass(Class c) {
/* 260 */       if (this.auxiliaryClasses.containsKey(c.getName())) { return;
/*     */       }
/*     */       try
/*     */       {
/* 264 */         Class c2 = super.loadClass(c.getName(), false);
/* 265 */         if (c2 != c) {
/* 266 */           throw new JaninoRuntimeException("Trying to add an auxiliary class \"" + c.getName() + "\" while another class with the same name is already loaded");
/*     */         }
/*     */       }
/*     */       catch (ClassNotFoundException ex) {}
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 276 */       this.auxiliaryClasses.put(c.getName(), c);
/*     */       
/*     */ 
/* 279 */       Class sc = c.getSuperclass();
/* 280 */       if (sc != null) { addAuxiliaryClass(sc);
/*     */       }
/*     */       
/*     */ 
/* 284 */       Class[] ifs = c.getInterfaces();
/* 285 */       for (int i = 0; i < ifs.length; i++) addAuxiliaryClass(ifs[i]);
/*     */     }
/*     */     
/*     */     public boolean equals(Object o)
/*     */     {
/* 290 */       if (!(o instanceof AuxiliaryClassLoader)) return false;
/* 291 */       AuxiliaryClassLoader that = (AuxiliaryClassLoader)o;
/*     */       
/*     */ 
/* 294 */       ClassLoader parentOfThis = getParent();
/* 295 */       ClassLoader parentOfThat = that.getParent();
/* 296 */       if (parentOfThis == null ? parentOfThat != null : !parentOfThis.equals(parentOfThat)) { return false;
/*     */       }
/*     */       
/* 299 */       return this.auxiliaryClasses.equals(that.auxiliaryClasses);
/*     */     }
/*     */     
/*     */     public int hashCode() {
/* 303 */       ClassLoader parent = getParent();
/* 304 */       return (parent == null ? 0 : parent.hashCode()) ^ this.auxiliaryClasses.hashCode();
/*     */     }
/*     */   }
/*     */   
/*     */   public ClassLoader getClassLoader() {
/* 309 */     if (getClass() != SimpleCompiler.class) {
/* 310 */       throw new IllegalStateException("Must not be called on derived instances");
/*     */     }
/* 312 */     if (this.result == null) throw new IllegalStateException("Must only be called after \"cook()\"");
/* 313 */     return this.result;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public boolean equals(Object o)
/*     */   {
/* 324 */     if (!(o instanceof SimpleCompiler)) return false;
/* 325 */     SimpleCompiler that = (SimpleCompiler)o;
/* 326 */     if (getClass() != that.getClass()) return false;
/* 327 */     if ((this.result == null) || (that.result == null)) {
/* 328 */       throw new IllegalStateException("Equality can only be checked after cooking");
/*     */     }
/* 330 */     return this.result.equals(that.result);
/*     */   }
/*     */   
/*     */   public int hashCode() {
/* 334 */     return this.classLoader.hashCode();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected Java.Type classToType(Location location, Class optionalClass)
/*     */   {
/* 344 */     if (optionalClass == null) { return null;
/*     */     }
/* 346 */     this.classLoader.addAuxiliaryClass(optionalClass);
/*     */     IClass iClass;
/*     */     try
/*     */     {
/* 350 */       iClass = this.iClassLoader.loadIClass(Descriptor.fromClassName(optionalClass.getName()));
/*     */     } catch (ClassNotFoundException ex) {
/* 352 */       throw new JaninoRuntimeException("Loading IClass \"" + optionalClass.getName() + "\": " + ex);
/*     */     }
/* 354 */     if (iClass == null) {
/* 355 */       throw new JaninoRuntimeException("Cannot load class \"" + optionalClass.getName() + "\" through the given ClassLoader");
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 362 */     return new Java.SimpleType(location, iClass);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected Java.Type[] classesToTypes(Location location, Class[] classes)
/*     */   {
/* 372 */     Java.Type[] types = new Java.Type[classes.length];
/* 373 */     for (int i = 0; i < classes.length; i++) {
/* 374 */       types[i] = classToType(location, classes[i]);
/*     */     }
/* 376 */     return types;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final ClassLoader compileToClassLoader(Java.CompilationUnit compilationUnit)
/*     */     throws CompileException
/*     */   {
/* 393 */     ClassFile[] classFiles = new UnitCompiler(compilationUnit, this.iClassLoader).compileUnit(this.debugSource, this.debugLines, this.debugVars);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 399 */     Map classes = new HashMap();
/* 400 */     for (int i = 0; i < classFiles.length; i++) {
/* 401 */       ClassFile cf = classFiles[i];
/* 402 */       classes.put(cf.getThisClassName(), cf.toByteArray());
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 422 */     this.result = ((ClassLoader)AccessController.doPrivileged(new PrivilegedAction() { private final Map val$classes;
/*     */       
/* 424 */       public Object run() { return new ByteArrayClassLoader(this.val$classes, SimpleCompiler.this.classLoader);
/*     */       }
/*     */       
/*     */ 
/*     */ 
/* 429 */     }));
/* 430 */     return this.result;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   protected void assertNotCooked()
/*     */   {
/* 437 */     if (this.classLoader != null) throw new IllegalStateException("Already cooked");
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\janino\SimpleCompiler.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       0.7.1
 */