/*     */ package net.sf.sevenzipjbinding.impl;
/*     */ 
/*     */ import java.text.MessageFormat;
/*     */ import java.util.ArrayList;
/*     */ import java.util.List;
/*     */ import net.sf.sevenzipjbinding.IArchiveOpenVolumeCallback;
/*     */ import net.sf.sevenzipjbinding.IInStream;
/*     */ import net.sf.sevenzipjbinding.PropID;
/*     */ import net.sf.sevenzipjbinding.SevenZipException;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class VolumedArchiveInStream
/*     */   implements IInStream
/*     */ {
/*     */   private static final String SEVEN_ZIP_FIRST_VOLUME_POSTFIX = ".7z.001";
/*     */   private long absoluteOffset;
/*  36 */   private long absoluteLength = -1L;
/*     */   
/*  38 */   private int currentIndex = -1;
/*     */   private IInStream currentInStream;
/*     */   private long currentVolumeOffset;
/*     */   private long currentVolumeLength;
/*  42 */   private List<Long> volumePositions = new ArrayList();
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   private final IArchiveOpenVolumeCallback archiveOpenVolumeCallback;
/*     */   
/*     */ 
/*     */ 
/*     */   private String cuttedVolumeFilename;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public VolumedArchiveInStream(IArchiveOpenVolumeCallback paramIArchiveOpenVolumeCallback)
/*     */     throws SevenZipException
/*     */   {
/*  59 */     this((String)paramIArchiveOpenVolumeCallback.getProperty(PropID.NAME), paramIArchiveOpenVolumeCallback);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public VolumedArchiveInStream(String paramString, IArchiveOpenVolumeCallback paramIArchiveOpenVolumeCallback)
/*     */     throws SevenZipException
/*     */   {
/*  75 */     this.archiveOpenVolumeCallback = paramIArchiveOpenVolumeCallback;
/*  76 */     this.volumePositions.add(Long.valueOf(0L));
/*     */     
/*  78 */     if (!paramString.endsWith(".7z.001")) {
/*  79 */       throw new SevenZipException("The first 7z volume filename '" + paramString + "' don't ends with the postfix: '" + ".7z.001" + "'. Can't proceed");
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*  84 */     this.cuttedVolumeFilename = paramString.substring(0, paramString.length() - 3);
/*  85 */     openVolume(1, true);
/*     */   }
/*     */   
/*     */   private void openVolume(int paramInt, boolean paramBoolean) throws SevenZipException {
/*  89 */     if (this.currentIndex == paramInt) {
/*  90 */       return;
/*     */     }
/*     */     
/*  93 */     for (int i = this.volumePositions.size(); (i < paramInt) && (this.absoluteLength == -1L); i++) {
/*  94 */       openVolume(i, false);
/*     */     }
/*     */     
/*  97 */     if ((this.absoluteLength != -1L) && (this.volumePositions.size() <= paramInt)) {
/*  98 */       return;
/*     */     }
/*     */     
/* 101 */     String str = this.cuttedVolumeFilename + MessageFormat.format("{0,number,000}", new Object[] { Integer.valueOf(paramInt) });
/*     */     
/*     */ 
/* 104 */     IInStream localIInStream = this.archiveOpenVolumeCallback.getStream(str);
/*     */     
/* 106 */     if (localIInStream == null) {
/* 107 */       this.absoluteLength = ((Long)this.volumePositions.get(this.volumePositions.size() - 1)).longValue();
/* 108 */       return;
/*     */     }
/*     */     
/* 111 */     this.currentInStream = localIInStream;
/*     */     
/* 113 */     if (this.volumePositions.size() == paramInt)
/*     */     {
/* 115 */       this.currentVolumeLength = this.currentInStream.seek(0L, 2);
/* 116 */       if (this.currentVolumeLength == 0L) {
/* 117 */         throw new RuntimeException("Volume " + paramInt + " is empty");
/*     */       }
/* 119 */       this.volumePositions.add(Long.valueOf(((Long)this.volumePositions.get(paramInt - 1)).longValue() + this.currentVolumeLength));
/*     */       
/* 121 */       if (paramBoolean) {
/* 122 */         this.currentInStream.seek(0L, 0);
/*     */       }
/*     */     } else {
/* 125 */       this.currentVolumeLength = (((Long)this.volumePositions.get(paramInt)).longValue() - ((Long)this.volumePositions.get(paramInt - 1)).longValue());
/*     */     }
/*     */     
/* 128 */     if (paramBoolean) {
/* 129 */       this.currentVolumeOffset = 0L;
/* 130 */       this.absoluteOffset = ((Long)this.volumePositions.get(paramInt - 1)).longValue();
/*     */     }
/*     */     
/* 133 */     this.currentIndex = paramInt;
/*     */   }
/*     */   
/*     */ 
/*     */   private void openVolumeToAbsoluteOffset()
/*     */     throws SevenZipException
/*     */   {
/* 140 */     int i = this.volumePositions.size() - 1;
/* 141 */     if ((this.absoluteLength != -1L) && (this.absoluteOffset >= this.absoluteLength)) {
/* 142 */       return;
/*     */     }
/* 144 */     while (((Long)this.volumePositions.get(i)).longValue() > this.absoluteOffset) {
/* 145 */       i--;
/*     */     }
/*     */     
/* 148 */     if (i < this.volumePositions.size() - 1) {
/* 149 */       openVolume(i + 1, false);
/*     */     }
/*     */     else {
/*     */       do
/*     */       {
/* 154 */         i++;
/* 155 */         openVolume(i, false);
/*     */       }
/* 157 */       while (((this.absoluteLength == -1L) || (this.absoluteOffset < this.absoluteLength)) && (((Long)this.volumePositions.get(i)).longValue() <= this.absoluteOffset));
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public long seek(long paramLong, int paramInt)
/*     */     throws SevenZipException
/*     */   {
/* 167 */     int i = 0;
/* 168 */     long l; switch (paramInt) {
/*     */     case 0: 
/* 170 */       l = paramLong;
/* 171 */       break;
/*     */     
/*     */     case 1: 
/* 174 */       l = this.absoluteOffset + paramLong;
/* 175 */       break;
/*     */     
/*     */     case 2: 
/* 178 */       if (this.absoluteLength == -1L) {
/* 179 */         openVolume(Integer.MAX_VALUE, false);
/* 180 */         i = 1;
/*     */       }
/* 182 */       l = this.absoluteLength + paramLong;
/* 183 */       break;
/*     */     
/*     */     default: 
/* 186 */       throw new RuntimeException("Seek: unknown origin: " + paramInt);
/*     */     }
/*     */     
/* 189 */     if ((l == this.absoluteOffset) && (i == 0)) {
/* 190 */       return l;
/*     */     }
/* 192 */     this.absoluteOffset = l;
/*     */     
/* 194 */     openVolumeToAbsoluteOffset();
/*     */     
/* 196 */     if ((this.absoluteLength != -1L) && (this.absoluteLength <= this.absoluteOffset)) {
/* 197 */       this.absoluteOffset = this.absoluteLength;
/* 198 */       return this.absoluteLength;
/*     */     }
/*     */     
/* 201 */     this.currentVolumeOffset = (this.absoluteOffset - ((Long)this.volumePositions.get(this.currentIndex - 1)).longValue());
/* 202 */     this.currentInStream.seek(this.currentVolumeOffset, 0);
/*     */     
/* 204 */     return l;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public int read(byte[] paramArrayOfByte)
/*     */     throws SevenZipException
/*     */   {
/* 212 */     if ((this.absoluteLength != -1L) && (this.absoluteOffset >= this.absoluteLength)) {
/* 213 */       return 0;
/*     */     }
/*     */     
/* 216 */     int i = this.currentInStream.read(paramArrayOfByte);
/*     */     
/* 218 */     this.absoluteOffset += i;
/* 219 */     this.currentVolumeOffset += i;
/*     */     
/* 221 */     if (this.currentVolumeOffset >= this.currentVolumeLength) {
/* 222 */       openVolume(this.currentIndex + 1, true);
/*     */     }
/*     */     
/* 225 */     return i;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\sevenzipjbinding.jar!\net\sf\sevenzipjbinding\impl\VolumedArchiveInStream.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */