package prefuse.action.assignment;

import java.util.logging.Logger;
import prefuse.Visualization;
import prefuse.data.Table;
import prefuse.data.column.ColumnMetadata;
import prefuse.data.column.Columns;
import prefuse.data.tuple.TupleSet;
import prefuse.util.MathLib;
import prefuse.util.PrefuseLib;
import prefuse.visual.VisualItem;

public class DataSizeAction
  extends SizeAction
{
  protected String m_dataField;
  protected double m_baseSize = 1.0D;
  protected double m_sizeRange;
  protected int m_scale = 0;
  protected int m_bins = -1;
  protected boolean m_inferBounds = true;
  protected boolean m_is2DArea = true;
  protected double[] m_dist;
  
  public DataSizeAction(String paramString1, String paramString2)
  {
    super(paramString1);
    this.m_dataField = paramString2;
  }
  
  public DataSizeAction(String paramString1, String paramString2, int paramInt)
  {
    this(paramString1, paramString2, paramInt, 0);
  }
  
  public DataSizeAction(String paramString1, String paramString2, int paramInt1, int paramInt2)
  {
    super(paramString1);
    this.m_dataField = paramString2;
    setScale(paramInt2);
    setBinCount(paramInt1);
  }
  
  public String getDataField()
  {
    return this.m_dataField;
  }
  
  public void setDataField(String paramString)
  {
    this.m_dataField = paramString;
  }
  
  public int getScale()
  {
    return this.m_scale;
  }
  
  public void setScale(int paramInt)
  {
    if ((paramInt < 0) || (paramInt >= 4)) {
      throw new IllegalArgumentException("Unrecognized scale value: " + paramInt);
    }
    this.m_scale = paramInt;
  }
  
  public int getBinCount()
  {
    return this.m_bins;
  }
  
  public void setBinCount(int paramInt)
  {
    if ((this.m_scale == 3) && (paramInt <= 0)) {
      throw new IllegalArgumentException("The quantile scale can not be used without binning. Use a bin value greater than zero.");
    }
    this.m_bins = paramInt;
  }
  
  public boolean is2DArea()
  {
    return this.m_is2DArea;
  }
  
  public void setIs2DArea(boolean paramBoolean)
  {
    this.m_is2DArea = paramBoolean;
  }
  
  public double getBaseSize()
  {
    return this.m_baseSize;
  }
  
  public void setBaseSize(double paramDouble)
  {
    this.m_baseSize = paramDouble;
  }
  
  public void run(double paramDouble)
  {
    TupleSet localTupleSet = this.m_vis.getGroup(this.m_group);
    if (!(localTupleSet instanceof Table)) {
      return;
    }
    Table localTable = (Table)localTupleSet;
    if (!localTable.canGetDouble(this.m_dataField)) {
      return;
    }
    int i = this.m_scale;
    ColumnMetadata localColumnMetadata = localTable.getMetadata(this.m_dataField);
    if (this.m_inferBounds)
    {
      if ((this.m_scale == 3) && (this.m_bins > 0))
      {
        double[] arrayOfDouble = Columns.toDoubleArray(localTable, this.m_dataField);
        this.m_dist = MathLib.quantiles(this.m_bins, arrayOfDouble);
      }
      else
      {
        if (this.m_scale == 3)
        {
          Logger.getLogger(getClass().getName()).warning("Can't use quantile scale with no binning. Defaulting to linear scale. Set the bin value greater than zero to use a quantile scale.");
          this.m_scale = 0;
        }
        int j = localColumnMetadata.getMinimumRow();
        int k = localColumnMetadata.getMaximumRow();
        this.m_dist = new double[2];
        this.m_dist[0] = localTable.getDouble(j, this.m_dataField);
        this.m_dist[1] = localTable.getDouble(k, this.m_dataField);
      }
      this.m_sizeRange = (this.m_dist[(this.m_dist.length - 1)] / this.m_dist[0] - this.m_baseSize);
    }
    super.run(paramDouble);
    this.m_scale = i;
  }
  
  public double getSize(VisualItem paramVisualItem)
  {
    double d1 = paramVisualItem.getDouble(this.m_dataField);
    double d2 = MathLib.interp(this.m_scale, d1, this.m_dist);
    if (this.m_bins < 1)
    {
      d1 = this.m_baseSize + d2 * this.m_sizeRange;
    }
    else
    {
      int i = (int)((d2 - 1.0E-4D) * this.m_bins);
      d1 = this.m_baseSize + i * (this.m_sizeRange / (this.m_bins - 1));
    }
    return this.m_is2DArea ? PrefuseLib.getSize2D(d1) : d1;
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\action\assignment\DataSizeAction.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */