package prefuse.action.layout.graph;

import java.awt.geom.Rectangle2D;
import java.io.PrintStream;
import java.util.Iterator;
import java.util.Random;
import prefuse.Visualization;
import prefuse.action.layout.Layout;
import prefuse.data.Graph;
import prefuse.data.Schema;
import prefuse.data.tuple.TupleSet;
import prefuse.util.PrefuseLib;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;

public class FruchtermanReingoldLayout
  extends Layout
{
  private double forceConstant;
  private double temp;
  private int maxIter = 700;
  protected String m_nodeGroup;
  protected String m_edgeGroup;
  protected int m_fidx;
  private static final double EPSILON = 1.0E-6D;
  private static final double ALPHA = 0.1D;
  public static final String PARAMS = "_fruchtermanReingoldParams";
  public static final Schema PARAMS_SCHEMA = new Schema();
  
  public FruchtermanReingoldLayout(String paramString)
  {
    this(paramString, 700);
  }
  
  public FruchtermanReingoldLayout(String paramString, int paramInt)
  {
    super(paramString);
    this.m_nodeGroup = PrefuseLib.getGroupName(paramString, Graph.NODES);
    this.m_edgeGroup = PrefuseLib.getGroupName(paramString, Graph.EDGES);
    this.maxIter = paramInt;
  }
  
  public int getMaxIterations()
  {
    return this.maxIter;
  }
  
  public void setMaxIterations(int paramInt)
  {
    this.maxIter = paramInt;
  }
  
  public void run(double paramDouble)
  {
    Graph localGraph = (Graph)this.m_vis.getGroup(this.m_group);
    Rectangle2D localRectangle2D = super.getLayoutBounds();
    init(localGraph, localRectangle2D);
    for (int i = 0; i < this.maxIter; i++)
    {
      Iterator localIterator = localGraph.nodes();
      Object localObject;
      while (localIterator.hasNext())
      {
        localObject = (NodeItem)localIterator.next();
        if (!((NodeItem)localObject).isFixed()) {
          calcRepulsion(localGraph, (NodeItem)localObject);
        }
      }
      localIterator = localGraph.edges();
      while (localIterator.hasNext())
      {
        localObject = (EdgeItem)localIterator.next();
        calcAttraction((EdgeItem)localObject);
      }
      localIterator = localGraph.nodes();
      while (localIterator.hasNext())
      {
        localObject = (NodeItem)localIterator.next();
        if (!((NodeItem)localObject).isFixed()) {
          calcPositions((NodeItem)localObject, localRectangle2D);
        }
      }
      cool(i);
    }
    finish(localGraph);
  }
  
  private void init(Graph paramGraph, Rectangle2D paramRectangle2D)
  {
    initSchema(paramGraph.getNodes());
    this.temp = (paramRectangle2D.getWidth() / 10.0D);
    this.forceConstant = (0.75D * Math.sqrt(paramRectangle2D.getHeight() * paramRectangle2D.getWidth() / paramGraph.getNodeCount()));
    Iterator localIterator = paramGraph.nodes();
    Random localRandom = new Random(42L);
    double d1 = 0.1D * paramRectangle2D.getWidth() / 2.0D;
    double d2 = 0.1D * paramRectangle2D.getHeight() / 2.0D;
    while (localIterator.hasNext())
    {
      NodeItem localNodeItem = (NodeItem)localIterator.next();
      Params localParams = getParams(localNodeItem);
      localParams.loc[0] = (paramRectangle2D.getCenterX() + localRandom.nextDouble() * d1);
      localParams.loc[1] = (paramRectangle2D.getCenterY() + localRandom.nextDouble() * d2);
    }
  }
  
  private void finish(Graph paramGraph)
  {
    Iterator localIterator = paramGraph.nodes();
    while (localIterator.hasNext())
    {
      NodeItem localNodeItem = (NodeItem)localIterator.next();
      Params localParams = getParams(localNodeItem);
      setX(localNodeItem, null, localParams.loc[0]);
      setY(localNodeItem, null, localParams.loc[1]);
    }
  }
  
  public void calcPositions(NodeItem paramNodeItem, Rectangle2D paramRectangle2D)
  {
    Params localParams = getParams(paramNodeItem);
    double d1 = Math.max(1.0E-6D, Math.sqrt(localParams.disp[0] * localParams.disp[0] + localParams.disp[1] * localParams.disp[1]));
    double d2 = localParams.disp[0] / d1 * Math.min(d1, this.temp);
    if (Double.isNaN(d2)) {
      System.err.println("Mathematical error... (calcPositions:xDisp)");
    }
    double d3 = localParams.disp[1] / d1 * Math.min(d1, this.temp);
    localParams.loc[0] += d2;
    localParams.loc[1] += d3;
    double d4 = paramRectangle2D.getWidth() / 50.0D;
    double d5 = localParams.loc[0];
    if (d5 < paramRectangle2D.getMinX() + d4) {
      d5 = paramRectangle2D.getMinX() + d4 + Math.random() * d4 * 2.0D;
    } else if (d5 > paramRectangle2D.getMaxX() - d4) {
      d5 = paramRectangle2D.getMaxX() - d4 - Math.random() * d4 * 2.0D;
    }
    double d6 = localParams.loc[1];
    if (d6 < paramRectangle2D.getMinY() + d4) {
      d6 = paramRectangle2D.getMinY() + d4 + Math.random() * d4 * 2.0D;
    } else if (d6 > paramRectangle2D.getMaxY() - d4) {
      d6 = paramRectangle2D.getMaxY() - d4 - Math.random() * d4 * 2.0D;
    }
    localParams.loc[0] = d5;
    localParams.loc[1] = d6;
  }
  
  public void calcAttraction(EdgeItem paramEdgeItem)
  {
    NodeItem localNodeItem1 = paramEdgeItem.getSourceItem();
    Params localParams1 = getParams(localNodeItem1);
    NodeItem localNodeItem2 = paramEdgeItem.getTargetItem();
    Params localParams2 = getParams(localNodeItem2);
    double d1 = localParams1.loc[0] - localParams2.loc[0];
    double d2 = localParams1.loc[1] - localParams2.loc[1];
    double d3 = Math.max(1.0E-6D, Math.sqrt(d1 * d1 + d2 * d2));
    double d4 = d3 * d3 / this.forceConstant;
    if (Double.isNaN(d4)) {
      System.err.println("Mathematical error...");
    }
    double d5 = d1 / d3 * d4;
    double d6 = d2 / d3 * d4;
    localParams1.disp[0] -= d5;
    localParams1.disp[1] -= d6;
    localParams2.disp[0] += d5;
    localParams2.disp[1] += d6;
  }
  
  public void calcRepulsion(Graph paramGraph, NodeItem paramNodeItem)
  {
    Params localParams1 = getParams(paramNodeItem);
    localParams1.disp[0] = 0.0D;
    localParams1.disp[1] = 0.0D;
    Iterator localIterator = paramGraph.nodes();
    while (localIterator.hasNext())
    {
      NodeItem localNodeItem = (NodeItem)localIterator.next();
      Params localParams2 = getParams(localNodeItem);
      if (!localNodeItem.isFixed()) {
        if (paramNodeItem != localNodeItem)
        {
          double d1 = localParams1.loc[0] - localParams2.loc[0];
          double d2 = localParams1.loc[1] - localParams2.loc[1];
          double d3 = Math.max(1.0E-6D, Math.sqrt(d1 * d1 + d2 * d2));
          double d4 = this.forceConstant * this.forceConstant / d3;
          if (Double.isNaN(d4)) {
            System.err.println("Mathematical error...");
          }
          localParams1.disp[0] += d1 / d3 * d4;
          localParams1.disp[1] += d2 / d3 * d4;
        }
      }
    }
  }
  
  private void cool(int paramInt)
  {
    this.temp *= (1.0D - paramInt / this.maxIter);
  }
  
  protected void initSchema(TupleSet paramTupleSet)
  {
    try
    {
      paramTupleSet.addColumns(PARAMS_SCHEMA);
    }
    catch (IllegalArgumentException localIllegalArgumentException) {}
  }
  
  private Params getParams(VisualItem paramVisualItem)
  {
    Params localParams = (Params)paramVisualItem.get("_fruchtermanReingoldParams");
    if (localParams == null)
    {
      localParams = new Params();
      paramVisualItem.set("_fruchtermanReingoldParams", localParams);
    }
    return localParams;
  }
  
  static
  {
    PARAMS_SCHEMA.addColumn("_fruchtermanReingoldParams", Params.class);
  }
  
  public static class Params
    implements Cloneable
  {
    double[] loc = new double[2];
    double[] disp = new double[2];
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\action\layout\graph\FruchtermanReingoldLayout.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */