package prefuse.controls;

import java.awt.Cursor;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Point2D.Double;
import java.util.Iterator;
import javax.swing.SwingUtilities;
import prefuse.Display;
import prefuse.Visualization;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;

public class SubtreeDragControl
  extends ControlAdapter
{
  private Point2D down = new Point2D.Double();
  private Point2D tmp = new Point2D.Double();
  private boolean wasFixed;
  
  public void itemEntered(VisualItem paramVisualItem, MouseEvent paramMouseEvent)
  {
    if (!(paramVisualItem instanceof NodeItem)) {
      return;
    }
    Display localDisplay = (Display)paramMouseEvent.getSource();
    localDisplay.setCursor(Cursor.getPredefinedCursor(12));
  }
  
  public void itemExited(VisualItem paramVisualItem, MouseEvent paramMouseEvent)
  {
    if (!(paramVisualItem instanceof NodeItem)) {
      return;
    }
    Display localDisplay = (Display)paramMouseEvent.getSource();
    localDisplay.setCursor(Cursor.getDefaultCursor());
  }
  
  public void itemPressed(VisualItem paramVisualItem, MouseEvent paramMouseEvent)
  {
    if (!SwingUtilities.isLeftMouseButton(paramMouseEvent)) {
      return;
    }
    if (!(paramVisualItem instanceof NodeItem)) {
      return;
    }
    Display localDisplay = (Display)paramMouseEvent.getComponent();
    this.down = localDisplay.getAbsoluteCoordinate(paramMouseEvent.getPoint(), this.down);
    this.wasFixed = paramVisualItem.isFixed();
    paramVisualItem.setFixed(true);
  }
  
  public void itemReleased(VisualItem paramVisualItem, MouseEvent paramMouseEvent)
  {
    if (!SwingUtilities.isLeftMouseButton(paramMouseEvent)) {
      return;
    }
    if (!(paramVisualItem instanceof NodeItem)) {
      return;
    }
    paramVisualItem.setFixed(this.wasFixed);
  }
  
  public void itemDragged(VisualItem paramVisualItem, MouseEvent paramMouseEvent)
  {
    if (!SwingUtilities.isLeftMouseButton(paramMouseEvent)) {
      return;
    }
    if (!(paramVisualItem instanceof NodeItem)) {
      return;
    }
    Display localDisplay = (Display)paramMouseEvent.getComponent();
    this.tmp = localDisplay.getAbsoluteCoordinate(paramMouseEvent.getPoint(), this.tmp);
    double d1 = this.tmp.getX() - this.down.getX();
    double d2 = this.tmp.getY() - this.down.getY();
    updateLocations((NodeItem)paramVisualItem, d1, d2);
    this.down.setLocation(this.tmp);
    paramVisualItem.getVisualization().repaint();
  }
  
  private void updateLocations(NodeItem paramNodeItem, double paramDouble1, double paramDouble2)
  {
    double d1 = paramNodeItem.getX();
    double d2 = paramNodeItem.getY();
    paramNodeItem.setStartX(d1);
    paramNodeItem.setStartY(d2);
    d1 += paramDouble1;
    d2 += paramDouble2;
    paramNodeItem.setX(d1);
    paramNodeItem.setY(d2);
    paramNodeItem.setEndX(d1);
    paramNodeItem.setEndY(d2);
    Iterator localIterator = paramNodeItem.children();
    while (localIterator.hasNext()) {
      updateLocations((NodeItem)localIterator.next(), paramDouble1, paramDouble2);
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\controls\SubtreeDragControl.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */