package prefuse.data.tuple;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.logging.Logger;
import prefuse.data.Table;
import prefuse.data.Tuple;
import prefuse.data.event.TupleSetListener;
import prefuse.data.expression.Expression;
import prefuse.data.expression.Predicate;
import prefuse.data.expression.parser.ExpressionParser;
import prefuse.util.collections.CompositeIterator;

public class CompositeTupleSet
  extends AbstractTupleSet
{
  private static final Logger s_logger = Logger.getLogger(CompositeTupleSet.class.getName());
  private Map m_map = new LinkedHashMap();
  private Set m_sets = new HashSet();
  private int m_count = 0;
  private Listener m_lstnr = paramBoolean ? new Listener(null) : null;
  
  public CompositeTupleSet()
  {
    this(true);
  }
  
  protected CompositeTupleSet(boolean paramBoolean) {}
  
  public void addSet(String paramString, TupleSet paramTupleSet)
  {
    if (hasSet(paramString)) {
      throw new IllegalArgumentException("Name already in use: " + paramString);
    }
    this.m_map.put(paramString, paramTupleSet);
    this.m_sets.add(paramTupleSet);
    this.m_count += paramTupleSet.getTupleCount();
    if (this.m_lstnr != null) {
      paramTupleSet.addTupleSetListener(this.m_lstnr);
    }
  }
  
  public boolean hasSet(String paramString)
  {
    return this.m_map.containsKey(paramString);
  }
  
  public boolean containsSet(TupleSet paramTupleSet)
  {
    return this.m_sets.contains(paramTupleSet);
  }
  
  public TupleSet getSet(String paramString)
  {
    return (TupleSet)this.m_map.get(paramString);
  }
  
  public Iterator setNames()
  {
    return this.m_map.keySet().iterator();
  }
  
  public Iterator sets()
  {
    return this.m_map.values().iterator();
  }
  
  public TupleSet removeSet(String paramString)
  {
    TupleSet localTupleSet = (TupleSet)this.m_map.remove(paramString);
    if (localTupleSet != null)
    {
      this.m_sets.remove(localTupleSet);
      if (this.m_lstnr != null) {
        localTupleSet.removeTupleSetListener(this.m_lstnr);
      }
    }
    return localTupleSet;
  }
  
  public void removeAllSets()
  {
    Iterator localIterator = this.m_map.entrySet().iterator();
    while (localIterator.hasNext())
    {
      Map.Entry localEntry = (Map.Entry)localIterator.next();
      TupleSet localTupleSet = (TupleSet)localEntry.getValue();
      localIterator.remove();
      this.m_sets.remove(localTupleSet);
      if (this.m_lstnr != null) {
        localTupleSet.removeTupleSetListener(this.m_lstnr);
      }
    }
    this.m_count = 0;
  }
  
  public void clear()
  {
    Iterator localIterator = this.m_map.entrySet().iterator();
    while (localIterator.hasNext())
    {
      Map.Entry localEntry = (Map.Entry)localIterator.next();
      ((TupleSet)localEntry.getValue()).clear();
    }
    this.m_count = 0;
  }
  
  public Tuple addTuple(Tuple paramTuple)
  {
    throw new UnsupportedOperationException();
  }
  
  public Tuple setTuple(Tuple paramTuple)
  {
    throw new UnsupportedOperationException();
  }
  
  public boolean removeTuple(Tuple paramTuple)
  {
    Table localTable = paramTuple.getTable();
    if (this.m_sets.contains(localTable)) {
      return localTable.removeTuple(paramTuple);
    }
    return false;
  }
  
  public boolean containsTuple(Tuple paramTuple)
  {
    Table localTable = paramTuple.getTable();
    return containsSet(localTable);
  }
  
  public int getTupleCount()
  {
    if (this.m_lstnr != null) {
      return this.m_count;
    }
    int i = 0;
    Iterator localIterator = this.m_map.entrySet().iterator();
    for (int j = 0; localIterator.hasNext(); j++)
    {
      Map.Entry localEntry = (Map.Entry)localIterator.next();
      TupleSet localTupleSet = (TupleSet)localEntry.getValue();
      i += localTupleSet.getTupleCount();
    }
    return i;
  }
  
  public Iterator tuples()
  {
    CompositeIterator localCompositeIterator = new CompositeIterator(this.m_map.size());
    Iterator localIterator = this.m_map.entrySet().iterator();
    for (int i = 0; localIterator.hasNext(); i++)
    {
      Map.Entry localEntry = (Map.Entry)localIterator.next();
      TupleSet localTupleSet = (TupleSet)localEntry.getValue();
      localCompositeIterator.setIterator(i, localTupleSet.tuples());
    }
    return localCompositeIterator;
  }
  
  public Iterator tuples(Predicate paramPredicate)
  {
    CompositeIterator localCompositeIterator = new CompositeIterator(this.m_map.size());
    Iterator localIterator = this.m_map.entrySet().iterator();
    for (int i = 0; localIterator.hasNext(); i++)
    {
      Map.Entry localEntry = (Map.Entry)localIterator.next();
      TupleSet localTupleSet = (TupleSet)localEntry.getValue();
      localCompositeIterator.setIterator(i, localTupleSet.tuples(paramPredicate));
    }
    return localCompositeIterator;
  }
  
  public boolean isAddColumnSupported()
  {
    return true;
  }
  
  public void addColumn(String paramString, Class paramClass, Object paramObject)
  {
    Iterator localIterator = this.m_map.entrySet().iterator();
    while (localIterator.hasNext())
    {
      Map.Entry localEntry = (Map.Entry)localIterator.next();
      TupleSet localTupleSet = (TupleSet)localEntry.getValue();
      if (localTupleSet.isAddColumnSupported()) {
        try
        {
          localTupleSet.addColumn(paramString, paramClass, paramObject);
        }
        catch (IllegalArgumentException localIllegalArgumentException) {}
      } else {
        s_logger.fine("Skipped addColumn for " + localEntry.getKey());
      }
    }
  }
  
  public void addColumn(String paramString, Class paramClass)
  {
    Iterator localIterator = this.m_map.entrySet().iterator();
    while (localIterator.hasNext())
    {
      Map.Entry localEntry = (Map.Entry)localIterator.next();
      TupleSet localTupleSet = (TupleSet)localEntry.getValue();
      if (localTupleSet.isAddColumnSupported()) {
        try
        {
          localTupleSet.addColumn(paramString, paramClass);
        }
        catch (IllegalArgumentException localIllegalArgumentException) {}
      } else {
        s_logger.fine("Skipped addColumn for " + localEntry.getKey());
      }
    }
  }
  
  public void addColumn(String paramString, Expression paramExpression)
  {
    Iterator localIterator = this.m_map.entrySet().iterator();
    while (localIterator.hasNext())
    {
      Map.Entry localEntry = (Map.Entry)localIterator.next();
      TupleSet localTupleSet = (TupleSet)localEntry.getValue();
      if (localTupleSet.isAddColumnSupported()) {
        try
        {
          localTupleSet.addColumn(paramString, paramExpression);
        }
        catch (IllegalArgumentException localIllegalArgumentException) {}
      } else {
        s_logger.fine("Skipped addColumn for " + localEntry.getKey());
      }
    }
  }
  
  public void addColumn(String paramString1, String paramString2)
  {
    Expression localExpression = ExpressionParser.parse(paramString2);
    Throwable localThrowable = ExpressionParser.getError();
    if (localThrowable != null) {
      throw new RuntimeException(localThrowable);
    }
    addColumn(paramString1, localExpression);
  }
  
  private class Listener
    implements TupleSetListener
  {
    private Listener() {}
    
    public void tupleSetChanged(TupleSet paramTupleSet, Tuple[] paramArrayOfTuple1, Tuple[] paramArrayOfTuple2)
    {
      CompositeTupleSet.access$112(CompositeTupleSet.this, paramArrayOfTuple1.length - paramArrayOfTuple2.length);
      CompositeTupleSet.this.fireTupleEvent(paramArrayOfTuple1, paramArrayOfTuple2);
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\tuple\CompositeTupleSet.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */