package prefuse.render;

import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Ellipse2D.Double;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Rectangle2D.Double;
import prefuse.visual.VisualItem;

public class ShapeRenderer
  extends AbstractShapeRenderer
{
  private int m_baseSize = 10;
  private Ellipse2D m_ellipse = new Ellipse2D.Double();
  private Rectangle2D m_rect = new Rectangle2D.Double();
  private GeneralPath m_path = new GeneralPath();
  
  public ShapeRenderer() {}
  
  public ShapeRenderer(int paramInt)
  {
    setBaseSize(paramInt);
  }
  
  public void setBaseSize(int paramInt)
  {
    this.m_baseSize = paramInt;
  }
  
  public int getBaseSize()
  {
    return this.m_baseSize;
  }
  
  protected Shape getRawShape(VisualItem paramVisualItem)
  {
    int i = paramVisualItem.getShape();
    double d1 = paramVisualItem.getX();
    if ((Double.isNaN(d1)) || (Double.isInfinite(d1))) {
      d1 = 0.0D;
    }
    double d2 = paramVisualItem.getY();
    if ((Double.isNaN(d2)) || (Double.isInfinite(d2))) {
      d2 = 0.0D;
    }
    double d3 = this.m_baseSize * paramVisualItem.getSize();
    if (d3 > 1.0D)
    {
      d1 -= d3 / 2.0D;
      d2 -= d3 / 2.0D;
    }
    switch (i)
    {
    case -1: 
      return null;
    case 0: 
      return rectangle(d1, d2, d3, d3);
    case 1: 
      return ellipse(d1, d2, d3, d3);
    case 5: 
      return triangle_up((float)d1, (float)d2, (float)d3);
    case 6: 
      return triangle_down((float)d1, (float)d2, (float)d3);
    case 7: 
      return triangle_left((float)d1, (float)d2, (float)d3);
    case 8: 
      return triangle_right((float)d1, (float)d2, (float)d3);
    case 3: 
      return cross((float)d1, (float)d2, (float)d3);
    case 4: 
      return star((float)d1, (float)d2, (float)d3);
    case 9: 
      return hexagon((float)d1, (float)d2, (float)d3);
    case 2: 
      return diamond((float)d1, (float)d2, (float)d3);
    }
    throw new IllegalStateException("Unknown shape type: " + i);
  }
  
  public Shape rectangle(double paramDouble1, double paramDouble2, double paramDouble3, double paramDouble4)
  {
    this.m_rect.setFrame(paramDouble1, paramDouble2, paramDouble3, paramDouble4);
    return this.m_rect;
  }
  
  public Shape ellipse(double paramDouble1, double paramDouble2, double paramDouble3, double paramDouble4)
  {
    this.m_ellipse.setFrame(paramDouble1, paramDouble2, paramDouble3, paramDouble4);
    return this.m_ellipse;
  }
  
  public Shape triangle_up(float paramFloat1, float paramFloat2, float paramFloat3)
  {
    this.m_path.reset();
    this.m_path.moveTo(paramFloat1, paramFloat2 + paramFloat3);
    this.m_path.lineTo(paramFloat1 + paramFloat3 / 2.0F, paramFloat2);
    this.m_path.lineTo(paramFloat1 + paramFloat3, paramFloat2 + paramFloat3);
    this.m_path.closePath();
    return this.m_path;
  }
  
  public Shape triangle_down(float paramFloat1, float paramFloat2, float paramFloat3)
  {
    this.m_path.reset();
    this.m_path.moveTo(paramFloat1, paramFloat2);
    this.m_path.lineTo(paramFloat1 + paramFloat3, paramFloat2);
    this.m_path.lineTo(paramFloat1 + paramFloat3 / 2.0F, paramFloat2 + paramFloat3);
    this.m_path.closePath();
    return this.m_path;
  }
  
  public Shape triangle_left(float paramFloat1, float paramFloat2, float paramFloat3)
  {
    this.m_path.reset();
    this.m_path.moveTo(paramFloat1 + paramFloat3, paramFloat2);
    this.m_path.lineTo(paramFloat1 + paramFloat3, paramFloat2 + paramFloat3);
    this.m_path.lineTo(paramFloat1, paramFloat2 + paramFloat3 / 2.0F);
    this.m_path.closePath();
    return this.m_path;
  }
  
  public Shape triangle_right(float paramFloat1, float paramFloat2, float paramFloat3)
  {
    this.m_path.reset();
    this.m_path.moveTo(paramFloat1, paramFloat2 + paramFloat3);
    this.m_path.lineTo(paramFloat1 + paramFloat3, paramFloat2 + paramFloat3 / 2.0F);
    this.m_path.lineTo(paramFloat1, paramFloat2);
    this.m_path.closePath();
    return this.m_path;
  }
  
  public Shape cross(float paramFloat1, float paramFloat2, float paramFloat3)
  {
    float f1 = 3.0F * paramFloat3 / 8.0F;
    float f2 = 5.0F * paramFloat3 / 8.0F;
    this.m_path.reset();
    this.m_path.moveTo(paramFloat1 + f1, paramFloat2);
    this.m_path.lineTo(paramFloat1 + f2, paramFloat2);
    this.m_path.lineTo(paramFloat1 + f2, paramFloat2 + f1);
    this.m_path.lineTo(paramFloat1 + paramFloat3, paramFloat2 + f1);
    this.m_path.lineTo(paramFloat1 + paramFloat3, paramFloat2 + f2);
    this.m_path.lineTo(paramFloat1 + f2, paramFloat2 + f2);
    this.m_path.lineTo(paramFloat1 + f2, paramFloat2 + paramFloat3);
    this.m_path.lineTo(paramFloat1 + f1, paramFloat2 + paramFloat3);
    this.m_path.lineTo(paramFloat1 + f1, paramFloat2 + f2);
    this.m_path.lineTo(paramFloat1, paramFloat2 + f2);
    this.m_path.lineTo(paramFloat1, paramFloat2 + f1);
    this.m_path.lineTo(paramFloat1 + f1, paramFloat2 + f1);
    this.m_path.closePath();
    return this.m_path;
  }
  
  public Shape star(float paramFloat1, float paramFloat2, float paramFloat3)
  {
    float f1 = (float)(paramFloat3 / (2.0D * Math.sin(Math.toRadians(54.0D))));
    float f2 = (float)(paramFloat3 / (2.0D * Math.tan(Math.toRadians(54.0D))));
    float f3 = (float)(f1 * Math.sin(Math.toRadians(18.0D)));
    float f4 = (float)(f1 * Math.cos(Math.toRadians(18.0D)));
    float f5 = (float)(f1 / (2.0D * Math.cos(Math.toRadians(36.0D))));
    float f6 = f5 * (float)Math.sin(Math.toRadians(36.0D));
    float f7 = f5 * (float)Math.cos(Math.toRadians(36.0D));
    this.m_path.reset();
    this.m_path.moveTo(paramFloat1, paramFloat2 + f2);
    this.m_path.lineTo(paramFloat1 + f5, paramFloat2 + f2);
    this.m_path.lineTo(paramFloat1 + paramFloat3 / 2.0F, paramFloat2);
    this.m_path.lineTo(paramFloat1 + paramFloat3 - f5, paramFloat2 + f2);
    this.m_path.lineTo(paramFloat1 + paramFloat3, paramFloat2 + f2);
    this.m_path.lineTo(paramFloat1 + paramFloat3 - f7, paramFloat2 + f2 + f6);
    this.m_path.lineTo(paramFloat1 + paramFloat3 - f3, paramFloat2 + paramFloat3);
    this.m_path.lineTo(paramFloat1 + paramFloat3 / 2.0F, paramFloat2 + f2 + f4 - f6);
    this.m_path.lineTo(paramFloat1 + f3, paramFloat2 + paramFloat3);
    this.m_path.lineTo(paramFloat1 + f7, paramFloat2 + f2 + f6);
    this.m_path.closePath();
    return this.m_path;
  }
  
  public Shape hexagon(float paramFloat1, float paramFloat2, float paramFloat3)
  {
    float f = paramFloat3 / 2.0F;
    this.m_path.reset();
    this.m_path.moveTo(paramFloat1, paramFloat2 + 0.5F * paramFloat3);
    this.m_path.lineTo(paramFloat1 + 0.5F * f, paramFloat2);
    this.m_path.lineTo(paramFloat1 + 1.5F * f, paramFloat2);
    this.m_path.lineTo(paramFloat1 + 2.0F * f, paramFloat2 + 0.5F * paramFloat3);
    this.m_path.lineTo(paramFloat1 + 1.5F * f, paramFloat2 + paramFloat3);
    this.m_path.lineTo(paramFloat1 + 0.5F * f, paramFloat2 + paramFloat3);
    this.m_path.closePath();
    return this.m_path;
  }
  
  public Shape diamond(float paramFloat1, float paramFloat2, float paramFloat3)
  {
    this.m_path.reset();
    this.m_path.moveTo(paramFloat1, paramFloat2 + 0.5F * paramFloat3);
    this.m_path.lineTo(paramFloat1 + 0.5F * paramFloat3, paramFloat2);
    this.m_path.lineTo(paramFloat1 + paramFloat3, paramFloat2 + 0.5F * paramFloat3);
    this.m_path.lineTo(paramFloat1 + 0.5F * paramFloat3, paramFloat2 + paramFloat3);
    this.m_path.closePath();
    return this.m_path;
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\render\ShapeRenderer.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */