package prefuse.util.collections;

import java.util.BitSet;
import java.util.Comparator;
import java.util.NoSuchElementException;

public class BooleanIntBitSetMap
  implements BooleanIntSortedMap
{
  private BitSet m_true = new BitSet();
  private BitSet m_false = new BitSet();
  
  public boolean firstKey()
  {
    return false;
  }
  
  public boolean lastKey()
  {
    return true;
  }
  
  public boolean containsKey(boolean paramBoolean)
  {
    BitSet localBitSet = paramBoolean ? this.m_true : this.m_false;
    return localBitSet.cardinality() > 0;
  }
  
  public IntIterator valueRangeIterator(boolean paramBoolean1, boolean paramBoolean2, boolean paramBoolean3, boolean paramBoolean4)
  {
    if ((!paramBoolean2) && (!paramBoolean4)) {
      return new BitSetIterator(null);
    }
    if ((paramBoolean1 == paramBoolean3) || (!paramBoolean4)) {
      return new BitSetIterator(paramBoolean1 ? this.m_true : this.m_false);
    }
    if (!paramBoolean2) {
      return new BitSetIterator(paramBoolean3 ? this.m_true : this.m_false);
    }
    return new BitSetIterator(paramBoolean1 ? this.m_true : this.m_false, paramBoolean3 ? this.m_true : this.m_false);
  }
  
  public LiteralIterator keyIterator()
  {
    return new BitSetIterator(this.m_false, this.m_true);
  }
  
  public LiteralIterator keyRangeIterator(boolean paramBoolean1, boolean paramBoolean2, boolean paramBoolean3, boolean paramBoolean4)
  {
    if ((!paramBoolean2) && (!paramBoolean4)) {
      return new BitSetIterator(null);
    }
    if ((paramBoolean1 == paramBoolean3) || (!paramBoolean4)) {
      return new BitSetIterator(paramBoolean1 ? this.m_true : this.m_false);
    }
    if (!paramBoolean2) {
      return new BitSetIterator(paramBoolean3 ? this.m_true : this.m_false);
    }
    return new BitSetIterator(paramBoolean1 ? this.m_true : this.m_false, paramBoolean3 ? this.m_true : this.m_false);
  }
  
  public int get(boolean paramBoolean)
  {
    BitSet localBitSet = paramBoolean ? this.m_true : this.m_false;
    return localBitSet.nextSetBit(0);
  }
  
  public int remove(boolean paramBoolean)
  {
    BitSet localBitSet = paramBoolean ? this.m_true : this.m_false;
    int i = localBitSet.length() - 1;
    localBitSet.clear(i);
    return i;
  }
  
  public int remove(boolean paramBoolean, int paramInt)
  {
    BitSet localBitSet = paramBoolean ? this.m_true : this.m_false;
    if (localBitSet.get(paramInt))
    {
      localBitSet.clear(paramInt);
      return paramInt;
    }
    return Integer.MIN_VALUE;
  }
  
  public int put(boolean paramBoolean, int paramInt)
  {
    BitSet localBitSet = paramBoolean ? this.m_true : this.m_false;
    boolean bool = localBitSet.get(paramInt);
    localBitSet.set(paramInt);
    return bool ? paramInt : Integer.MIN_VALUE;
  }
  
  public int getMinimum()
  {
    if (this.m_false.cardinality() > 0) {
      return this.m_false.nextSetBit(0);
    }
    if (this.m_true.cardinality() > 0) {
      return this.m_true.nextSetBit(0);
    }
    return Integer.MIN_VALUE;
  }
  
  public int getMaximum()
  {
    int i = this.m_true.length() - 1;
    return i > 0 ? i : this.m_false.length() - 1;
  }
  
  public int getMedian()
  {
    int i = this.m_false.cardinality();
    int j = this.m_true.cardinality();
    if ((i == 0) && (j == 0)) {
      return Integer.MIN_VALUE;
    }
    int k = (i + j) / 2;
    BitSet localBitSet = i > j ? this.m_false : this.m_true;
    int m = localBitSet.nextSetBit(0);
    for (int n = 0; m >= 0; n++)
    {
      if (n == k) {
        return m;
      }
      m = localBitSet.nextSetBit(m + 1);
    }
    return Integer.MIN_VALUE;
  }
  
  public int getUniqueCount()
  {
    int i = 0;
    if (this.m_false.cardinality() > 0) {
      i++;
    }
    if (this.m_true.cardinality() > 0) {
      i++;
    }
    return i;
  }
  
  public boolean isAllowDuplicates()
  {
    return true;
  }
  
  public int size()
  {
    return this.m_true.cardinality() + this.m_false.cardinality();
  }
  
  public boolean isEmpty()
  {
    return (this.m_true.isEmpty()) && (this.m_false.isEmpty());
  }
  
  public Comparator comparator()
  {
    return DefaultLiteralComparator.getInstance();
  }
  
  public void clear()
  {
    this.m_true.clear();
    this.m_false.clear();
  }
  
  public boolean containsValue(int paramInt)
  {
    return (this.m_false.get(paramInt)) || (this.m_true.get(paramInt));
  }
  
  public IntIterator valueIterator(boolean paramBoolean)
  {
    if (!paramBoolean) {
      return new BitSetIterator(this.m_true, this.m_false);
    }
    return new BitSetIterator(this.m_false, this.m_true);
  }
  
  public class BitSetIterator
    extends IntIterator
  {
    private BitSet m_cur;
    private BitSet m_next;
    private int m_val = -1;
    
    public BitSetIterator(BitSet paramBitSet)
    {
      this(paramBitSet, null);
    }
    
    public BitSetIterator(BitSet paramBitSet1, BitSet paramBitSet2)
    {
      this.m_cur = paramBitSet1;
      this.m_next = paramBitSet2;
      if (paramBitSet1 == null)
      {
        this.m_val = -2;
      }
      else
      {
        this.m_val = -1;
        advance();
      }
    }
    
    private void advance()
    {
      int i = this.m_cur.nextSetBit(this.m_val + 1);
      if (i < 0)
      {
        if (this.m_next != null)
        {
          this.m_cur = this.m_next;
          this.m_next = null;
          this.m_val = -1;
          advance();
        }
        else
        {
          this.m_val = -2;
        }
        return;
      }
      this.m_val = i;
    }
    
    public int nextInt()
    {
      if (this.m_val < 0) {
        throw new NoSuchElementException();
      }
      int i = this.m_val;
      advance();
      return i;
    }
    
    public boolean nextBoolean()
    {
      if (this.m_cur == BooleanIntBitSetMap.this.m_true)
      {
        advance();
        return true;
      }
      if (this.m_cur == BooleanIntBitSetMap.this.m_false)
      {
        advance();
        return false;
      }
      throw new NoSuchElementException();
    }
    
    public boolean hasNext()
    {
      return this.m_val >= 0;
    }
    
    public void remove()
    {
      throw new UnsupportedOperationException();
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\collections\BooleanIntBitSetMap.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */