package prefuse.util.collections;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.lang.reflect.Array;
import java.util.AbstractList;
import java.util.Collection;
import java.util.ConcurrentModificationException;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.NoSuchElementException;
import java.util.RandomAccess;

public class CopyOnWriteArrayList
  implements List, RandomAccess, Cloneable, Serializable
{
  private static final long serialVersionUID = 8673264195747942595L;
  private volatile transient Object[] array;
  
  public Object[] getArray()
  {
    return this.array;
  }
  
  void setArray(Object[] paramArrayOfObject)
  {
    this.array = paramArrayOfObject;
  }
  
  public CopyOnWriteArrayList()
  {
    setArray(new Object[0]);
  }
  
  public CopyOnWriteArrayList(Collection paramCollection)
  {
    Object[] arrayOfObject = new Object[paramCollection.size()];
    int i = 0;
    Iterator localIterator = paramCollection.iterator();
    while (localIterator.hasNext())
    {
      Object localObject = localIterator.next();
      arrayOfObject[(i++)] = localObject;
    }
    setArray(arrayOfObject);
  }
  
  public CopyOnWriteArrayList(Object[] paramArrayOfObject)
  {
    copyIn(paramArrayOfObject, 0, paramArrayOfObject.length);
  }
  
  private void copyIn(Object[] paramArrayOfObject, int paramInt1, int paramInt2)
  {
    int i = paramInt1 + paramInt2;
    if (i > paramArrayOfObject.length) {
      throw new IndexOutOfBoundsException();
    }
    Object[] arrayOfObject = copyOfRange(paramArrayOfObject, paramInt1, i, Object[].class);
    synchronized (this)
    {
      setArray(arrayOfObject);
    }
  }
  
  public int size()
  {
    return getArray().length;
  }
  
  public boolean isEmpty()
  {
    return size() == 0;
  }
  
  private static boolean eq(Object paramObject1, Object paramObject2)
  {
    return paramObject1 == null ? false : paramObject2 == null ? true : paramObject1.equals(paramObject2);
  }
  
  private static int indexOf(Object paramObject, Object[] paramArrayOfObject, int paramInt1, int paramInt2)
  {
    int i;
    if (paramObject == null) {
      for (i = paramInt1; i < paramInt2; i++) {
        if (paramArrayOfObject[i] == null) {
          return i;
        }
      }
    } else {
      for (i = paramInt1; i < paramInt2; i++) {
        if (paramObject.equals(paramArrayOfObject[i])) {
          return i;
        }
      }
    }
    return -1;
  }
  
  private static int lastIndexOf(Object paramObject, Object[] paramArrayOfObject, int paramInt)
  {
    int i;
    if (paramObject == null) {
      for (i = paramInt; i >= 0; i--) {
        if (paramArrayOfObject[i] == null) {
          return i;
        }
      }
    } else {
      for (i = paramInt; i >= 0; i--) {
        if (paramObject.equals(paramArrayOfObject[i])) {
          return i;
        }
      }
    }
    return -1;
  }
  
  public boolean contains(Object paramObject)
  {
    Object[] arrayOfObject = getArray();
    return indexOf(paramObject, arrayOfObject, 0, arrayOfObject.length) >= 0;
  }
  
  public int indexOf(Object paramObject)
  {
    Object[] arrayOfObject = getArray();
    return indexOf(paramObject, arrayOfObject, 0, arrayOfObject.length);
  }
  
  public int indexOf(Object paramObject, int paramInt)
  {
    Object[] arrayOfObject = getArray();
    return indexOf(paramObject, arrayOfObject, paramInt, arrayOfObject.length);
  }
  
  public int lastIndexOf(Object paramObject)
  {
    Object[] arrayOfObject = getArray();
    return lastIndexOf(paramObject, arrayOfObject, arrayOfObject.length - 1);
  }
  
  public int lastIndexOf(Object paramObject, int paramInt)
  {
    Object[] arrayOfObject = getArray();
    return lastIndexOf(paramObject, arrayOfObject, paramInt);
  }
  
  public Object clone()
  {
    try
    {
      return super.clone();
    }
    catch (CloneNotSupportedException localCloneNotSupportedException)
    {
      throw new InternalError();
    }
  }
  
  public Object[] toArray()
  {
    Object[] arrayOfObject = getArray();
    return copyOf(arrayOfObject, arrayOfObject.length);
  }
  
  public Object[] toArray(Object[] paramArrayOfObject)
  {
    Object[] arrayOfObject = getArray();
    int i = arrayOfObject.length;
    if (paramArrayOfObject.length < i) {
      return copyOf(arrayOfObject, i, paramArrayOfObject.getClass());
    }
    System.arraycopy(arrayOfObject, 0, paramArrayOfObject, 0, i);
    if (paramArrayOfObject.length > i) {
      paramArrayOfObject[i] = null;
    }
    return paramArrayOfObject;
  }
  
  public Object get(int paramInt)
  {
    return getArray()[paramInt];
  }
  
  public synchronized Object set(int paramInt, Object paramObject)
  {
    Object[] arrayOfObject1 = getArray();
    int i = arrayOfObject1.length;
    Object localObject = arrayOfObject1[paramInt];
    if (localObject != paramObject)
    {
      Object[] arrayOfObject2 = copyOf(arrayOfObject1, i);
      arrayOfObject2[paramInt] = paramObject;
      setArray(arrayOfObject2);
    }
    return localObject;
  }
  
  public boolean add(Object paramObject)
  {
    synchronized (this)
    {
      Object[] arrayOfObject1 = getArray();
      int i = arrayOfObject1.length;
      Object[] arrayOfObject2 = copyOf(arrayOfObject1, i + 1);
      arrayOfObject2[i] = paramObject;
      setArray(arrayOfObject2);
    }
    return true;
  }
  
  public synchronized void add(int paramInt, Object paramObject)
  {
    Object[] arrayOfObject1 = getArray();
    int i = arrayOfObject1.length;
    if ((paramInt > i) || (paramInt < 0)) {
      throw new IndexOutOfBoundsException("Index: " + paramInt + ", Size: " + i);
    }
    int j = i - paramInt;
    Object[] arrayOfObject2;
    if (j == 0)
    {
      arrayOfObject2 = copyOf(arrayOfObject1, i + 1);
    }
    else
    {
      arrayOfObject2 = new Object[i + 1];
      System.arraycopy(arrayOfObject1, 0, arrayOfObject2, 0, paramInt);
      System.arraycopy(arrayOfObject1, paramInt, arrayOfObject2, paramInt + 1, j);
    }
    arrayOfObject2[paramInt] = paramObject;
    setArray(arrayOfObject2);
  }
  
  public synchronized Object remove(int paramInt)
  {
    Object[] arrayOfObject1 = getArray();
    int i = arrayOfObject1.length;
    Object localObject = arrayOfObject1[paramInt];
    int j = i - paramInt - 1;
    if (j == 0)
    {
      setArray(copyOf(arrayOfObject1, i - 1));
    }
    else
    {
      Object[] arrayOfObject2 = new Object[i - 1];
      System.arraycopy(arrayOfObject1, 0, arrayOfObject2, 0, paramInt);
      System.arraycopy(arrayOfObject1, paramInt + 1, arrayOfObject2, paramInt, j);
      setArray(arrayOfObject2);
    }
    return localObject;
  }
  
  public synchronized boolean remove(Object paramObject)
  {
    Object[] arrayOfObject1 = getArray();
    int i = arrayOfObject1.length;
    if (i != 0)
    {
      int j = i - 1;
      Object[] arrayOfObject2 = new Object[j];
      for (int k = 0; k < j; k++)
      {
        if (eq(paramObject, arrayOfObject1[k]))
        {
          for (int m = k + 1; m < i; m++) {
            arrayOfObject2[(m - 1)] = arrayOfObject1[m];
          }
          setArray(arrayOfObject2);
          return true;
        }
        arrayOfObject2[k] = arrayOfObject1[k];
      }
      if (eq(paramObject, arrayOfObject1[j]))
      {
        setArray(arrayOfObject2);
        return true;
      }
    }
    return false;
  }
  
  private synchronized void removeRange(int paramInt1, int paramInt2)
  {
    Object[] arrayOfObject1 = getArray();
    int i = arrayOfObject1.length;
    if ((paramInt1 < 0) || (paramInt1 >= i) || (paramInt2 > i) || (paramInt2 < paramInt1)) {
      throw new IndexOutOfBoundsException();
    }
    int j = i - (paramInt2 - paramInt1);
    int k = i - paramInt2;
    if (k == 0)
    {
      setArray(copyOf(arrayOfObject1, j));
    }
    else
    {
      Object[] arrayOfObject2 = new Object[j];
      System.arraycopy(arrayOfObject1, 0, arrayOfObject2, 0, paramInt1);
      System.arraycopy(arrayOfObject1, paramInt2, arrayOfObject2, paramInt1, k);
      setArray(arrayOfObject2);
    }
  }
  
  public synchronized boolean addIfAbsent(Object paramObject)
  {
    Object[] arrayOfObject1 = getArray();
    int i = arrayOfObject1.length;
    Object[] arrayOfObject2 = new Object[i + 1];
    for (int j = 0; j < i; j++)
    {
      if (eq(paramObject, arrayOfObject1[j])) {
        return false;
      }
      arrayOfObject2[j] = arrayOfObject1[j];
    }
    arrayOfObject2[i] = paramObject;
    setArray(arrayOfObject2);
    return true;
  }
  
  public boolean containsAll(Collection paramCollection)
  {
    Object[] arrayOfObject = getArray();
    int i = arrayOfObject.length;
    Iterator localIterator = paramCollection.iterator();
    while (localIterator.hasNext())
    {
      Object localObject = localIterator.next();
      if (indexOf(localObject, arrayOfObject, 0, i) < 0) {
        return false;
      }
    }
    return true;
  }
  
  public synchronized boolean removeAll(Collection paramCollection)
  {
    Object[] arrayOfObject1 = getArray();
    int i = arrayOfObject1.length;
    if (i != 0)
    {
      int j = 0;
      Object[] arrayOfObject2 = new Object[i];
      for (int k = 0; k < i; k++)
      {
        Object localObject = arrayOfObject1[k];
        if (!paramCollection.contains(localObject)) {
          arrayOfObject2[(j++)] = localObject;
        }
      }
      if (j != i)
      {
        setArray(copyOfRange(arrayOfObject2, 0, j, Object[].class));
        return true;
      }
    }
    return false;
  }
  
  public synchronized boolean retainAll(Collection paramCollection)
  {
    Object[] arrayOfObject1 = getArray();
    int i = arrayOfObject1.length;
    if (i != 0)
    {
      int j = 0;
      Object[] arrayOfObject2 = new Object[i];
      for (int k = 0; k < i; k++)
      {
        Object localObject = arrayOfObject1[k];
        if (paramCollection.contains(localObject)) {
          arrayOfObject2[(j++)] = localObject;
        }
      }
      if (j != i)
      {
        setArray(copyOfRange(arrayOfObject2, 0, j, Object[].class));
        return true;
      }
    }
    return false;
  }
  
  public int addAllAbsent(Collection paramCollection)
  {
    int i = paramCollection.size();
    if (i == 0) {
      return 0;
    }
    synchronized (this)
    {
      Object[] arrayOfObject1 = getArray();
      int j = arrayOfObject1.length;
      Object[] arrayOfObject2 = new Object[i];
      int k = 0;
      Object localObject1 = paramCollection.iterator();
      while (((Iterator)localObject1).hasNext())
      {
        Object localObject2 = ((Iterator)localObject1).next();
        if ((indexOf(localObject2, arrayOfObject1, 0, j) < 0) && (indexOf(localObject2, arrayOfObject2, 0, k) < 0)) {
          arrayOfObject2[(k++)] = localObject2;
        }
      }
      if (k != 0)
      {
        localObject1 = new Object[j + k];
        System.arraycopy(arrayOfObject1, 0, localObject1, 0, j);
        System.arraycopy(arrayOfObject2, 0, localObject1, j, k);
        setArray((Object[])localObject1);
      }
      return k;
    }
  }
  
  public synchronized void clear()
  {
    setArray(new Object[0]);
  }
  
  public boolean addAll(Collection paramCollection)
  {
    int i = paramCollection.size();
    if (i == 0) {
      return false;
    }
    synchronized (this)
    {
      Object[] arrayOfObject1 = getArray();
      int j = arrayOfObject1.length;
      Object[] arrayOfObject2 = new Object[j + i];
      System.arraycopy(arrayOfObject1, 0, arrayOfObject2, 0, j);
      Iterator localIterator = paramCollection.iterator();
      while (localIterator.hasNext())
      {
        Object localObject1 = localIterator.next();
        arrayOfObject2[(j++)] = localObject1;
      }
      setArray(arrayOfObject2);
      return true;
    }
  }
  
  public boolean addAll(int paramInt, Collection paramCollection)
  {
    int i = paramCollection.size();
    synchronized (this)
    {
      Object[] arrayOfObject1 = getArray();
      int j = arrayOfObject1.length;
      if ((paramInt > j) || (paramInt < 0)) {
        throw new IndexOutOfBoundsException("Index: " + paramInt + ", Size: " + j);
      }
      if (i == 0) {
        return false;
      }
      int k = j - paramInt;
      Object[] arrayOfObject2;
      if (k == 0)
      {
        arrayOfObject2 = copyOf(arrayOfObject1, j + i);
      }
      else
      {
        arrayOfObject2 = new Object[j + i];
        System.arraycopy(arrayOfObject1, 0, arrayOfObject2, 0, paramInt);
        System.arraycopy(arrayOfObject1, paramInt, arrayOfObject2, paramInt + i, k);
      }
      Iterator localIterator = paramCollection.iterator();
      while (localIterator.hasNext())
      {
        Object localObject1 = localIterator.next();
        arrayOfObject2[(paramInt++)] = localObject1;
      }
      setArray(arrayOfObject2);
      return true;
    }
  }
  
  private void writeObject(ObjectOutputStream paramObjectOutputStream)
    throws IOException
  {
    paramObjectOutputStream.defaultWriteObject();
    Object[] arrayOfObject = getArray();
    int i = arrayOfObject.length;
    paramObjectOutputStream.writeInt(i);
    for (int j = 0; j < i; j++) {
      paramObjectOutputStream.writeObject(arrayOfObject[j]);
    }
  }
  
  private void readObject(ObjectInputStream paramObjectInputStream)
    throws IOException, ClassNotFoundException
  {
    paramObjectInputStream.defaultReadObject();
    int i = paramObjectInputStream.readInt();
    Object[] arrayOfObject = new Object[i];
    for (int j = 0; j < i; j++) {
      arrayOfObject[j] = paramObjectInputStream.readObject();
    }
    setArray(arrayOfObject);
  }
  
  public String toString()
  {
    Object[] arrayOfObject = getArray();
    int i = arrayOfObject.length - 1;
    StringBuffer localStringBuffer = new StringBuffer();
    localStringBuffer.append("[");
    for (int j = 0; j <= i; j++)
    {
      localStringBuffer.append(String.valueOf(arrayOfObject[j]));
      if (j < i) {
        localStringBuffer.append(", ");
      }
    }
    localStringBuffer.append("]");
    return localStringBuffer.toString();
  }
  
  public boolean equals(Object paramObject)
  {
    if (paramObject == this) {
      return true;
    }
    if (!(paramObject instanceof List)) {
      return false;
    }
    List localList = (List)paramObject;
    if (size() != localList.size()) {
      return false;
    }
    ListIterator localListIterator1 = listIterator();
    ListIterator localListIterator2 = localList.listIterator();
    while (localListIterator1.hasNext()) {
      if (!eq(localListIterator1.next(), localListIterator2.next())) {
        return false;
      }
    }
    return true;
  }
  
  public int hashCode()
  {
    int i = 1;
    for (Object localObject : getArray()) {
      i = 31 * i + (localObject == null ? 0 : localObject.hashCode());
    }
    return i;
  }
  
  public Iterator iterator()
  {
    return new COWIterator(getArray(), 0, null);
  }
  
  public ListIterator listIterator()
  {
    return new COWIterator(getArray(), 0, null);
  }
  
  public ListIterator listIterator(int paramInt)
  {
    Object[] arrayOfObject = getArray();
    int i = arrayOfObject.length;
    if ((paramInt < 0) || (paramInt > i)) {
      throw new IndexOutOfBoundsException("Index: " + paramInt);
    }
    return new COWIterator(getArray(), paramInt, null);
  }
  
  public synchronized List subList(int paramInt1, int paramInt2)
  {
    Object[] arrayOfObject = getArray();
    int i = arrayOfObject.length;
    if ((paramInt1 < 0) || (paramInt2 > i) || (paramInt1 > paramInt2)) {
      throw new IndexOutOfBoundsException();
    }
    return new COWSubList(this, paramInt1, paramInt2, null);
  }
  
  private static Object[] copyOfRange(Object[] paramArrayOfObject, int paramInt1, int paramInt2, Class paramClass)
  {
    int i = paramInt2 - paramInt1;
    if (i < 0) {
      throw new IllegalArgumentException(paramInt1 + " > " + paramInt2);
    }
    Object[] arrayOfObject = (Object[])Array.newInstance(paramClass.getComponentType(), i);
    System.arraycopy(paramArrayOfObject, paramInt1, arrayOfObject, 0, Math.min(paramArrayOfObject.length - paramInt1, i));
    return arrayOfObject;
  }
  
  private static Object[] copyOf(Object[] paramArrayOfObject, int paramInt, Class paramClass)
  {
    Object[] arrayOfObject = (Object[])Array.newInstance(paramClass.getComponentType(), paramInt);
    System.arraycopy(paramArrayOfObject, 0, arrayOfObject, 0, Math.min(paramArrayOfObject.length, paramInt));
    return arrayOfObject;
  }
  
  private static Object[] copyOf(Object[] paramArrayOfObject, int paramInt)
  {
    return copyOf(paramArrayOfObject, paramInt, paramArrayOfObject.getClass());
  }
  
  private static class COWSubListIterator
    implements ListIterator
  {
    private final ListIterator i;
    private final int offset;
    private final int size;
    
    private COWSubListIterator(List paramList, int paramInt1, int paramInt2, int paramInt3)
    {
      this.offset = paramInt2;
      this.size = paramInt3;
      this.i = paramList.listIterator(paramInt1 + paramInt2);
    }
    
    public boolean hasNext()
    {
      return nextIndex() < this.size;
    }
    
    public Object next()
    {
      if (hasNext()) {
        return this.i.next();
      }
      throw new NoSuchElementException();
    }
    
    public boolean hasPrevious()
    {
      return previousIndex() >= 0;
    }
    
    public Object previous()
    {
      if (hasPrevious()) {
        return this.i.previous();
      }
      throw new NoSuchElementException();
    }
    
    public int nextIndex()
    {
      return this.i.nextIndex() - this.offset;
    }
    
    public int previousIndex()
    {
      return this.i.previousIndex() - this.offset;
    }
    
    public void remove()
    {
      throw new UnsupportedOperationException();
    }
    
    public void set(Object paramObject)
    {
      throw new UnsupportedOperationException();
    }
    
    public void add(Object paramObject)
    {
      throw new UnsupportedOperationException();
    }
  }
  
  private static class COWSubList
    extends AbstractList
  {
    private final CopyOnWriteArrayList l;
    private final int offset;
    private int size;
    private Object[] expectedArray;
    
    private COWSubList(CopyOnWriteArrayList paramCopyOnWriteArrayList, int paramInt1, int paramInt2)
    {
      this.l = paramCopyOnWriteArrayList;
      this.expectedArray = this.l.getArray();
      this.offset = paramInt1;
      this.size = (paramInt2 - paramInt1);
    }
    
    private void checkForComodification()
    {
      if (this.l.getArray() != this.expectedArray) {
        throw new ConcurrentModificationException();
      }
    }
    
    private void rangeCheck(int paramInt)
    {
      if ((paramInt < 0) || (paramInt >= this.size)) {
        throw new IndexOutOfBoundsException("Index: " + paramInt + ",Size: " + this.size);
      }
    }
    
    public Object set(int paramInt, Object paramObject)
    {
      synchronized (this.l)
      {
        rangeCheck(paramInt);
        checkForComodification();
        Object localObject1 = this.l.set(paramInt + this.offset, paramObject);
        this.expectedArray = this.l.getArray();
        return localObject1;
      }
    }
    
    public Object get(int paramInt)
    {
      synchronized (this.l)
      {
        rangeCheck(paramInt);
        checkForComodification();
        return this.l.get(paramInt + this.offset);
      }
    }
    
    public int size()
    {
      synchronized (this.l)
      {
        checkForComodification();
        return this.size;
      }
    }
    
    public void add(int paramInt, Object paramObject)
    {
      synchronized (this.l)
      {
        checkForComodification();
        if ((paramInt < 0) || (paramInt > this.size)) {
          throw new IndexOutOfBoundsException();
        }
        this.l.add(paramInt + this.offset, paramObject);
        this.expectedArray = this.l.getArray();
        this.size += 1;
      }
    }
    
    public void clear()
    {
      synchronized (this.l)
      {
        checkForComodification();
        this.l.removeRange(this.offset, this.offset + this.size);
        this.expectedArray = this.l.getArray();
        this.size = 0;
      }
    }
    
    public Object remove(int paramInt)
    {
      synchronized (this.l)
      {
        rangeCheck(paramInt);
        checkForComodification();
        Object localObject1 = this.l.remove(paramInt + this.offset);
        this.expectedArray = this.l.getArray();
        this.size -= 1;
        return localObject1;
      }
    }
    
    public Iterator iterator()
    {
      synchronized (this.l)
      {
        checkForComodification();
        return new CopyOnWriteArrayList.COWSubListIterator(this.l, 0, this.offset, this.size, null);
      }
    }
    
    public ListIterator listIterator(int paramInt)
    {
      synchronized (this.l)
      {
        checkForComodification();
        if ((paramInt < 0) || (paramInt > this.size)) {
          throw new IndexOutOfBoundsException("Index: " + paramInt + ", Size: " + this.size);
        }
        return new CopyOnWriteArrayList.COWSubListIterator(this.l, paramInt, this.offset, this.size, null);
      }
    }
    
    public List subList(int paramInt1, int paramInt2)
    {
      synchronized (this.l)
      {
        checkForComodification();
        if ((paramInt1 < 0) || (paramInt2 > this.size)) {
          throw new IndexOutOfBoundsException();
        }
        return new COWSubList(this.l, paramInt1 + this.offset, paramInt2 + this.offset);
      }
    }
  }
  
  private static class COWIterator
    implements ListIterator
  {
    private final Object[] snapshot;
    private int cursor;
    
    private COWIterator(Object[] paramArrayOfObject, int paramInt)
    {
      this.cursor = paramInt;
      this.snapshot = paramArrayOfObject;
    }
    
    public boolean hasNext()
    {
      return this.cursor < this.snapshot.length;
    }
    
    public boolean hasPrevious()
    {
      return this.cursor > 0;
    }
    
    public Object next()
    {
      try
      {
        return this.snapshot[(this.cursor++)];
      }
      catch (IndexOutOfBoundsException localIndexOutOfBoundsException)
      {
        throw new NoSuchElementException();
      }
    }
    
    public Object previous()
    {
      try
      {
        return this.snapshot[(--this.cursor)];
      }
      catch (IndexOutOfBoundsException localIndexOutOfBoundsException)
      {
        throw new NoSuchElementException();
      }
    }
    
    public int nextIndex()
    {
      return this.cursor;
    }
    
    public int previousIndex()
    {
      return this.cursor - 1;
    }
    
    public void remove()
    {
      throw new UnsupportedOperationException();
    }
    
    public void set(Object paramObject)
    {
      throw new UnsupportedOperationException();
    }
    
    public void add(Object paramObject)
    {
      throw new UnsupportedOperationException();
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\collections\CopyOnWriteArrayList.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */