package prefuse.util.ui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.Document;
import prefuse.Visualization;
import prefuse.data.Tuple;
import prefuse.data.event.TupleSetListener;
import prefuse.data.search.PrefixSearchTupleSet;
import prefuse.data.search.SearchTupleSet;
import prefuse.data.tuple.TupleSet;
import prefuse.util.ColorLib;

public class JSearchPanel
  extends JPanel
  implements DocumentListener, ActionListener
{
  private Object m_lock;
  private SearchTupleSet m_searcher;
  private JTextField m_queryF = new JTextField(15);
  private JLabel m_resultL = new JLabel("          ");
  private JLabel m_searchL = new JLabel("search >> ");
  private Box m_sbox = new Box(0);
  private String[] m_fields;
  private boolean m_includeHitCount = false;
  private boolean m_monitorKeys = false;
  private boolean m_autoIndex = true;
  private boolean m_showBorder = true;
  private boolean m_showCancel = true;
  
  public JSearchPanel(SearchTupleSet paramSearchTupleSet, String paramString)
  {
    this(paramSearchTupleSet, paramString, false);
  }
  
  public JSearchPanel(SearchTupleSet paramSearchTupleSet, String paramString, boolean paramBoolean)
  {
    this(null, paramSearchTupleSet, new String[] { paramString }, false, paramBoolean);
  }
  
  public JSearchPanel(TupleSet paramTupleSet, SearchTupleSet paramSearchTupleSet, String[] paramArrayOfString, boolean paramBoolean1, boolean paramBoolean2)
  {
    this.m_lock = new Object();
    this.m_fields = paramArrayOfString;
    this.m_autoIndex = paramBoolean1;
    this.m_monitorKeys = paramBoolean2;
    this.m_searcher = (paramSearchTupleSet != null ? paramSearchTupleSet : new PrefixSearchTupleSet());
    init(paramTupleSet);
  }
  
  public JSearchPanel(Visualization paramVisualization, String paramString)
  {
    this(paramVisualization, Visualization.ALL_ITEMS, paramString, true);
  }
  
  public JSearchPanel(Visualization paramVisualization, String paramString1, String paramString2)
  {
    this(paramVisualization, paramString1, paramString2, true);
  }
  
  public JSearchPanel(Visualization paramVisualization, String paramString1, String paramString2, boolean paramBoolean)
  {
    this(paramVisualization, paramString1, Visualization.SEARCH_ITEMS, new String[] { paramString2 }, paramBoolean, false);
  }
  
  public JSearchPanel(Visualization paramVisualization, String paramString1, String paramString2, boolean paramBoolean1, boolean paramBoolean2)
  {
    this(paramVisualization, paramString1, Visualization.SEARCH_ITEMS, new String[] { paramString2 }, paramBoolean1, true);
  }
  
  public JSearchPanel(Visualization paramVisualization, String paramString1, String paramString2, String paramString3, boolean paramBoolean1, boolean paramBoolean2)
  {
    this(paramVisualization, paramString1, paramString2, new String[] { paramString3 }, paramBoolean1, paramBoolean2);
  }
  
  public JSearchPanel(Visualization paramVisualization, String paramString1, String paramString2, String[] paramArrayOfString, boolean paramBoolean1, boolean paramBoolean2)
  {
    this.m_lock = paramVisualization;
    this.m_fields = paramArrayOfString;
    this.m_autoIndex = paramBoolean1;
    this.m_monitorKeys = paramBoolean2;
    TupleSet localTupleSet = paramVisualization.getGroup(paramString2);
    if (localTupleSet != null)
    {
      if ((localTupleSet instanceof SearchTupleSet)) {
        this.m_searcher = ((SearchTupleSet)localTupleSet);
      } else {
        throw new IllegalStateException("Search focus set not instance of SearchTupleSet!");
      }
    }
    else
    {
      this.m_searcher = new PrefixSearchTupleSet();
      paramVisualization.addFocusGroup(paramString2, this.m_searcher);
    }
    init(paramVisualization.getGroup(paramString1));
  }
  
  private void init(TupleSet paramTupleSet)
  {
    if ((this.m_autoIndex) && (paramTupleSet != null))
    {
      for (int i = 0; i < this.m_fields.length; i++) {
        this.m_searcher.index(paramTupleSet.tuples(), this.m_fields[i]);
      }
      paramTupleSet.addTupleSetListener(new TupleSetListener()
      {
        public void tupleSetChanged(TupleSet paramAnonymousTupleSet, Tuple[] paramAnonymousArrayOfTuple1, Tuple[] paramAnonymousArrayOfTuple2)
        {
          int i;
          int j;
          if (paramAnonymousArrayOfTuple1 != null) {
            for (i = 0; i < paramAnonymousArrayOfTuple1.length; i++) {
              for (j = 0; j < JSearchPanel.this.m_fields.length; j++) {
                JSearchPanel.this.m_searcher.index(paramAnonymousArrayOfTuple1[i], JSearchPanel.this.m_fields[j]);
              }
            }
          }
          if ((paramAnonymousArrayOfTuple2 != null) && (JSearchPanel.this.m_searcher.isUnindexSupported())) {
            for (i = 0; i < paramAnonymousArrayOfTuple2.length; i++) {
              for (j = 0; j < JSearchPanel.this.m_fields.length; j++) {
                JSearchPanel.this.m_searcher.unindex(paramAnonymousArrayOfTuple2[i], JSearchPanel.this.m_fields[j]);
              }
            }
          }
        }
      });
    }
    this.m_queryF.addActionListener(this);
    if (this.m_monitorKeys) {
      this.m_queryF.getDocument().addDocumentListener(this);
    }
    this.m_queryF.setMaximumSize(new Dimension(400, 100));
    this.m_queryF.setPreferredSize(new Dimension(200, 20));
    this.m_queryF.setBorder(null);
    setBackground(Color.WHITE);
    initUI();
  }
  
  private void initUI()
  {
    removeAll();
    setLayout(new BoxLayout(this, 0));
    this.m_sbox.removeAll();
    this.m_sbox.add(Box.createHorizontalStrut(3));
    this.m_sbox.add(this.m_queryF);
    this.m_sbox.add(Box.createHorizontalStrut(3));
    if (this.m_showCancel)
    {
      this.m_sbox.add(new CancelButton());
      this.m_sbox.add(Box.createHorizontalStrut(3));
    }
    if (this.m_showBorder) {
      this.m_sbox.setBorder(BorderFactory.createLineBorder(getForeground()));
    } else {
      this.m_sbox.setBorder(null);
    }
    this.m_sbox.setMaximumSize(new Dimension(400, 100));
    this.m_sbox.setPreferredSize(new Dimension(171, 20));
    Box localBox = new Box(0);
    if (this.m_includeHitCount)
    {
      localBox.add(this.m_resultL);
      localBox.add(Box.createHorizontalStrut(10));
    }
    localBox.add(this.m_searchL);
    localBox.add(Box.createHorizontalStrut(3));
    localBox.add(this.m_sbox);
    add(localBox);
  }
  
  public void requestFocus()
  {
    this.m_queryF.requestFocus();
  }
  
  public void setLock(Object paramObject)
  {
    this.m_lock = paramObject;
  }
  
  public void setShowResultCount(boolean paramBoolean)
  {
    this.m_includeHitCount = paramBoolean;
    initUI();
    validate();
  }
  
  public void setShowBorder(boolean paramBoolean)
  {
    this.m_showBorder = paramBoolean;
    initUI();
    validate();
  }
  
  public void setShowCancel(boolean paramBoolean)
  {
    this.m_showCancel = paramBoolean;
    initUI();
    validate();
  }
  
  protected void searchUpdate()
  {
    String str = this.m_queryF.getText();
    synchronized (this.m_lock)
    {
      this.m_searcher.search(str);
      if (this.m_searcher.getQuery().length() == 0)
      {
        this.m_resultL.setText(null);
      }
      else
      {
        int i = this.m_searcher.getTupleCount();
        this.m_resultL.setText(i + " match" + (i == 1 ? "" : "es"));
      }
    }
  }
  
  public void setQuery(String paramString)
  {
    Document localDocument = this.m_queryF.getDocument();
    localDocument.removeDocumentListener(this);
    this.m_queryF.setText(paramString);
    localDocument.addDocumentListener(this);
    searchUpdate();
  }
  
  public void setBackground(Color paramColor)
  {
    super.setBackground(paramColor);
    if (this.m_queryF != null) {
      this.m_queryF.setBackground(paramColor);
    }
    if (this.m_resultL != null) {
      this.m_resultL.setBackground(paramColor);
    }
    if (this.m_searchL != null) {
      this.m_searchL.setBackground(paramColor);
    }
  }
  
  public void setForeground(Color paramColor)
  {
    super.setForeground(paramColor);
    if (this.m_queryF != null)
    {
      this.m_queryF.setForeground(paramColor);
      this.m_queryF.setCaretColor(paramColor);
    }
    if (this.m_resultL != null) {
      this.m_resultL.setForeground(paramColor);
    }
    if (this.m_searchL != null) {
      this.m_searchL.setForeground(paramColor);
    }
    if ((this.m_sbox != null) && (this.m_showBorder)) {
      this.m_sbox.setBorder(BorderFactory.createLineBorder(paramColor));
    }
  }
  
  public void setFont(Font paramFont)
  {
    super.setFont(paramFont);
    if (this.m_queryF != null) {
      this.m_queryF.setFont(paramFont);
    }
    if (this.m_resultL != null) {
      this.m_resultL.setFont(paramFont);
    }
    if (this.m_searchL != null) {
      this.m_searchL.setFont(paramFont);
    }
  }
  
  public void setLabelText(String paramString)
  {
    this.m_searchL.setText(paramString);
  }
  
  public void changedUpdate(DocumentEvent paramDocumentEvent)
  {
    searchUpdate();
  }
  
  public void insertUpdate(DocumentEvent paramDocumentEvent)
  {
    searchUpdate();
  }
  
  public void removeUpdate(DocumentEvent paramDocumentEvent)
  {
    searchUpdate();
  }
  
  public void actionPerformed(ActionEvent paramActionEvent)
  {
    Object localObject = paramActionEvent.getSource();
    if (localObject == this.m_queryF) {
      searchUpdate();
    }
  }
  
  public class CancelButton
    extends JComponent
    implements MouseListener
  {
    private boolean hover = false;
    private int[] outline = { 0, 0, 2, 0, 4, 2, 5, 2, 7, 0, 9, 0, 9, 2, 7, 4, 7, 5, 9, 7, 9, 9, 7, 9, 5, 7, 4, 7, 2, 9, 0, 9, 0, 7, 2, 5, 2, 4, 0, 2, 0, 0 };
    private int[] fill = { 1, 1, 8, 8, 1, 2, 7, 8, 2, 1, 8, 7, 7, 1, 1, 7, 8, 2, 2, 8, 1, 8, 8, 1 };
    
    public CancelButton()
    {
      Dimension localDimension = new Dimension(10, 10);
      setPreferredSize(localDimension);
      setMinimumSize(localDimension);
      setMaximumSize(localDimension);
      setFocusable(false);
      addMouseListener(this);
    }
    
    public void paintComponent(Graphics paramGraphics)
    {
      if (this.hover)
      {
        paramGraphics.setColor(ColorLib.getColor(255, 75, 75));
        for (i = 0; i + 3 < this.fill.length; i += 4) {
          paramGraphics.drawLine(this.fill[i], this.fill[(i + 1)], this.fill[(i + 2)], this.fill[(i + 3)]);
        }
      }
      paramGraphics.setColor(JSearchPanel.this.getForeground());
      for (int i = 0; i + 3 < this.outline.length; i += 2) {
        paramGraphics.drawLine(this.outline[i], this.outline[(i + 1)], this.outline[(i + 2)], this.outline[(i + 3)]);
      }
    }
    
    public void mouseClicked(MouseEvent paramMouseEvent)
    {
      JSearchPanel.this.setQuery(null);
    }
    
    public void mousePressed(MouseEvent paramMouseEvent) {}
    
    public void mouseReleased(MouseEvent paramMouseEvent) {}
    
    public void mouseEntered(MouseEvent paramMouseEvent)
    {
      this.hover = true;
      repaint();
    }
    
    public void mouseExited(MouseEvent paramMouseEvent)
    {
      this.hover = false;
      repaint();
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\ui\JSearchPanel.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */